import json
from tqdm import tqdm
from functools import cmp_to_key
import math

from utils.utils import eval_ex_match

solvable_val = ["nt-2",
                "nt-8",
                "nt-9",
                "nt-24",
                "nt-34",
                "nt-53",
                "nt-64",
                "nt-67",
                "nt-68",
                "nt-84",
                "nt-87",
                "nt-92",
                "nt-107",
                "nt-111",
                "nt-116",
                "nt-132",
                "nt-148",
                "nt-159",
                "nt-160",
                "nt-169",
                "nt-170",
                "nt-219",
                "nt-227",
                "nt-248",
                "nt-260",
                "nt-270",
                "nt-292",
                "nt-305",
                "nt-308",
                "nt-330",
                "nt-332",
                "nt-336",
                "nt-344",
                "nt-348",
                "nt-350",
                "nt-353",
                "nt-364",
                "nt-367",
                "nt-378",
                "nt-380",
                "nt-392",
                "nt-397",
                "nt-411",
                "nt-416",
                "nt-423",
                "nt-428",
                "nt-439",
                "nt-447",
                "nt-450",
                "nt-457",
                "nt-458",
                "nt-470",
                "nt-473",
                "nt-493",
                "nt-506",
                "nt-512",
                "nt-516",
                "nt-534",
                "nt-542",
                "nt-560",
                "nt-574",
                "nt-579",
                "nt-582",
                "nt-614",
                "nt-615",
                "nt-624",
                "nt-634",
                "nt-640",
                "nt-647",
                "nt-652",
                "nt-667",
                "nt-668",
                "nt-673",
                "nt-680",
                "nt-683",
                "nt-692",
                "nt-694",
                "nt-708",
                "nt-713",
                "nt-725",
                "nt-727",
                "nt-730",
                "nt-748",
                "nt-756",
                "nt-762",
                "nt-769",
                "nt-771",
                "nt-793",
                "nt-805",
                "nt-815",
                "nt-818",
                "nt-823",
                "nt-825",
                "nt-826",
                "nt-830",
                "nt-832",
                "nt-836",
                "nt-837",
                "nt-842",
                "nt-856",
                "nt-876",
                "nt-880",
                "nt-881",
                "nt-892",
                "nt-908",
                "nt-912",
                "nt-924",
                "nt-941",
                "nt-944",
                "nt-963",
                "nt-986",
                "nt-994",
                "nt-1004",
                "nt-1007",
                "nt-1010",
                "nt-1013",
                "nt-1014",
                "nt-1015",
                "nt-1028",
                "nt-1037",
                "nt-1042",
                "nt-1052",
                "nt-1053",
                "nt-1058",
                "nt-1075",
                "nt-1077",
                "nt-1078",
                "nt-1082",
                "nt-1086",
                "nt-1105",
                "nt-1109",
                "nt-1111",
                "nt-1133",
                "nt-1135",
                "nt-1152",
                "nt-1158",
                "nt-1167",
                "nt-1177",
                "nt-1178",
                "nt-1183",
                "nt-1184",
                "nt-1190",
                "nt-1198",
                "nt-1204",
                "nt-1207",
                "nt-1212",
                "nt-1214",
                "nt-1217",
                "nt-1219",
                "nt-1223",
                "nt-1233",
                "nt-1242",
                "nt-1244",
                "nt-1259",
                "nt-1260",
                "nt-1261",
                "nt-1269",
                "nt-1273",
                "nt-1285",
                "nt-1291",
                "nt-1312",
                "nt-1327",
                "nt-1336",
                "nt-1338",
                "nt-1343",
                "nt-1368",
                "nt-1384",
                "nt-1385",
                "nt-1386",
                "nt-1394",
                "nt-1397",
                "nt-1399",
                "nt-1405",
                "nt-1421",
                "nt-1433",
                "nt-1446",
                "nt-1449",
                "nt-1454",
                "nt-1456",
                "nt-1472",
                "nt-1473",
                "nt-1475",
                "nt-1478",
                "nt-1480",
                "nt-1488",
                "nt-1490",
                "nt-1494",
                "nt-1502",
                "nt-1503",
                "nt-1510",
                "nt-1521",
                "nt-1528",
                "nt-1529",
                "nt-1530",
                "nt-1531",
                "nt-1544",
                "nt-1550",
                "nt-1553",
                "nt-1556",
                "nt-1564",
                "nt-1569",
                "nt-1573",
                "nt-1574",
                "nt-1581",
                "nt-1585",
                "nt-1591",
                "nt-1607",
                "nt-1614",
                "nt-1618",
                "nt-1624",
                "nt-1625",
                "nt-1626",
                "nt-1627",
                "nt-1651",
                "nt-1653",
                "nt-1664",
                "nt-1667",
                "nt-1668",
                "nt-1682",
                "nt-1707",
                "nt-1711",
                "nt-1714",
                "nt-1732",
                "nt-1734",
                "nt-1744",
                "nt-1760",
                "nt-1767",
                "nt-1772",
                "nt-1780",
                "nt-1782",
                "nt-1790",
                "nt-1806",
                "nt-1813",
                "nt-1827",
                "nt-1843",
                "nt-1845",
                "nt-1850",
                "nt-1856",
                "nt-1858",
                "nt-1864",
                "nt-1867",
                "nt-1883",
                "nt-1889",
                "nt-1906",
                "nt-1908",
                "nt-1919",
                "nt-1920",
                "nt-1939",
                "nt-1944",
                "nt-1967",
                "nt-1977",
                "nt-1984",
                "nt-1992",
                "nt-2000",
                "nt-2021",
                "nt-2035",
                "nt-2054",
                "nt-2055",
                "nt-2062",
                "nt-2074",
                "nt-2087",
                "nt-2103",
                "nt-2104",
                "nt-2121",
                "nt-2125",
                "nt-2141",
                "nt-2143",
                "nt-2149",
                "nt-2151",
                "nt-2165",
                "nt-2191",
                "nt-2192",
                "nt-2193",
                "nt-2205",
                "nt-2212",
                "nt-2228",
                "nt-2229",
                "nt-2231",
                "nt-2245",
                "nt-2254",
                "nt-2279",
                "nt-2286",
                "nt-2290",
                "nt-2295",
                "nt-2297",
                "nt-2308",
                "nt-2345",
                "nt-2387",
                "nt-2405",
                "nt-2419",
                "nt-2444",
                "nt-2455",
                "nt-2461",
                "nt-2467",
                "nt-2471",
                "nt-2475",
                "nt-2514",
                "nt-2518",
                "nt-2540",
                "nt-2555",
                "nt-2565",
                "nt-2569",
                "nt-2571",
                "nt-2577",
                "nt-2585",
                "nt-2592",
                "nt-2593",
                "nt-2595",
                "nt-2617",
                "nt-2635",
                "nt-2661",
                "nt-2663",
                "nt-2670",
                "nt-2690",
                "nt-2706",
                "nt-2730",
                "nt-2733",
                "nt-2750",
                "nt-2760",
                "nt-2769",
                "nt-2775",
                "nt-2786",
                "nt-2788",
                "nt-2815",
                "nt-2822",
                "nt-2826",
                "nt-2838",
                "nt-2842",
                "nt-2860",
                "nt-2863",
                "nt-2880",
                "nt-2882",
                "nt-2886",
                "nt-2894",
                "nt-2919",
                "nt-2952",
                "nt-2956",
                "nt-2960",
                "nt-2965",
                "nt-2967",
                "nt-2969",
                "nt-2983",
                "nt-2991",
                "nt-2992",
                "nt-2993",
                "nt-3004",
                "nt-3011",
                "nt-3012",
                "nt-3015",
                "nt-3034",
                "nt-3040",
                "nt-3050",
                "nt-3087",
                "nt-3098",
                "nt-3116",
                "nt-3122",
                "nt-3125",
                "nt-3127",
                "nt-3128",
                "nt-3136",
                "nt-3139",
                "nt-3150",
                "nt-3155",
                "nt-3169",
                "nt-3184",
                "nt-3196",
                "nt-3202",
                "nt-3222",
                "nt-3225",
                "nt-3233",
                "nt-3241",
                "nt-3246",
                "nt-3254",
                "nt-3260",
                "nt-3261",
                "nt-3264",
                "nt-3267",
                "nt-3268",
                "nt-3289",
                "nt-3293",
                "nt-3294",
                "nt-3311",
                "nt-3313",
                "nt-3321",
                "nt-3331",
                "nt-3337",
                "nt-3342",
                "nt-3347",
                "nt-3352",
                "nt-3364",
                "nt-3384",
                "nt-3392",
                "nt-3399",
                "nt-3414",
                "nt-3415",
                "nt-3419",
                "nt-3430",
                "nt-3437",
                "nt-3457",
                "nt-3459",
                "nt-3465",
                "nt-3483",
                "nt-3493",
                "nt-3515",
                "nt-3517",
                "nt-3521",
                "nt-3525",
                "nt-3537",
                "nt-3543",
                "nt-3548",
                "nt-3567",
                "nt-3587",
                "nt-3605",
                "nt-3613",
                "nt-3630",
                "nt-3651",
                "nt-3657",
                "nt-3674",
                "nt-3682",
                "nt-3690",
                "nt-3705",
                "nt-3721",
                "nt-3727",
                "nt-3732",
                "nt-3735",
                "nt-3743",
                "nt-3751",
                "nt-3762",
                "nt-3763",
                "nt-3767",
                "nt-3771",
                "nt-3780",
                "nt-3788",
                "nt-3789",
                "nt-3801",
                "nt-3824",
                "nt-3828",
                "nt-3854",
                "nt-3871",
                "nt-3874",
                "nt-3899",
                "nt-3942",
                "nt-3948",
                "nt-3964",
                "nt-3966",
                "nt-3975",
                "nt-3990",
                "nt-3994",
                "nt-3999",
                "nt-4026",
                "nt-4027",
                "nt-4041",
                "nt-4053",
                "nt-4083",
                "nt-4091",
                "nt-4095",
                "nt-4097",
                "nt-4108",
                "nt-4112",
                "nt-4114",
                "nt-4116",
                "nt-4120",
                "nt-4128",
                "nt-4139",
                "nt-4167",
                "nt-4181",
                "nt-4182",
                "nt-4186",
                "nt-4200",
                "nt-4202",
                "nt-4206",
                "nt-4212",
                "nt-4220",
                "nt-4250",
                "nt-4255",
                "nt-4265",
                "nt-4278",
                "nt-4281",
                "nt-4285",
                "nt-4289",
                "nt-4301",
                "nt-4303",
                "nt-4309",
                "nt-4328",
                "nt-4347",
                "nt-4350",
                "nt-4363",
                "nt-4365",
                "nt-4367",
                "nt-4369",
                "nt-4371",
                "nt-4390",
                "nt-4392",
                "nt-4416",
                "nt-4419",
                "nt-4421",
                "nt-4425",
                "nt-4426",
                "nt-4436",
                "nt-4441",
                "nt-4448",
                "nt-4459",
                "nt-4461",
                "nt-4467",
                "nt-4468",
                "nt-4500",
                "nt-4512",
                "nt-4516",
                "nt-4522",
                "nt-4530",
                "nt-4537",
                "nt-4550",
                "nt-4554",
                "nt-4559",
                "nt-4564",
                "nt-4569",
                "nt-4578",
                "nt-4585",
                "nt-4620",
                "nt-4627",
                "nt-4651",
                "nt-4653",
                "nt-4660",
                "nt-4664",
                "nt-4666",
                "nt-4670",
                "nt-4676",
                "nt-4693",
                "nt-4704",
                "nt-4708",
                "nt-4713",
                "nt-4740",
                "nt-4750",
                "nt-4764",
                "nt-4766",
                "nt-4771",
                "nt-4782",
                "nt-4789",
                "nt-4803",
                "nt-4806",
                "nt-4808",
                "nt-4809",
                "nt-4820",
                "nt-4821",
                "nt-4835",
                "nt-4842",
                "nt-4844",
                "nt-4858",
                "nt-4892",
                "nt-4900",
                "nt-4917",
                "nt-4918",
                "nt-4920",
                "nt-4932",
                "nt-4936",
                "nt-4944",
                "nt-4953",
                "nt-4975",
                "nt-4991",
                "nt-5010",
                "nt-5017",
                "nt-5042",
                "nt-5043",
                "nt-5048",
                "nt-5054",
                "nt-5061",
                "nt-5080",
                "nt-5095",
                "nt-5096",
                "nt-5103",
                "nt-5119",
                "nt-5121",
                "nt-5129",
                "nt-5135",
                "nt-5143",
                "nt-5159",
                "nt-5176",
                "nt-5178",
                "nt-5179",
                "nt-5186",
                "nt-5199",
                "nt-5207",
                "nt-5232",
                "nt-5240",
                "nt-5241",
                "nt-5244",
                "nt-5252",
                "nt-5259",
                "nt-5267",
                "nt-5268",
                "nt-5294",
                "nt-5298",
                "nt-5300",
                "nt-5310",
                "nt-5318",
                "nt-5319",
                "nt-5322",
                "nt-5325",
                "nt-5331",
                "nt-5337",
                "nt-5338",
                "nt-5341",
                "nt-5362",
                "nt-5390",
                "nt-5401",
                "nt-5416",
                "nt-5420",
                "nt-5425",
                "nt-5431",
                "nt-5433",
                "nt-5445",
                "nt-5449",
                "nt-5458",
                "nt-5461",
                "nt-5477",
                "nt-5486",
                "nt-5490",
                "nt-5502",
                "nt-5506",
                "nt-5508",
                "nt-5510",
                "nt-5547",
                "nt-5549",
                "nt-5551",
                "nt-5573",
                "nt-5592",
                "nt-5594",
                "nt-5619",
                "nt-5627",
                "nt-5628",
                "nt-5630",
                "nt-5642",
                "nt-5648",
                "nt-5670",
                "nt-5677",
                "nt-5693",
                "nt-5694",
                "nt-5696",
                "nt-5700",
                "nt-5705",
                "nt-5709",
                "nt-5733",
                "nt-5734",
                "nt-5736",
                "nt-5737",
                "nt-5751",
                "nt-5753",
                "nt-5768",
                "nt-5773",
                "nt-5786",
                "nt-5790",
                "nt-5792",
                "nt-5793",
                "nt-5804",
                "nt-5813",
                "nt-5817",
                "nt-5863",
                "nt-5868",
                "nt-5873",
                "nt-5881",
                "nt-5889",
                "nt-5891",
                "nt-5899",
                "nt-5905",
                "nt-5912",
                "nt-5924",
                "nt-5929",
                "nt-5960",
                "nt-5961",
                "nt-5969",
                "nt-5977",
                "nt-5985",
                "nt-5987",
                "nt-5992",
                "nt-5999",
                "nt-6016",
                "nt-6020",
                "nt-6027",
                "nt-6040",
                "nt-6045",
                "nt-6068",
                "nt-6074",
                "nt-6086",
                "nt-6101",
                "nt-6105",
                "nt-6112",
                "nt-6116",
                "nt-6117",
                "nt-6125",
                "nt-6127",
                "nt-6129",
                "nt-6151",
                "nt-6164",
                "nt-6181",
                "nt-6183",
                "nt-6185",
                "nt-6196",
                "nt-6204",
                "nt-6206",
                "nt-6210",
                "nt-6212",
                "nt-6219",
                "nt-6221",
                "nt-6224",
                "nt-6248",
                "nt-6258",
                "nt-6265",
                "nt-6267",
                "nt-6272",
                "nt-6274",
                "nt-6301",
                "nt-6302",
                "nt-6307",
                "nt-6325",
                "nt-6327",
                "nt-6331",
                "nt-6336",
                "nt-6340",
                "nt-6343",
                "nt-6350",
                "nt-6358",
                "nt-6390",
                "nt-6394",
                "nt-6396",
                "nt-6403",
                "nt-6414",
                "nt-6417",
                "nt-6426",
                "nt-6431",
                "nt-6434",
                "nt-6439",
                "nt-6441",
                "nt-6446",
                "nt-6458",
                "nt-6459",
                "nt-6478",
                "nt-6520",
                "nt-6526",
                "nt-6541",
                "nt-6548",
                "nt-6565",
                "nt-6608",
                "nt-6610",
                "nt-6615",
                "nt-6617",
                "nt-6625",
                "nt-6628",
                "nt-6635",
                "nt-6639",
                "nt-6679",
                "nt-6691",
                "nt-6698",
                "nt-6712",
                "nt-6731",
                "nt-6733",
                "nt-6738",
                "nt-6755",
                "nt-6760",
                "nt-6764",
                "nt-6778",
                "nt-6787",
                "nt-6811",
                "nt-6818",
                "nt-6819",
                "nt-6829",
                "nt-6830",
                "nt-6833",
                "nt-6845",
                "nt-6862",
                "nt-6865",
                "nt-6893",
                "nt-6898",
                "nt-6927",
                "nt-6932",
                "nt-6941",
                "nt-6973",
                "nt-6977",
                "nt-6985",
                "nt-6990",
                "nt-6994",
                "nt-6998",
                "nt-7001",
                "nt-7026",
                "nt-7031",
                "nt-7046",
                "nt-7052",
                "nt-7054",
                "nt-7057",
                "nt-7061",
                "nt-7090",
                "nt-7100",
                "nt-7118",
                "nt-7133",
                "nt-7134",
                "nt-7156",
                "nt-7167",
                "nt-7183",
                "nt-7197",
                "nt-7209",
                "nt-7211",
                "nt-7213",
                "nt-7218",
                "nt-7219",
                "nt-7229",
                "nt-7231",
                "nt-7233",
                "nt-7234",
                "nt-7246",
                "nt-7260",
                "nt-7263",
                "nt-7268",
                "nt-7271",
                "nt-7287",
                "nt-7294",
                "nt-7301",
                "nt-7310",
                "nt-7329",
                "nt-7331",
                "nt-7340",
                "nt-7348",
                "nt-7349",
                "nt-7351",
                "nt-7353",
                "nt-7364",
                "nt-7369",
                "nt-7401",
                "nt-7411",
                "nt-7416",
                "nt-7422",
                "nt-7429",
                "nt-7437",
                "nt-7464",
                "nt-7466",
                "nt-7474",
                "nt-7486",
                "nt-7487",
                "nt-7504",
                "nt-7509",
                "nt-7521",
                "nt-7525",
                "nt-7529",
                "nt-7532",
                "nt-7545",
                "nt-7551",
                "nt-7569",
                "nt-7579",
                "nt-7583",
                "nt-7613",
                "nt-7636",
                "nt-7640",
                "nt-7641",
                "nt-7662",
                "nt-7667",
                "nt-7681",
                "nt-7684",
                "nt-7685",
                "nt-7692",
                "nt-7701",
                "nt-7719",
                "nt-7740",
                "nt-7746",
                "nt-7764",
                "nt-7767",
                "nt-7770",
                "nt-7776",
                "nt-7807",
                "nt-7814",
                "nt-7830",
                "nt-7852",
                "nt-7861",
                "nt-7869",
                "nt-7878",
                "nt-7879",
                "nt-7882",
                "nt-7897",
                "nt-7902",
                "nt-7906",
                "nt-7918",
                "nt-7934",
                "nt-7937",
                "nt-7943",
                "nt-7946",
                "nt-7960",
                "nt-7967",
                "nt-7971",
                "nt-7976",
                "nt-7979",
                "nt-7981",
                "nt-7986",
                "nt-7995",
                "nt-7999",
                "nt-8004",
                "nt-8005",
                "nt-8009",
                "nt-8066",
                "nt-8078",
                "nt-8099",
                "nt-8107",
                "nt-8112",
                "nt-8119",
                "nt-8122",
                "nt-8124",
                "nt-8127",
                "nt-8130",
                "nt-8146",
                "nt-8149",
                "nt-8177",
                "nt-8198",
                "nt-8207",
                "nt-8210",
                "nt-8222",
                "nt-8226",
                "nt-8228",
                "nt-8235",
                "nt-8243",
                "nt-8248",
                "nt-8249",
                "nt-8266",
                "nt-8275",
                "nt-8288",
                "nt-8289",
                "nt-8293",
                "nt-8300",
                "nt-8306",
                "nt-8342",
                "nt-8350",
                "nt-8359",
                "nt-8366",
                "nt-8386",
                "nt-8391",
                "nt-8394",
                "nt-8395",
                "nt-8399",
                "nt-8409",
                "nt-8414",
                "nt-8422",
                "nt-8431",
                "nt-8435",
                "nt-8438",
                "nt-8440",
                "nt-8444",
                "nt-8445",
                "nt-8458",
                "nt-8460",
                "nt-8464",
                "nt-8468",
                "nt-8475",
                "nt-8476",
                "nt-8484",
                "nt-8485",
                "nt-8497",
                "nt-8512",
                "nt-8519",
                "nt-8523",
                "nt-8531",
                "nt-8535",
                "nt-8536",
                "nt-8541",
                "nt-8580",
                "nt-8583",
                "nt-8590",
                "nt-8606",
                "nt-8611",
                "nt-8615",
                "nt-8627",
                "nt-8631",
                "nt-8634",
                "nt-8659",
                "nt-8663",
                "nt-8669",
                "nt-8673",
                "nt-8687",
                "nt-8692",
                "nt-8703",
                "nt-8708",
                "nt-8710",
                "nt-8725",
                "nt-8745",
                "nt-8746",
                "nt-8777",
                "nt-8794",
                "nt-8800",
                "nt-8801",
                "nt-8804",
                "nt-8809",
                "nt-8822",
                "nt-8867",
                "nt-8876",
                "nt-8877",
                "nt-8881",
                "nt-8892",
                "nt-8902",
                "nt-8914",
                "nt-8942",
                "nt-8946",
                "nt-8949",
                "nt-8955",
                "nt-8961",
                "nt-8968",
                "nt-8971",
                "nt-8980",
                "nt-8993",
                "nt-9008",
                "nt-9027",
                "nt-9050",
                "nt-9073",
                "nt-9099",
                "nt-9114",
                "nt-9126",
                "nt-9127",
                "nt-9128",
                "nt-9133",
                "nt-9146",
                "nt-9152",
                "nt-9156",
                "nt-9163",
                "nt-9173",
                "nt-9177",
                "nt-9181",
                "nt-9196",
                "nt-9197",
                "nt-9203",
                "nt-9232",
                "nt-9238",
                "nt-9257",
                "nt-9261",
                "nt-9263",
                "nt-9270",
                "nt-9272",
                "nt-9285",
                "nt-9298",
                "nt-9299",
                "nt-9310",
                "nt-9313",
                "nt-9318",
                "nt-9326",
                "nt-9343",
                "nt-9346",
                "nt-9352",
                "nt-9362",
                "nt-9363",
                "nt-9370",
                "nt-9371",
                "nt-9378",
                "nt-9390",
                "nt-9405",
                "nt-9406",
                "nt-9409",
                "nt-9416",
                "nt-9421",
                "nt-9426",
                "nt-9431",
                "nt-9435",
                "nt-9441",
                "nt-9465",
                "nt-9476",
                "nt-9488",
                "nt-9493",
                "nt-9495",
                "nt-9504",
                "nt-9505",
                "nt-9506",
                "nt-9517",
                "nt-9529",
                "nt-9534",
                "nt-9540",
                "nt-9556",
                "nt-9566",
                "nt-9577",
                "nt-9589",
                "nt-9606",
                "nt-9607",
                "nt-9608",
                "nt-9609",
                "nt-9628",
                "nt-9630",
                "nt-9634",
                "nt-9654",
                "nt-9679",
                "nt-9691",
                "nt-9702",
                "nt-9705",
                "nt-9712",
                "nt-9715",
                "nt-9732",
                "nt-9739",
                "nt-9760",
                "nt-9767",
                "nt-9769",
                "nt-9779",
                "nt-9781",
                "nt-9812",
                "nt-9819",
                "nt-9822",
                "nt-9828",
                "nt-9850",
                "nt-9852",
                "nt-9855",
                "nt-9871",
                "nt-9884",
                "nt-9888",
                "nt-9894",
                "nt-9903",
                "nt-9907",
                "nt-9921",
                "nt-9924",
                "nt-9932",
                "nt-9933",
                "nt-9944",
                "nt-9946",
                "nt-9951",
                "nt-9954",
                "nt-9967",
                "nt-9977",
                "nt-10022",
                "nt-10026",
                "nt-10034",
                "nt-10043",
                "nt-10057",
                "nt-10059",
                "nt-10060",
                "nt-10066",
                "nt-10081",
                "nt-10087",
                "nt-10095",
                "nt-10107",
                "nt-10113",
                "nt-10119",
                "nt-10135",
                "nt-10155",
                "nt-10158",
                "nt-10160",
                "nt-10162",
                "nt-10176",
                "nt-10198",
                "nt-10200",
                "nt-10208",
                "nt-10227",
                "nt-10228",
                "nt-10235",
                "nt-10237",
                "nt-10247",
                "nt-10248",
                "nt-10259",
                "nt-10309",
                "nt-10334",
                "nt-10339",
                "nt-10363",
                "nt-10374",
                "nt-10389",
                "nt-10390",
                "nt-10411",
                "nt-10428",
                "nt-10430",
                "nt-10435",
                "nt-10442",
                "nt-10454",
                "nt-10461",
                "nt-10464",
                "nt-10465",
                "nt-10467",
                "nt-10471",
                "nt-10481",
                "nt-10483",
                "nt-10485",
                "nt-10486",
                "nt-10492",
                "nt-10500",
                "nt-10507",
                "nt-10508",
                "nt-10523",
                "nt-10535",
                "nt-10536",
                "nt-10542",
                "nt-10562",
                "nt-10597",
                "nt-10630",
                "nt-10654",
                "nt-10659",
                "nt-10664",
                "nt-10667",
                "nt-10681",
                "nt-10692",
                "nt-10707",
                "nt-10708",
                "nt-10726",
                "nt-10731",
                "nt-10736",
                "nt-10744",
                "nt-10757",
                "nt-10763",
                "nt-10767",
                "nt-10783",
                "nt-10798",
                "nt-10812",
                "nt-10827",
                "nt-10849",
                "nt-10859",
                "nt-10860",
                "nt-10865",
                "nt-10878",
                "nt-10881",
                "nt-10891",
                "nt-10894",
                "nt-10897",
                "nt-10923",
                "nt-10927",
                "nt-10931",
                "nt-10932",
                "nt-10947",
                "nt-10953",
                "nt-10959",
                "nt-10961",
                "nt-10972",
                "nt-10989",
                "nt-11003",
                "nt-11014",
                "nt-11021",
                "nt-11025",
                "nt-11029",
                "nt-11030",
                "nt-11034",
                "nt-11039",
                "nt-11042",
                "nt-11048",
                "nt-11053",
                "nt-11054",
                "nt-11066",
                "nt-11074",
                "nt-11088",
                "nt-11090",
                "nt-11092",
                "nt-11106",
                "nt-11119",
                "nt-11122",
                "nt-11124",
                "nt-11130",
                "nt-11142",
                "nt-11153",
                "nt-11163",
                "nt-11169",
                "nt-11173",
                "nt-11176",
                "nt-11182",
                "nt-11184",
                "nt-11191",
                "nt-11204",
                "nt-11207",
                "nt-11216",
                "nt-11230",
                "nt-11232",
                "nt-11255",
                "nt-11267",
                "nt-11278",
                "nt-11289",
                "nt-11303",
                "nt-11306",
                "nt-11307",
                "nt-11309",
                "nt-11310",
                "nt-11312",
                "nt-11326",
                "nt-11342",
                "nt-11351",
                "nt-11353",
                "nt-11358",
                "nt-11359",
                "nt-11377",
                "nt-11378",
                "nt-11382",
                "nt-11390",
                "nt-11404",
                "nt-11409",
                "nt-11413",
                "nt-11418",
                "nt-11419",
                "nt-11430",
                "nt-11435",
                "nt-11455",
                "nt-11470",
                "nt-11483",
                "nt-11488",
                "nt-11498",
                "nt-11500",
                "nt-11509",
                "nt-11526",
                "nt-11538",
                "nt-11539",
                "nt-11542",
                "nt-11544",
                "nt-11559",
                "nt-11575",
                "nt-11584",
                "nt-11607",
                "nt-11628",
                "nt-11632",
                "nt-11635",
                "nt-11641",
                "nt-11643",
                "nt-11650",
                "nt-11654",
                "nt-11664",
                "nt-11665",
                "nt-11671",
                "nt-11676",
                "nt-11677",
                "nt-11678",
                "nt-11679",
                "nt-11684",
                "nt-11700",
                "nt-11706",
                "nt-11717",
                "nt-11718",
                "nt-11724",
                "nt-11725",
                "nt-11727",
                "nt-11729",
                "nt-11747",
                "nt-11754",
                "nt-11762",
                "nt-11778",
                "nt-11791",
                "nt-11805",
                "nt-11807",
                "nt-11818",
                "nt-11839",
                "nt-11845",
                "nt-11870",
                "nt-11922",
                "nt-11928",
                "nt-11929",
                "nt-11950",
                "nt-11951",
                "nt-11953",
                "nt-11969",
                "nt-11978",
                "nt-11988",
                "nt-11996",
                "nt-12004",
                "nt-12014",
                "nt-12025",
                "nt-12026",
                "nt-12039",
                "nt-12044",
                "nt-12046",
                "nt-12052",
                "nt-12055",
                "nt-12061",
                "nt-12077",
                "nt-12082",
                "nt-12100",
                "nt-12104",
                "nt-12107",
                "nt-12113",
                "nt-12117",
                "nt-12125",
                "nt-12132",
                "nt-12141",
                "nt-12144",
                "nt-12149",
                "nt-12153",
                "nt-12171",
                "nt-12192",
                "nt-12200",
                "nt-12205",
                "nt-12207",
                "nt-12221",
                "nt-12239",
                "nt-12243",
                "nt-12248",
                "nt-12249",
                "nt-12255",
                "nt-12256",
                "nt-12258",
                "nt-12265",
                "nt-12268",
                "nt-12271",
                "nt-12277",
                "nt-12301",
                "nt-12313",
                "nt-12318",
                "nt-12319",
                "nt-12333",
                "nt-12340",
                "nt-12347",
                "nt-12349",
                "nt-12356",
                "nt-12387",
                "nt-12390",
                "nt-12391",
                "nt-12397",
                "nt-12423",
                "nt-12426",
                "nt-12427",
                "nt-12430",
                "nt-12433",
                "nt-12437",
                "nt-12438",
                "nt-12454",
                "nt-12455",
                "nt-12461",
                "nt-12467",
                "nt-12476",
                "nt-12495",
                "nt-12498",
                "nt-12521",
                "nt-12543",
                "nt-12554",
                "nt-12558",
                "nt-12562",
                "nt-12565",
                "nt-12567",
                "nt-12570",
                "nt-12578",
                "nt-12590",
                "nt-12592",
                "nt-12596",
                "nt-12609",
                "nt-12611",
                "nt-12632",
                "nt-12636",
                "nt-12637",
                "nt-12650",
                "nt-12673",
                "nt-12675",
                "nt-12679",
                "nt-12686",
                "nt-12709",
                "nt-12727",
                "nt-12747",
                "nt-12761",
                "nt-12765",
                "nt-12772",
                "nt-12784",
                "nt-12788",
                "nt-12792",
                "nt-12803",
                "nt-12807",
                "nt-12810",
                "nt-12816",
                "nt-12817",
                "nt-12850",
                "nt-12856",
                "nt-12872",
                "nt-12888",
                "nt-12889",
                "nt-12907",
                "nt-12914",
                "nt-12920",
                "nt-12921",
                "nt-12923",
                "nt-12925",
                "nt-12940",
                "nt-12960",
                "nt-12965",
                "nt-12967",
                "nt-12978",
                "nt-12980",
                "nt-12998",
                "nt-13000",
                "nt-13002",
                "nt-13029",
                "nt-13034",
                "nt-13040",
                "nt-13047",
                "nt-13072",
                "nt-13116",
                "nt-13118",
                "nt-13155",
                "nt-13176",
                "nt-13177",
                "nt-13185",
                "nt-13190",
                "nt-13196",
                "nt-13215",
                "nt-13224",
                "nt-13233",
                "nt-13235",
                "nt-13237",
                "nt-13264",
                "nt-13266",
                "nt-13281",
                "nt-13295",
                "nt-13299",
                "nt-13303",
                "nt-13311",
                "nt-13313",
                "nt-13316",
                "nt-13319",
                "nt-13340",
                "nt-13343",
                "nt-13352",
                "nt-13378",
                "nt-13389",
                "nt-13414",
                "nt-13420",
                "nt-13431",
                "nt-13437",
                "nt-13459",
                "nt-13467",
                "nt-13468",
                "nt-13471",
                "nt-13477",
                "nt-13483",
                "nt-13486",
                "nt-13506",
                "nt-13512",
                "nt-13517",
                "nt-13518",
                "nt-13527",
                "nt-13529",
                "nt-13535",
                "nt-13546",
                "nt-13556",
                "nt-13565",
                "nt-13569",
                "nt-13573",
                "nt-13591",
                "nt-13594",
                "nt-13609",
                "nt-13633",
                "nt-13644",
                "nt-13653",
                "nt-13659",
                "nt-13660",
                "nt-13665",
                "nt-13671",
                "nt-13699",
                "nt-13703",
                "nt-13706",
                "nt-13728",
                "nt-13734",
                "nt-13735",
                "nt-13764",
                "nt-13770",
                "nt-13777",
                "nt-13783",
                "nt-13784",
                "nt-13785",
                "nt-13788",
                "nt-13801",
                "nt-13816",
                "nt-13819",
                "nt-13822",
                "nt-13848",
                "nt-13851",
                "nt-13866",
                "nt-13869",
                "nt-13870",
                "nt-13871",
                "nt-13877",
                "nt-13879",
                "nt-13888",
                "nt-13891",
                "nt-13916",
                "nt-13923",
                "nt-13932",
                "nt-13933",
                "nt-13937",
                "nt-13949",
                "nt-13950",
                "nt-13954",
                "nt-13963",
                "nt-13988",
                "nt-13990",
                "nt-13992",
                "nt-13998",
                "nt-14008",
                "nt-14011",
                "nt-14020",
                "nt-14029",
                "nt-14059",
                "nt-14073",
                "nt-14095",
                "nt-14096",
                "nt-14097",
                "nt-14107",
                "nt-14135",
                "nt-14144",
                "nt-14150"
                ]
missing_ids = ['nt-2254', 'nt-3125', 'nt-3264', 'nt-6446', 'nt-7882', 'nt-8692', 'nt-9126', 'nt-9181', 'nt-12427',
               'nt-525', 'nt-9240', 'nt-11334', 'nt-623', 'nt-1679', 'nt-3255', 'nt-4637', 'nt-4675', 'nt-6948',
               'nt-7250', 'nt-10180', 'nt-10945', 'nt-12693', 'nt-605', 'nt-718', 'nt-3124', 'nt-4439', 'nt-5989',
               'nt-7940', 'nt-8371', 'nt-9433', 'nt-13593', 'nt-859', 'nt-3105', 'nt-5539', 'nt-6988', 'nt-11729',
               'nt-12596', 'nt-14129', 'nt-2426', 'nt-4547', 'nt-5281', 'nt-6387', 'nt-8219', 'nt-8267', 'nt-10165',
               'nt-10820', 'nt-11023', 'nt-11869', 'nt-12681', 'nt-13248', 'nt-941', 'nt-3246', 'nt-5080', 'nt-6426',
               'nt-7026', 'nt-8445', 'nt-9623', 'nt-11488', 'nt-12792', 'nt-6361', 'nt-10261', 'nt-1348', 'nt-976',
               'nt-1654', 'nt-4394', 'nt-4573', 'nt-5234', 'nt-13358', 'nt-1267', 'nt-1688', 'nt-1803', 'nt-2452',
               'nt-4370', 'nt-5110', 'nt-6110', 'nt-7358', 'nt-9412', 'nt-13993', 'nt-3020', 'nt-5435', 'nt-5538',
               'nt-5815', 'nt-6237', 'nt-6353', 'nt-7998', 'nt-11293', 'nt-1746', 'nt-2864', 'nt-4253', 'nt-1487',
               'nt-3243', 'nt-6107', 'nt-7278', 'nt-7523', 'nt-10091', 'nt-14123', 'nt-3339', 'nt-4926', 'nt-5514',
               'nt-6800', 'nt-7749', 'nt-9587', 'nt-11078', 'nt-12851', 'nt-1078', 'nt-1711', 'nt-5054', 'nt-5573',
               'nt-6764', 'nt-11949', 'nt-12567', 'nt-135', 'nt-431', 'nt-5968', 'nt-6511', 'nt-7511', 'nt-8617',
               'nt-9420', 'nt-10973', 'nt-11214', 'nt-2647', 'nt-4702', 'nt-11486', 'nt-5973', 'nt-6472', 'nt-7683',
               'nt-10646', 'nt-2117', 'nt-2210', 'nt-2694', 'nt-6487', 'nt-12119', 'nt-12962', 'nt-12966', 'nt-13129',
               'nt-1363', 'nt-4595', 'nt-5497', 'nt-6634', 'nt-6805', 'nt-9097', 'nt-11208', 'nt-2037', 'nt-3208',
               'nt-4555', 'nt-6013', 'nt-6668', 'nt-7109', 'nt-7426', 'nt-8553', 'nt-9151', 'nt-11104', 'nt-11924',
               'nt-13709', 'nt-56', 'nt-346', 'nt-4427', 'nt-4768', 'nt-2406', 'nt-6166', 'nt-7309', 'nt-10837',
               'nt-10841', 'nt-11474', 'nt-12242', 'nt-13793', 'nt-4530', 'nt-13594', 'nt-13653', 'nt-71', 'nt-776',
               'nt-1526', 'nt-2280', 'nt-5088', 'nt-5831', 'nt-8097', 'nt-12009', 'nt-13246', 'nt-1635', 'nt-1737',
               'nt-5032', 'nt-8022', 'nt-8284', 'nt-8632', 'nt-8933', 'nt-9246', 'nt-10515', 'nt-10739', 'nt-10805',
               'nt-857', 'nt-2809', 'nt-3830', 'nt-6929', 'nt-9697', 'nt-9814', 'nt-11445', 'nt-13055', 'nt-2740',
               'nt-5452', 'nt-6369', 'nt-6579', 'nt-7477', 'nt-8619', 'nt-8624', 'nt-9401', 'nt-10609', 'nt-2870',
               'nt-4019', 'nt-4239', 'nt-5744', 'nt-6299', 'nt-8053', 'nt-9168', 'nt-11656', 'nt-2872', 'nt-1607',
               'nt-2205', 'nt-673', 'nt-2519', 'nt-2860', 'nt-3294', 'nt-4202', 'nt-4363', 'nt-5887', 'nt-6403',
               'nt-10681', 'nt-11900', 'nt-12592', 'nt-1254', 'nt-1924', 'nt-2951', 'nt-4643', 'nt-10479', 'nt-10895',
               'nt-11193', 'nt-11453', 'nt-13189', 'nt-399', 'nt-2835', 'nt-3755', 'nt-6324', 'nt-7483', 'nt-13585',
               'nt-13825', 'nt-367', 'nt-924', 'nt-5298', 'nt-6898', 'nt-8146', 'nt-9852', 'nt-12044', 'nt-13528',
               'nt-68', 'nt-3605', 'nt-4662', 'nt-9377', 'nt-11169', 'nt-11176', 'nt-11641', 'nt-12248', 'nt-12258',
               'nt-12347', 'nt-586', 'nt-1288', 'nt-1505', 'nt-3583', 'nt-5016', 'nt-10349', 'nt-12551', 'nt-12979',
               'nt-13717', 'nt-13732', 'nt-743', 'nt-6923', 'nt-7298', 'nt-7553', 'nt-7712', 'nt-10844', 'nt-881',
               'nt-1386', 'nt-4849', 'nt-11191', 'nt-12221', 'nt-806', 'nt-2903', 'nt-3648', 'nt-5937', 'nt-8977',
               'nt-9649', 'nt-12984', 'nt-1646', 'nt-5496', 'nt-6842', 'nt-8232', 'nt-11936', 'nt-4610', 'nt-5945',
               'nt-6590', 'nt-7217', 'nt-7670', 'nt-8123', 'nt-9309', 'nt-10988', 'nt-1987', 'nt-3180', 'nt-4318',
               'nt-5659', 'nt-10232', 'nt-12560', 'nt-13195', 'nt-923', 'nt-3389', 'nt-3907', 'nt-5441', 'nt-6326',
               'nt-6938', 'nt-8163', 'nt-14142', 'nt-511', 'nt-4383', 'nt-4853', 'nt-4894', 'nt-9770', 'nt-12049',
               'nt-13873', 'nt-1242', 'nt-1269', 'nt-5881', 'nt-10363', 'nt-11267', 'nt-11378', 'nt-12493', 'nt-13706',
               'nt-13915', 'nt-3153', 'nt-3497', 'nt-4827', 'nt-5216', 'nt-7113', 'nt-10042', 'nt-11022', 'nt-273',
               'nt-2045', 'nt-3724', 'nt-3739', 'nt-5771', 'nt-8352', 'nt-11577', 'nt-8155', 'nt-7139', 'nt-8788',
               'nt-12515', 'nt-12814', 'nt-254', 'nt-3205', 'nt-3276', 'nt-3958', 'nt-12108', 'nt-13461', 'nt-13914',
               'nt-214', 'nt-4633', 'nt-1876', 'nt-2423', 'nt-2608', 'nt-4141', 'nt-13510', 'nt-1824', 'nt-1925',
               'nt-6360', 'nt-7039', 'nt-8808', 'nt-8937', 'nt-11036', 'nt-650', 'nt-1122', 'nt-9722', 'nt-10971',
               'nt-11890', 'nt-11966', 'nt-12186', 'nt-1466', 'nt-1469', 'nt-1963', 'nt-5640', 'nt-9153', 'nt-11503',
               'nt-713', 'nt-1042', 'nt-3630', 'nt-5017', 'nt-6302', 'nt-7197', 'nt-8107', 'nt-9390', 'nt-9606',
               'nt-12125', 'nt-702', 'nt-1871', 'nt-2594', 'nt-3447', 'nt-3947', 'nt-6303', 'nt-7230', 'nt-9131',
               'nt-9786', 'nt-9963', 'nt-13037', 'nt-13918', 'nt-14084', 'nt-897', 'nt-3489', 'nt-3772', 'nt-9640',
               'nt-10427', 'nt-10694', 'nt-10710', 'nt-10779', 'nt-10788', 'nt-11247', 'nt-1366', 'nt-1706', 'nt-2252',
               'nt-5251', 'nt-8269', 'nt-9125', 'nt-11339', 'nt-11907', 'nt-2908', 'nt-2944', 'nt-5972', 'nt-6341',
               'nt-6581', 'nt-6786', 'nt-10532', 'nt-11405', 'nt-12492', 'nt-1509', 'nt-2800', 'nt-3490', 'nt-6490',
               'nt-7313', 'nt-12199', 'nt-12571', 'nt-13147', 'nt-630', 'nt-2625', 'nt-2758', 'nt-6516', 'nt-7817',
               'nt-8847', 'nt-9756', 'nt-10331', 'nt-12619', 'nt-3964', 'nt-6417', 'nt-6698', 'nt-6755', 'nt-11762',
               'nt-13998', 'nt-2429', 'nt-3815', 'nt-3993', 'nt-5671', 'nt-6359', 'nt-7784', 'nt-10603', 'nt-11237',
               'nt-3944', 'nt-5554', 'nt-6213', 'nt-6975', 'nt-8770', 'nt-10801', 'nt-12439', 'nt-12989', 'nt-13148',
               'nt-596', 'nt-1621', 'nt-2053', 'nt-3209', 'nt-4811', 'nt-5311', 'nt-5312', 'nt-5710', 'nt-5964',
               'nt-6223', 'nt-9785', 'nt-12324', 'nt-12787', 'nt-1522', 'nt-2083', 'nt-4770', 'nt-6006', 'nt-6959',
               'nt-8938', 'nt-11236', 'nt-12751', 'nt-413', 'nt-11939', 'nt-13359', 'nt-13622', 'nt-392', 'nt-1624',
               'nt-1626', 'nt-6164', 'nt-9318', 'nt-10763', 'nt-12940', 'nt-13992', 'nt-2912', 'nt-6114', 'nt-9630',
               'nt-10677', 'nt-12720', 'nt-13656', 'nt-4288', 'nt-10240', 'nt-10990', 'nt-11002', 'nt-12961',
               'nt-13673', 'nt-779', 'nt-1100', 'nt-2753', 'nt-2918', 'nt-4686', 'nt-5603', 'nt-7556', 'nt-10009',
               'nt-11537', 'nt-12626', 'nt-13117', 'nt-14022', 'nt-33', 'nt-3417', 'nt-6402', 'nt-9625', 'nt-10182',
               'nt-10789', 'nt-13252', 'nt-1500', 'nt-6021', 'nt-9862', 'nt-10494', 'nt-1292', 'nt-3916', 'nt-4168',
               'nt-4469', 'nt-4658', 'nt-6573', 'nt-8487', 'nt-9477', 'nt-10203', 'nt-12736', 'nt-3068', 'nt-3220',
               'nt-4305', 'nt-7356', 'nt-7512', 'nt-11215', 'nt-11918', 'nt-12997', 'nt-13519', 'nt-13601', 'nt-880',
               'nt-8255', 'nt-8745', 'nt-13340', 'nt-13556', 'nt-1138', 'nt-2200', 'nt-3696', 'nt-5661', 'nt-10618',
               'nt-11535', 'nt-1312', 'nt-3004', 'nt-3537', 'nt-4991', 'nt-11664', 'nt-12803', 'nt-13299', 'nt-1555',
               'nt-1687', 'nt-8963', 'nt-10647', 'nt-13411', 'nt-13979', 'nt-1757', 'nt-1832', 'nt-2336', 'nt-4345',
               'nt-6562', 'nt-7032', 'nt-8230', 'nt-10691', 'nt-11562', 'nt-12222', 'nt-899', 'nt-7174', 'nt-9824',
               'nt-1445', 'nt-5222', 'nt-5836', 'nt-6187', 'nt-6619', 'nt-6879', 'nt-7715', 'nt-8823', 'nt-9472',
               'nt-10992', 'nt-11569', 'nt-11759', 'nt-12809', 'nt-651', 'nt-1010', 'nt-1682', 'nt-1893', 'nt-2967',
               'nt-4627', 'nt-8306', 'nt-9577', 'nt-12611', 'nt-7562', 'nt-2446', 'nt-4235', 'nt-5522', 'nt-7207',
               'nt-9295', 'nt-1710', 'nt-2120', 'nt-5855', 'nt-5983', 'nt-6540', 'nt-9428', 'nt-10526', 'nt-1279',
               'nt-1529', 'nt-6331', 'nt-6778', 'nt-10113', 'nt-11901', 'nt-12584', 'nt-3428', 'nt-5926', 'nt-7189',
               'nt-7560', 'nt-9658', 'nt-13781', 'nt-2693', 'nt-6587', 'nt-8010', 'nt-9369', 'nt-12887', 'nt-1180',
               'nt-1929', 'nt-3719', 'nt-4038', 'nt-9457', 'nt-12899', 'nt-1308', 'nt-1768', 'nt-2196', 'nt-9273',
               'nt-9991', 'nt-770', 'nt-6126', 'nt-10769', 'nt-12831', 'nt-3101', 'nt-3143', 'nt-4201', 'nt-4244',
               'nt-4843', 'nt-5049', 'nt-6195', 'nt-6199', 'nt-8557', 'nt-11710', 'nt-13017', 'nt-13109', 'nt-1955',
               'nt-4205', 'nt-8405', 'nt-10086', 'nt-11212', 'nt-11846', 'nt-12295', 'nt-2926', 'nt-3210', 'nt-3463',
               'nt-4209', 'nt-8080', 'nt-9644', 'nt-12025', 'nt-13127', 'nt-1314', 'nt-1977', 'nt-2620', 'nt-2635',
               'nt-4496', 'nt-6414', 'nt-6783', 'nt-7537', 'nt-11342', 'nt-578', 'nt-1443', 'nt-2713', 'nt-5370',
               'nt-5532', 'nt-6670', 'nt-8389', 'nt-8581', 'nt-9508', 'nt-10123', 'nt-13481', 'nt-118', 'nt-863',
               'nt-1353', 'nt-3160', 'nt-3502', 'nt-4100', 'nt-6822', 'nt-9943', 'nt-10977', 'nt-13150', 'nt-530',
               'nt-1788', 'nt-2641', 'nt-2707', 'nt-4748', 'nt-6558', 'nt-6947', 'nt-8404', 'nt-9533', 'nt-11816',
               'nt-13790', 'nt-856', 'nt-2516', 'nt-4000', 'nt-4409', 'nt-4441', 'nt-4951', 'nt-5362', 'nt-5648',
               'nt-7504', 'nt-8293', 'nt-9362', 'nt-11272', 'nt-891', 'nt-4652', 'nt-5444', 'nt-5557', 'nt-5618',
               'nt-7732', 'nt-12168', 'nt-408', 'nt-1781', 'nt-4279', 'nt-6644', 'nt-7102', 'nt-7549', 'nt-8178',
               'nt-10643', 'nt-12198', 'nt-12441', 'nt-12646', 'nt-13188', 'nt-561', 'nt-1877', 'nt-3703', 'nt-8995',
               'nt-9266', 'nt-9731', 'nt-10130', 'nt-10289', 'nt-12494', 'nt-12725', 'nt-59', 'nt-932', 'nt-6561',
               'nt-7228', 'nt-8203', 'nt-9086', 'nt-10218', 'nt-10491', 'nt-10514', 'nt-12145', 'nt-1609', 'nt-2174',
               'nt-2346', 'nt-3416', 'nt-6832', 'nt-7617', 'nt-10258', 'nt-11587', 'nt-12233', 'nt-13620', 'nt-9750',
               'nt-13677', 'nt-278', 'nt-459', 'nt-1420', 'nt-2227', 'nt-3036', 'nt-4417', 'nt-4449', 'nt-8642',
               'nt-10156', 'nt-11209', 'nt-12833', 'nt-4461', 'nt-5021', 'nt-11542', 'nt-4268', 'nt-4611', 'nt-5003',
               'nt-5410', 'nt-13554', 'nt-246', 'nt-953', 'nt-1057', 'nt-3360', 'nt-3929', 'nt-7541', 'nt-8191',
               'nt-8524', 'nt-8598', 'nt-12308', 'nt-13550', 'nt-398', 'nt-1365', 'nt-2242', 'nt-3427', 'nt-3718',
               'nt-4348', 'nt-6984', 'nt-10039', 'nt-11825', 'nt-1375', 'nt-2026', 'nt-2914', 'nt-3117', 'nt-6318',
               'nt-7676', 'nt-8369', 'nt-9733', 'nt-11958', 'nt-13069', 'nt-13496', 'nt-13672', 'nt-1280', 'nt-5986',
               'nt-6695', 'nt-8628', 'nt-9028', 'nt-10115', 'nt-10700', 'nt-11388', 'nt-11634', 'nt-13401', 'nt-1961',
               'nt-5204', 'nt-6746', 'nt-7115', 'nt-8625', 'nt-9391', 'nt-12854', 'nt-13739', 'nt-14065', 'nt-668',
               'nt-1456', 'nt-3237', 'nt-9163', 'nt-13002', 'nt-2283', 'nt-4762', 'nt-6627', 'nt-2385', 'nt-5030',
               'nt-6784', 'nt-10020', 'nt-10747', 'nt-10919', 'nt-1182', 'nt-4502', 'nt-6534', 'nt-8101', 'nt-9908',
               'nt-11174', 'nt-4838', 'nt-5761', 'nt-7978', 'nt-9255', 'nt-13482', 'nt-10223', 'nt-13458', 'nt-4007',
               'nt-6192', 'nt-6790', 'nt-7302', 'nt-9547', 'nt-10364', 'nt-10638', 'nt-12002', 'nt-1741', 'nt-4601',
               'nt-5007', 'nt-7709', 'nt-8913', 'nt-9564', 'nt-9823', 'nt-12890', 'nt-79', 'nt-926', 'nt-1444',
               'nt-7069', 'nt-9287', 'nt-10955', 'nt-1725', 'nt-5682', 'nt-8291', 'nt-1148', 'nt-2233', 'nt-3216',
               'nt-4589', 'nt-5163', 'nt-7247', 'nt-9394', 'nt-10446', 'nt-10686', 'nt-11917', 'nt-12409', 'nt-2644',
               'nt-8153', 'nt-8489', 'nt-10588', 'nt-2998', 'nt-4018', 'nt-5605', 'nt-7290', 'nt-8542', 'nt-9244',
               'nt-11408', 'nt-429', 'nt-2363', 'nt-2526', 'nt-3402', 'nt-4934', 'nt-5422', 'nt-8070', 'nt-9655',
               'nt-11383', 'nt-12031', 'nt-4613', 'nt-5639', 'nt-11786', 'nt-724', 'nt-1000', 'nt-1414', 'nt-6096',
               'nt-7434', 'nt-9110', 'nt-10815', 'nt-10999', 'nt-11103', 'nt-2080', 'nt-6061', 'nt-7283', 'nt-7602',
               'nt-993', 'nt-1587', 'nt-4765', 'nt-4929', 'nt-5306', 'nt-6227', 'nt-7391', 'nt-7813', 'nt-11006',
               'nt-11794', 'nt-12394', 'nt-12786', 'nt-13778', 'nt-9893', 'nt-12344', 'nt-2113', 'nt-2513', 'nt-3063',
               'nt-4032', 'nt-5289', 'nt-7698', 'nt-9714', 'nt-9773', 'nt-9976', 'nt-10408', 'nt-50', 'nt-1431',
               'nt-4046', 'nt-6960', 'nt-9228', 'nt-12523', 'nt-1899', 'nt-2889', 'nt-3370', 'nt-4241', 'nt-5378',
               'nt-6530', 'nt-7346', 'nt-12949', 'nt-5477', 'nt-9238', 'nt-10536', 'nt-14020', 'nt-683', 'nt-2218',
               'nt-10628', 'nt-11048', 'nt-11951', 'nt-357', 'nt-1084', 'nt-3081', 'nt-6244', 'nt-8593', 'nt-9344',
               'nt-10175', 'nt-12106', 'nt-5269', 'nt-5704', 'nt-7768', 'nt-7914', 'nt-8292', 'nt-10414', 'nt-10527',
               'nt-3795', 'nt-5817', 'nt-6679', 'nt-7116', 'nt-7335', 'nt-12920', 'nt-12960', 'nt-13381', 'nt-961',
               'nt-1504', 'nt-4445', 'nt-4847', 'nt-5302', 'nt-6079', 'nt-7548', 'nt-10100', 'nt-11162', 'nt-1255',
               'nt-7012', 'nt-8142', 'nt-8518', 'nt-8551', 'nt-8806', 'nt-8939', 'nt-13221', 'nt-2308', 'nt-3034',
               'nt-3196', 'nt-3493', 'nt-4112', 'nt-4670', 'nt-11376', 'nt-12268', 'nt-600', 'nt-2206', 'nt-2899',
               'nt-6167', 'nt-9928', 'nt-11716', 'nt-13765', 'nt-8538', 'nt-12807', 'nt-12980', 'nt-13076', 'nt-3519',
               'nt-6444', 'nt-9845', 'nt-11076', 'nt-11894', 'nt-11984', 'nt-13346', 'nt-13712', 'nt-14041', 'nt-738',
               'nt-5051', 'nt-5321', 'nt-6493', 'nt-8264', 'nt-8459', 'nt-2246', 'nt-5909', 'nt-9941', 'nt-6236',
               'nt-6285', 'nt-8504', 'nt-10326', 'nt-11883', 'nt-1954', 'nt-4185', 'nt-7128', 'nt-9798', 'nt-14046',
               'nt-3859', 'nt-5085', 'nt-6076', 'nt-10344', 'nt-11350', 'nt-12555', 'nt-1808', 'nt-4379', 'nt-5008',
               'nt-8233', 'nt-12227', 'nt-12724', 'nt-13169', 'nt-5148', 'nt-6141', 'nt-7629', 'nt-8790', 'nt-9937',
               'nt-11322', 'nt-11485', 'nt-11740', 'nt-12247', 'nt-13211', 'nt-13823', 'nt-1675', 'nt-4696', 'nt-5071',
               'nt-6872', 'nt-9419', 'nt-9942', 'nt-11990', 'nt-12293', 'nt-12664', 'nt-13826', 'nt-887', 'nt-3527',
               'nt-3666', 'nt-9169', 'nt-10567', 'nt-10937', 'nt-2018', 'nt-2393', 'nt-6655', 'nt-7938', 'nt-8970',
               'nt-9886', 'nt-11696', 'nt-13170', 'nt-87', 'nt-3321', 'nt-9721', 'nt-12675', 'nt-13034', 'nt-13176',
               'nt-558', 'nt-811', 'nt-3189', 'nt-3453', 'nt-12396', 'nt-5929', 'nt-7167', 'nt-7525', 'nt-9855',
               'nt-13237', 'nt-1476', 'nt-3701', 'nt-5785', 'nt-6491', 'nt-11647', 'nt-14009', 'nt-2932', 'nt-3144',
               'nt-9038', 'nt-11638', 'nt-12202', 'nt-12540', 'nt-301', 'nt-1524', 'nt-3290', 'nt-4147', 'nt-4562',
               'nt-5691', 'nt-6515', 'nt-8622', 'nt-9193', 'nt-10161', 'nt-11037', 'nt-11456', 'nt-2556', 'nt-7530',
               'nt-8539', 'nt-9899', 'nt-10252', 'nt-12841', 'nt-1565', 'nt-2312', 'nt-2384', 'nt-2746', 'nt-2877',
               'nt-5716', 'nt-6156', 'nt-8357', 'nt-9459', 'nt-2580', 'nt-4824', 'nt-7271', 'nt-7902', 'nt-9214',
               'nt-9977', 'nt-13699', 'nt-13784', 'nt-5423', 'nt-7121', 'nt-7536', 'nt-7706', 'nt-8195', 'nt-8214',
               'nt-10004', 'nt-12548', 'nt-13457', 'nt-8686', 'nt-5257', 'nt-6700', 'nt-7141', 'nt-7479', 'nt-8958',
               'nt-11913', 'nt-13043', 'nt-3231', 'nt-3528', 'nt-4470', 'nt-5162', 'nt-7599', 'nt-8001', 'nt-8180',
               'nt-9341', 'nt-9612', 'nt-13769', 'nt-1051', 'nt-2439', 'nt-2823', 'nt-4172', 'nt-4524', 'nt-5689',
               'nt-8896', 'nt-9874', 'nt-9969', 'nt-10033', 'nt-12685', 'nt-13984', 'nt-1152', 'nt-1405', 'nt-2054',
               'nt-5992', 'nt-9654', 'nt-12100', 'nt-12810', 'nt-14150', 'nt-3614', 'nt-5028', 'nt-6671', 'nt-12134',
               'nt-12312', 'nt-12485', 'nt-13799', 'nt-1050', 'nt-2056', 'nt-4476', 'nt-6867', 'nt-8303', 'nt-9530',
               'nt-1185', 'nt-4987', 'nt-5415', 'nt-13473', 'nt-1794', 'nt-3611', 'nt-4745', 'nt-13083', 'nt-356',
               'nt-2042', 'nt-2459', 'nt-4111', 'nt-4375', 'nt-5381', 'nt-9562', 'nt-12665', 'nt-13130', 'nt-6893',
               'nt-11988', 'nt-1838', 'nt-2065', 'nt-2803', 'nt-4233', 'nt-5347', 'nt-6575', 'nt-8712', 'nt-11766',
               'nt-2849', 'nt-3026', 'nt-3714', 'nt-5363', 'nt-12343', 'nt-12900', 'nt-5985', 'nt-10389', 'nt-11377',
               'nt-12637', 'nt-632', 'nt-1960', 'nt-2779', 'nt-5005', 'nt-5122', 'nt-7575', 'nt-7880', 'nt-9446',
               'nt-2404', 'nt-3104', 'nt-3183', 'nt-10220', 'nt-11156', 'nt-12801', 'nt-13621', 'nt-13905', 'nt-674',
               'nt-1351', 'nt-2375', 'nt-2483', 'nt-2850', 'nt-3992', 'nt-5019', 'nt-7478', 'nt-7831', 'nt-7866',
               'nt-13779', 'nt-755', 'nt-2782', 'nt-8176', 'nt-10307', 'nt-11843', 'nt-4096', 'nt-134', 'nt-1783',
               'nt-3348', 'nt-4896', 'nt-7526', 'nt-8852', 'nt-9666', 'nt-10230', 'nt-602', 'nt-5842', 'nt-7727',
               'nt-7895', 'nt-8397', 'nt-10846', 'nt-402', 'nt-3161', 'nt-8554', 'nt-9134', 'nt-11864', 'nt-12232',
               'nt-12481', 'nt-12629', 'nt-13730', 'nt-2546', 'nt-3476', 'nt-6748', 'nt-9642', 'nt-10074', 'nt-13509',
               'nt-2376', 'nt-7756', 'nt-8824', 'nt-10148', 'nt-12625', 'nt-13334', 'nt-2832', 'nt-10334', 'nt-10401',
               'nt-10767', 'nt-12318', 'nt-12872', 'nt-7231', 'nt-8800', 'nt-3301', 'nt-3894', 'nt-6594', 'nt-9501',
               'nt-10350', 'nt-10874', 'nt-14089', 'nt-1214', 'nt-1618', 'nt-4347', 'nt-6833', 'nt-7946', 'nt-10630',
               'nt-11289', 'nt-13546', 'nt-11411', 'nt-12259', 'nt-12329', 'nt-13403', 'nt-366', 'nt-1245', 'nt-1373',
               'nt-5094', 'nt-7538', 'nt-8587', 'nt-9910', 'nt-12348', 'nt-606', 'nt-2463', 'nt-6407', 'nt-8950',
               'nt-12552', 'nt-12796', 'nt-3533', 'nt-3566', 'nt-3789', 'nt-7583', 'nt-8177', 'nt-10030', 'nt-10523',
               'nt-13963', 'nt-1997', 'nt-2347', 'nt-2476', 'nt-5371', 'nt-5870', 'nt-10585', 'nt-11060', 'nt-622',
               'nt-1407', 'nt-2258', 'nt-2494', 'nt-2727', 'nt-6128', 'nt-8259', 'nt-11244', 'nt-11920', 'nt-549',
               'nt-2600', 'nt-2929', 'nt-4133', 'nt-4623', 'nt-4626', 'nt-7515', 'nt-8020', 'nt-9350', 'nt-10044',
               'nt-13191', 'nt-466', 'nt-1462', 'nt-2216', 'nt-2923', 'nt-4628', 'nt-7761', 'nt-8138', 'nt-11321',
               'nt-13632', 'nt-965', 'nt-2692', 'nt-3675', 'nt-11019', 'nt-2551', 'nt-3898', 'nt-5455', 'nt-5853',
               'nt-6321', 'nt-7065', 'nt-7275', 'nt-10386', 'nt-805', 'nt-2886', 'nt-5510', 'nt-6001', 'nt-9087',
               'nt-12253', 'nt-5908', 'nt-9974', 'nt-11701', 'nt-1044', 'nt-2066', 'nt-3186', 'nt-3943', 'nt-3945',
               'nt-4197', 'nt-4854', 'nt-5109', 'nt-8605', 'nt-1107', 'nt-1718', 'nt-3371', 'nt-5635', 'nt-194',
               'nt-1620', 'nt-3303', 'nt-9590', 'nt-615', 'nt-1984', 'nt-2825', 'nt-5268', 'nt-8395', 'nt-9517',
               'nt-11929', 'nt-13313', 'nt-4169', 'nt-4950', 'nt-7067', 'nt-7637', 'nt-8047', 'nt-8569', 'nt-8839',
               'nt-10441', 'nt-10880', 'nt-12545', 'nt-260', 'nt-1488', 'nt-2281', 'nt-2919', 'nt-3657', 'nt-4572',
               'nt-5668', 'nt-5912', 'nt-8785', 'nt-9951', 'nt-11312', 'nt-13116', 'nt-77', 'nt-3956', 'nt-5285',
               'nt-5544', 'nt-6596', 'nt-10016', 'nt-10286', 'nt-10933', 'nt-11878', 'nt-11995', 'nt-1174', 'nt-2105',
               'nt-5877', 'nt-6218', 'nt-7646', 'nt-8797', 'nt-9860', 'nt-9945', 'nt-12230', 'nt-12468', 'nt-104',
               'nt-130', 'nt-1515', 'nt-4301', 'nt-4892', 'nt-6779', 'nt-7052', 'nt-7401', 'nt-11526', 'nt-13777',
               'nt-14029', 'nt-6129', 'nt-9177', 'nt-2529', 'nt-5178', 'nt-8541', 'nt-1155', 'nt-1814', 'nt-1930',
               'nt-6058', 'nt-6539', 'nt-8104', 'nt-8418', 'nt-8730', 'nt-9213', 'nt-11150', 'nt-11380', 'nt-13975',
               'nt-1125', 'nt-2507', 'nt-2783', 'nt-4102', 'nt-4935', 'nt-5712', 'nt-5854', 'nt-7663', 'nt-8161',
               'nt-11128', 'nt-12191', 'nt-13042', 'nt-5776', 'nt-5957', 'nt-6397', 'nt-6510', 'nt-10784', 'nt-11273',
               'nt-13670', 'nt-7928', 'nt-3925', 'nt-11387', 'nt-4274', 'nt-5443', 'nt-6582', 'nt-6741', 'nt-8150',
               'nt-8347', 'nt-8802', 'nt-8871', 'nt-9170', 'nt-12451', 'nt-13320', 'nt-656', 'nt-2061', 'nt-2754',
               'nt-3889', 'nt-4484', 'nt-5427', 'nt-6180', 'nt-7626', 'nt-8244', 'nt-13864', 'nt-229', 'nt-934',
               'nt-1060', 'nt-3103', 'nt-5814', 'nt-6542', 'nt-8515', 'nt-10177', 'nt-11773', 'nt-1404', 'nt-3018',
               'nt-8763', 'nt-3214', 'nt-7178', 'nt-7242', 'nt-7321', 'nt-7932', 'nt-8411', 'nt-10006', 'nt-10201',
               'nt-11879', 'nt-12643', 'nt-12714', 'nt-13800', 'nt-154', 'nt-1144', 'nt-2190', 'nt-2902', 'nt-3589',
               'nt-4076', 'nt-9727', 'nt-13104', 'nt-502', 'nt-629', 'nt-950', 'nt-989', 'nt-4152', 'nt-4514',
               'nt-4957', 'nt-5069', 'nt-6460', 'nt-8256', 'nt-8258', 'nt-9380', 'nt-10152', 'nt-11148', 'nt-11517',
               'nt-2209', 'nt-2453', 'nt-5518', 'nt-8448', 'nt-8528', 'nt-8917', 'nt-10887', 'nt-10984', 'nt-12105',
               'nt-13095', 'nt-541', 'nt-2806', 'nt-5399', 'nt-8486', 'nt-10409', 'nt-10966', 'nt-11599', 'nt-13521',
               'nt-13560', 'nt-1389', 'nt-5978', 'nt-9738', 'nt-1885', 'nt-2571', 'nt-4651', 'nt-6635', 'nt-8955',
               'nt-12455', 'nt-12784', 'nt-13573', 'nt-93', 'nt-1457', 'nt-3421', 'nt-5226', 'nt-7679', 'nt-8759',
               'nt-13972', 'nt-2240', 'nt-2636', 'nt-3093', 'nt-7311', 'nt-8570', 'nt-8864', 'nt-13760', 'nt-13842',
               'nt-2552', 'nt-4522', 'nt-4425', 'nt-5416', 'nt-7976', 'nt-12144', 'nt-1839', 'nt-2090', 'nt-2278',
               'nt-3895', 'nt-9058', 'nt-9288', 'nt-10111', 'nt-10174', 'nt-10426', 'nt-11514', 'nt-2710', 'nt-4045',
               'nt-4908', 'nt-10478', 'nt-11381', 'nt-12142', 'nt-13432', 'nt-139', 'nt-4546', 'nt-5421', 'nt-9983',
               'nt-12988', 'nt-13415', 'nt-14037', 'nt-944', 'nt-1856', 'nt-2565', 'nt-2956', 'nt-5804', 'nt-6196',
               'nt-6610', 'nt-8391', 'nt-9285', 'nt-10390', 'nt-11676', 'nt-12978', 'nt-13565', 'nt-13937', 'nt-1103',
               'nt-1337', 'nt-1953', 'nt-4634', 'nt-6475', 'nt-6765', 'nt-9765', 'nt-12413', 'nt-2318', 'nt-4694',
               'nt-6685', 'nt-8733', 'nt-11627', 'nt-13061', 'nt-13909', 'nt-576', 'nt-1639', 'nt-2844', 'nt-3738',
               'nt-3927', 'nt-5067', 'nt-5201', 'nt-6972', 'nt-3330', 'nt-5574', 'nt-7332', 'nt-12842', 'nt-13647',
               'nt-1371', 'nt-1472', 'nt-4181', 'nt-6620', 'nt-10500', 'nt-11643', 'nt-13266', 'nt-209', 'nt-4940',
               'nt-6808', 'nt-7604', 'nt-8819', 'nt-12939', 'nt-729', 'nt-3553', 'nt-7152', 'nt-7808', 'nt-9695',
               'nt-9821', 'nt-13564', 'nt-15', 'nt-1919', 'nt-2661', 'nt-3115', 'nt-3415', 'nt-3439', 'nt-5095',
               'nt-5764', 'nt-8350', 'nt-8649', 'nt-9219', 'nt-10723', 'nt-1320', 'nt-1804', 'nt-2144', 'nt-4510',
               'nt-11573', 'nt-12270', 'nt-12296', 'nt-6366', 'nt-7287', 'nt-8801', 'nt-49', 'nt-3062', 'nt-6483',
               'nt-7592', 'nt-8074', 'nt-11268', 'nt-12126', 'nt-12963', 'nt-500', 'nt-6795', 'nt-11635', 'nt-13186',
               'nt-1040', 'nt-2577', 'nt-2717', 'nt-4953', 'nt-7986', 'nt-8536', 'nt-8627', 'nt-10485', 'nt-11151',
               'nt-13912', 'nt-867', 'nt-2996', 'nt-5799', 'nt-6168', 'nt-7513', 'nt-7742', 'nt-8727', 'nt-13470',
               'nt-2907', 'nt-5665', 'nt-6535', 'nt-12424', 'nt-259', 'nt-3883', 'nt-6352', 'nt-7485', 'nt-10614',
               'nt-11295', 'nt-13805', 'nt-296', 'nt-3471', 'nt-6184', 'nt-7650', 'nt-8756', 'nt-13153', 'nt-4220',
               'nt-7701', 'nt-9566', 'nt-10535', 'nt-10891', 'nt-12132', 'nt-14097', 'nt-1520', 'nt-3547', 'nt-5495',
               'nt-6259', 'nt-9706', 'nt-12463', 'nt-2623', 'nt-3514', 'nt-5990', 'nt-6450', 'nt-8643', 'nt-11007',
               'nt-12609', 'nt-13761', 'nt-13901', 'nt-912', 'nt-1528', 'nt-1782', 'nt-3122', 'nt-4278', 'nt-4975',
               'nt-4993', 'nt-6845', 'nt-7897', 'nt-10859', 'nt-13871', 'nt-1601', 'nt-2241', 'nt-10335', 'nt-10493',
               'nt-13075', 'nt-13701', 'nt-13722', 'nt-13768', 'nt-786', 'nt-1225', 'nt-3096', 'nt-8451', 'nt-13035',
               'nt-14114', 'nt-510', 'nt-565', 'nt-1258', 'nt-2718', 'nt-3257', 'nt-6550', 'nt-8997', 'nt-12317',
               'nt-1171', 'nt-1203', 'nt-4213', 'nt-4925', 'nt-11331', 'nt-41', 'nt-1059', 'nt-2324', 'nt-5171',
               'nt-11826', 'nt-13492', 'nt-13878', 'nt-5318', 'nt-173', 'nt-3783', 'nt-2004', 'nt-13355', 'nt-4592',
               'nt-7367', 'nt-9853', 'nt-10821', 'nt-11325', 'nt-1761', 'nt-3292', 'nt-7024', 'nt-7805', 'nt-7969',
               'nt-8287', 'nt-8491', 'nt-9581', 'nt-11437', 'nt-13146', 'nt-570', 'nt-3473', 'nt-4875', 'nt-6686',
               'nt-7307', 'nt-8572', 'nt-13368', 'nt-13617', 'nt-2221', 'nt-3896', 'nt-11971', 'nt-374', 'nt-387',
               'nt-597', 'nt-2511', 'nt-2586', 'nt-3152', 'nt-3557', 'nt-4101', 'nt-7873', 'nt-8735', 'nt-9397',
               'nt-9620', 'nt-10368', 'nt-14088', 'nt-2731', 'nt-7044', 'nt-8562', 'nt-9808', 'nt-10843', 'nt-13690',
               'nt-3133', 'nt-6583', 'nt-11552', 'nt-4584', 'nt-4901', 'nt-7019', 'nt-13077', 'nt-13277', 'nt-830',
               'nt-4713', 'nt-6206', 'nt-7061', 'nt-11431', 'nt-13929', 'nt-1754', 'nt-2225', 'nt-7114', 'nt-9940',
               'nt-11012', 'nt-13140', 'nt-385', 'nt-5136', 'nt-7101', 'nt-8527', 'nt-8675', 'nt-11494', 'nt-11752',
               'nt-728', 'nt-1819', 'nt-3704', 'nt-5533', 'nt-8030', 'nt-11480', 'nt-236', 'nt-1586', 'nt-4043',
               'nt-4475', 'nt-4521', 'nt-5536', 'nt-5782', 'nt-6519', 'nt-7472', 'nt-8340', 'nt-8354', 'nt-10476',
               'nt-14105', 'nt-807', 'nt-1189', 'nt-1247', 'nt-2164', 'nt-4314', 'nt-6612', 'nt-7458', 'nt-10581',
               'nt-11954', 'nt-13209', 'nt-13454', 'nt-13691', 'nt-1217', 'nt-2591', 'nt-3796', 'nt-6185', 'nt-7937',
               'nt-12113', 'nt-2033', 'nt-3052', 'nt-10670', 'nt-12371', 'nt-13493', 'nt-556', 'nt-2599', 'nt-4674',
               'nt-6598', 'nt-11043', 'nt-2669', 'nt-5816', 'nt-6474', 'nt-624', 'nt-4371', 'nt-5187', 'nt-6125',
               'nt-11074', 'nt-12249', 'nt-1241', 'nt-6306', 'nt-6970', 'nt-7522', 'nt-9615', 'nt-9838', 'nt-2969',
               'nt-8580', 'nt-9894', 'nt-9907', 'nt-13264', 'nt-123', 'nt-2355', 'nt-4942', 'nt-5780', 'nt-6683',
               'nt-6934', 'nt-7968', 'nt-8116', 'nt-8857', 'nt-681', 'nt-875', 'nt-2970', 'nt-3106', 'nt-5165',
               'nt-8014', 'nt-9215', 'nt-13515', 'nt-29', 'nt-3853', 'nt-5328', 'nt-516', 'nt-1105', 'nt-3517',
               'nt-6183', 'nt-13517', 'nt-916', 'nt-6716', 'nt-7080', 'nt-9898', 'nt-10539', 'nt-12420', 'nt-12897',
               'nt-361', 'nt-3319', 'nt-3410', 'nt-8038', 'nt-9179', 'nt-10076', 'nt-11118', 'nt-11765', 'nt-12781',
               'nt-625', 'nt-7188', 'nt-9761', 'nt-2', 'nt-1135', 'nt-2165', 'nt-2690', 'nt-5103', 'nt-5259', 'nt-5390',
               'nt-5863', 'nt-6628', 'nt-6691', 'nt-7263', 'nt-7349', 'nt-9409', 'nt-13483', 'nt-13703', 'nt-3584',
               'nt-4622', 'nt-8849', 'nt-10533', 'nt-604', 'nt-2684', 'nt-2866', 'nt-3044', 'nt-5791', 'nt-8690',
               'nt-9926', 'nt-10359', 'nt-11115', 'nt-990', 'nt-452', 'nt-550', 'nt-3071', 'nt-3491', 'nt-3760',
               'nt-4741', 'nt-6937', 'nt-12162', 'nt-823', 'nt-1260', 'nt-4820', 'nt-5696', 'nt-13343', 'nt-2391',
               'nt-3095', 'nt-9653', 'nt-9701', 'nt-9978', 'nt-10357', 'nt-10868', 'nt-13004', 'nt-242', 'nt-753',
               'nt-942', 'nt-3590', 'nt-4962', 'nt-5376', 'nt-935', 'nt-1165', 'nt-1220', 'nt-12314', 'nt-12866',
               'nt-13857', 'nt-14072', 'nt-14074', 'nt-295', 'nt-2523', 'nt-4015', 'nt-4066', 'nt-4551', 'nt-4600',
               'nt-7245', 'nt-8131', 'nt-11426', 'nt-11591', 'nt-1812', 'nt-2325', 'nt-3461', 'nt-3951', 'nt-9692',
               'nt-10720', 'nt-11363', 'nt-3080', 'nt-3725', 'nt-4921', 'nt-5834', 'nt-9109', 'nt-10548', 'nt-11075',
               'nt-11726', 'nt-12386', 'nt-1354', 'nt-1380', 'nt-1400', 'nt-4055', 'nt-9115', 'nt-9867', 'nt-9971',
               'nt-10297', 'nt-12522', 'nt-12699', 'nt-13444', 'nt-13456', 'nt-3239', 'nt-3670', 'nt-3834', 'nt-5366',
               'nt-5465', 'nt-8905', 'nt-11847', 'nt-13326', 'nt-13349', 'nt-540', 'nt-2474', 'nt-4491', 'nt-5967',
               'nt-7621', 'nt-10651', 'nt-10889', 'nt-11876', 'nt-12229', 'nt-12885', 'nt-1570', 'nt-2447', 'nt-5301',
               'nt-5690', 'nt-10362', 'nt-11175', 'nt-12384', 'nt-13360', 'nt-13582', 'nt-3844', 'nt-5237', 'nt-6957',
               'nt-10110', 'nt-10807', 'nt-8079', 'nt-10625', 'nt-1207', 'nt-2135', 'nt-2750', 'nt-3184', 'nt-5551',
               'nt-9781', 'nt-11066', 'nt-13671', 'nt-1900', 'nt-2411', 'nt-8843', 'nt-9187', 'nt-9843', 'nt-11466',
               'nt-12630', 'nt-86', 'nt-3170', 'nt-4099', 'nt-6473', 'nt-9432', 'nt-11555', 'nt-1531', 'nt-6738',
               'nt-6901', 'nt-6931', 'nt-11539', 'nt-11629', 'nt-13429', 'nt-2146', 'nt-2695', 'nt-3505', 'nt-10946',
               'nt-11135', 'nt-11360', 'nt-4258', 'nt-5959', 'nt-9342', 'nt-12525', 'nt-426', 'nt-1830', 'nt-2784',
               'nt-4293', 'nt-7015', 'nt-10776', 'nt-12393', 'nt-12538', 'nt-2269', 'nt-2910', 'nt-3338', 'nt-4443',
               'nt-4687', 'nt-7597', 'nt-8061', 'nt-10914', 'nt-1544', 'nt-2593', 'nt-4554', 'nt-4578', 'nt-5325',
               'nt-6325', 'nt-11821', 'nt-13006', 'nt-1473', 'nt-4186', 'nt-5057', 'nt-7268', 'nt-9441', 'nt-10790',
               'nt-12761', 'nt-6404', 'nt-13222', 'nt-1133', 'nt-2670', 'nt-5048', 'nt-5705', 'nt-6760', 'nt-11672',
               'nt-11754', 'nt-12462', 'nt-12929', 'nt-13633', 'nt-2706', 'nt-5227', 'nt-5486', 'nt-6219', 'nt-6637',
               'nt-7218', 'nt-9298', 'nt-12001', 'nt-12319', 'nt-8', 'nt-4455', 'nt-7301', 'nt-7416', 'nt-7529',
               'nt-8210', 'nt-10783', 'nt-11230', 'nt-2533', 'nt-2748', 'nt-2789', 'nt-4604', 'nt-7672', 'nt-8559',
               'nt-8697', 'nt-8965', 'nt-9043', 'nt-12602', 'nt-12731', 'nt-13578', 'nt-4673', 'nt-8025', 'nt-8469',
               'nt-10052', 'nt-10804', 'nt-12019', 'nt-12244', 'nt-23', 'nt-4602', 'nt-9968', 'nt-12081', 'nt-2349',
               'nt-5641', 'nt-9710', 'nt-10985', 'nt-11796', 'nt-5064', 'nt-6037', 'nt-448', 'nt-1836', 'nt-2943',
               'nt-11324', 'nt-4597', 'nt-6939', 'nt-7317', 'nt-7654', 'nt-9667', 'nt-12139', 'nt-812', 'nt-6208',
               'nt-7028', 'nt-8534', 'nt-9305', 'nt-3278', 'nt-3866', 'nt-6039', 'nt-8384', 'nt-8433', 'nt-9060',
               'nt-9664', 'nt-10279', 'nt-11121', 'nt-2778', 'nt-4617', 'nt-5962', 'nt-8567', 'nt-8768', 'nt-8999',
               'nt-10421', 'nt-12645', 'nt-1378', 'nt-2361', 'nt-4807', 'nt-12877', 'nt-13138', 'nt-13749', 'nt-13754',
               'nt-14063', 'nt-908', 'nt-1944', 'nt-3613', 'nt-3674', 'nt-7291', 'nt-8422', 'nt-9338', 'nt-9715',
               'nt-13950', 'nt-1523', 'nt-2478', 'nt-10164', 'nt-13394', 'nt-5326', 'nt-5921', 'nt-7194', 'nt-7607',
               'nt-8081', 'nt-8120', 'nt-9842', 'nt-12460', 'nt-13685', 'nt-418', 'nt-2950', 'nt-5634', 'nt-7251',
               'nt-8102', 'nt-10282', 'nt-11603', 'nt-13063', 'nt-13356', 'nt-603', 'nt-3741', 'nt-13436', 'nt-2528',
               'nt-6518', 'nt-7033', 'nt-7680', 'nt-8817', 'nt-10754', 'nt-2082', 'nt-2224', 'nt-8452', 'nt-8907',
               'nt-9188', 'nt-11357', 'nt-11490', 'nt-12364', 'nt-12541', 'nt-497', 'nt-2531', 'nt-2893', 'nt-8241',
               'nt-8415', 'nt-8766', 'nt-10621', 'nt-1161', 'nt-5174', 'nt-7606', 'nt-7661', 'nt-11626', 'nt-13455',
               'nt-14069', 'nt-4317', 'nt-11652', 'nt-9887', 'nt-10495', 'nt-11073', 'nt-12917', 'nt-285', 'nt-998',
               'nt-1800', 'nt-2752', 'nt-3633', 'nt-4075', 'nt-4179', 'nt-5966', 'nt-6884', 'nt-7867', 'nt-2215',
               'nt-3148', 'nt-3973', 'nt-6906', 'nt-7615', 'nt-9063', 'nt-12087', 'nt-13085', 'nt-7765', 'nt-12459',
               'nt-1196', 'nt-3938', 'nt-6471', 'nt-7043', 'nt-8540', 'nt-8865', 'nt-10245', 'nt-12028', 'nt-1199',
               'nt-4117', 'nt-10775', 'nt-13357', 'nt-13920', 'nt-2263', 'nt-3386', 'nt-4944', 'nt-5322', 'nt-8809',
               'nt-9257', 'nt-11684', 'nt-13866', 'nt-1090', 'nt-1226', 'nt-1432', 'nt-2101', 'nt-3259', 'nt-5950',
               'nt-10672', 'nt-11636', 'nt-13351', 'nt-13630', 'nt-890', 'nt-4107', 'nt-5572', 'nt-6694', 'nt-6844',
               'nt-8886', 'nt-10569', 'nt-10991', 'nt-11195', 'nt-11787', 'nt-12241', 'nt-13893', 'nt-4911', 'nt-5431',
               'nt-7645', 'nt-7677', 'nt-10718', 'nt-11275', 'nt-12045', 'nt-12510', 'nt-147', 'nt-705', 'nt-4456',
               'nt-4819', 'nt-7581', 'nt-8075', 'nt-11412', 'nt-12757', 'nt-1615', 'nt-3509', 'nt-5327', 'nt-7173',
               'nt-7269', 'nt-10935', 'nt-11146', 'nt-13836', 'nt-411', 'nt-6625', 'nt-7746', 'nt-8122', 'nt-9529',
               'nt-10081', 'nt-10119', 'nt-11029', 'nt-1339', 'nt-1608', 'nt-2136', 'nt-3380', 'nt-6010', 'nt-7854',
               'nt-9407', 'nt-9919', 'nt-11133', 'nt-13533', 'nt-190', 'nt-1599', 'nt-5446', 'nt-9374', 'nt-9970',
               'nt-11620', 'nt-13571', 'nt-664', 'nt-3188', 'nt-4275', 'nt-4515', 'nt-6354', 'nt-7238', 'nt-10391',
               'nt-12869', 'nt-13128', 'nt-6815', 'nt-7375', 'nt-10256', 'nt-13947', 'nt-390', 'nt-2481', 'nt-5845',
               'nt-8944', 'nt-11621', 'nt-12506', 'nt-162', 'nt-1284', 'nt-2110', 'nt-6329', 'nt-6837', 'nt-8925',
               'nt-11855', 'nt-12674', 'nt-1248', 'nt-3195', 'nt-7649', 'nt-10616', 'nt-12030', 'nt-13373', 'nt-2374',
               'nt-2700', 'nt-3367', 'nt-5235', 'nt-8193', 'nt-9264', 'nt-11782', 'nt-2023', 'nt-2867', 'nt-4208',
               'nt-4725', 'nt-5144', 'nt-7448', 'nt-10695', 'nt-10950', 'nt-13145', 'nt-1045', 'nt-2512', 'nt-4410',
               'nt-8331', 'nt-13922', 'nt-1266', 'nt-2319', 'nt-2921', 'nt-6599', 'nt-7220', 'nt-8752', 'nt-4139',
               'nt-6390', 'nt-6478', 'nt-6733', 'nt-7906', 'nt-8468', 'nt-2787', 'nt-2885', 'nt-4359', 'nt-5749',
               'nt-5812', 'nt-7847', 'nt-9680', 'nt-9730', 'nt-10007', 'nt-11072', 'nt-11253', 'nt-12043', 'nt-13323',
               'nt-1384', 'nt-5736', 'nt-5961', 'nt-10374', 'nt-14', 'nt-284', 'nt-3169', 'nt-3261', 'nt-7018',
               'nt-8573', 'nt-13431', 'nt-114', 'nt-894', 'nt-1537', 'nt-6194', 'nt-6342', 'nt-9555', 'nt-92',
               'nt-1530', 'nt-1573', 'nt-3222', 'nt-4666', 'nt-4704', 'nt-6431', 'nt-6998', 'nt-8485', 'nt-9232',
               'nt-10057', 'nt-11818', 'nt-544', 'nt-4243', 'nt-6338', 'nt-9503', 'nt-11415', 'nt-42', 'nt-129',
               'nt-2270', 'nt-3742', 'nt-4160', 'nt-4462', 'nt-5006', 'nt-4098', 'nt-4222', 'nt-5602', 'nt-12156',
               'nt-12811', 'nt-725', 'nt-2445', 'nt-4564', 'nt-9099', 'nt-10034', 'nt-10483', 'nt-10726', 'nt-12433',
               'nt-6835', 'nt-12490', 'nt-13284', 'nt-1728', 'nt-6544', 'nt-6971', 'nt-7913', 'nt-9010', 'nt-9979',
               'nt-11139', 'nt-13445', 'nt-4143', 'nt-5923', 'nt-7884', 'nt-8526', 'nt-8555', 'nt-701', 'nt-2618',
               'nt-2686', 'nt-5340', 'nt-5981', 'nt-8918', 'nt-10713', 'nt-10795', 'nt-10976', 'nt-11590', 'nt-13553',
               'nt-1257', 'nt-4772', 'nt-5666', 'nt-6216', 'nt-7492', 'nt-9613', 'nt-13032', 'nt-151', 'nt-467',
               'nt-1927', 'nt-2010', 'nt-5535', 'nt-5979', 'nt-6310', 'nt-7103', 'nt-7408', 'nt-8327', 'nt-12336',
               'nt-12941', 'nt-13643', 'nt-206', 'nt-3543', 'nt-4421', 'nt-5386', 'nt-5401', 'nt-5768', 'nt-6308',
               'nt-7122', 'nt-7918', 'nt-9103', 'nt-11413', 'nt-13512', 'nt-72', 'nt-2161', 'nt-4836', 'nt-13278',
               'nt-83', 'nt-1833', 'nt-2689', 'nt-4590', 'nt-5037', 'nt-6191', 'nt-10453', 'nt-11239', 'nt-11916',
               'nt-1416', 'nt-2425', 'nt-2813', 'nt-3809', 'nt-4089', 'nt-4216', 'nt-4360', 'nt-4641', 'nt-5611',
               'nt-9788', 'nt-9789', 'nt-14106', 'nt-1325', 'nt-4056', 'nt-4492', 'nt-4734', 'nt-6384', 'nt-6605',
               'nt-6909', 'nt-7841', 'nt-10025', 'nt-10166', 'nt-10981', 'nt-12561', 'nt-12928', 'nt-2055', 'nt-3015',
               'nt-4858', 'nt-5621', 'nt-11159', 'nt-11682', 'nt-415', 'nt-1605', 'nt-4047', 'nt-4349', 'nt-5157',
               'nt-5307', 'nt-3899', 'nt-7310', 'nt-8634', 'nt-11953', 'nt-4059', 'nt-5305', 'nt-8909', 'nt-12297',
               'nt-3150', 'nt-4026', 'nt-7767', 'nt-7776', 'nt-9705', 'nt-9946', 'nt-10102', 'nt-11969', 'nt-13268',
               'nt-14135', 'nt-4166', 'nt-4970', 'nt-4995', 'nt-7319', 'nt-8164', 'nt-9486', 'nt-9774', 'nt-13078',
               'nt-13304', 'nt-1715', 'nt-1973', 'nt-3350', 'nt-5075', 'nt-5189', 'nt-8480', 'nt-9861', 'nt-11231',
               'nt-12272', 'nt-3698', 'nt-6348', 'nt-6536', 'nt-9463', 'nt-13318', 'nt-13736', 'nt-1823', 'nt-3534',
               'nt-5437', 'nt-7170', 'nt-9579', 'nt-11527', 'nt-13402', 'nt-4365', 'nt-5899', 'nt-7133', 'nt-12430',
               'nt-12921', 'nt-13665', 'nt-871', 'nt-2024', 'nt-2085', 'nt-2856', 'nt-3723', 'nt-5217', 'nt-9746',
               'nt-10850', 'nt-11520', 'nt-12215', 'nt-12501', 'nt-7823', 'nt-9314', 'nt-7211', 'nt-7640', 'nt-12055',
               'nt-164', 'nt-898', 'nt-2888', 'nt-7925', 'nt-9147', 'nt-9384', 'nt-10315', 'nt-11288', 'nt-12289',
               'nt-13798', 'nt-13830', 'nt-1002', 'nt-3033', 'nt-3107', 'nt-3877', 'nt-5011', 'nt-6092', 'nt-8578',
               'nt-12352', 'nt-12582', 'nt-13787', 'nt-902', 'nt-3592', 'nt-4341', 'nt-5015', 'nt-6315', 'nt-8600',
               'nt-10448', 'nt-10756', 'nt-11038', 'nt-11549', 'nt-12871', 'nt-1211', 'nt-3379', 'nt-3758', 'nt-5612',
               'nt-5778', 'nt-6263', 'nt-7006', 'nt-7921', 'nt-10128', 'nt-2536', 'nt-2954', 'nt-4217', 'nt-6726',
               'nt-8729', 'nt-10792', 'nt-11407', 'nt-3079', 'nt-5099', 'nt-5622', 'nt-6653', 'nt-6870', 'nt-11510',
               'nt-12060', 'nt-12855', 'nt-275', 'nt-6232', 'nt-8795', 'nt-9148', 'nt-9297', 'nt-9302', 'nt-9320',
               'nt-10586', 'nt-11755', 'nt-13329', 'nt-6521', 'nt-6809', 'nt-7149', 'nt-7447', 'nt-8196', 'nt-8387',
               'nt-10019', 'nt-294', 'nt-921', 'nt-2906', 'nt-5191', 'nt-5608', 'nt-5697', 'nt-5784', 'nt-6512',
               'nt-7473', 'nt-10360', 'nt-14141', 'nt-8084', 'nt-9513', 'nt-34', 'nt-1004', 'nt-4416', 'nt-6740',
               'nt-7521', 'nt-8981', 'nt-9702', 'nt-11082', 'nt-1588', 'nt-286', 'nt-2327', 'nt-6109', 'nt-6850',
               'nt-7842', 'nt-11633', 'nt-13103', 'nt-2229', 'nt-3636', 'nt-5960', 'nt-8375', 'nt-13082', 'nt-492',
               'nt-691', 'nt-1659', 'nt-1676', 'nt-4086', 'nt-6771', 'nt-12487', 'nt-12743', 'nt-13283', 'nt-13928',
               'nt-457', 'nt-1845', 'nt-3430', 'nt-5490', 'nt-6548', 'nt-10200', 'nt-10508', 'nt-616', 'nt-2158',
               'nt-2435', 'nt-13868', 'nt-854', 'nt-3240', 'nt-3499', 'nt-3820', 'nt-5059', 'nt-5365', 'nt-6015',
               'nt-7403', 'nt-9189', 'nt-6307', 'nt-9203', 'nt-9495', 'nt-11294', 'nt-11307', 'nt-2440', 'nt-10155',
               'nt-11129', 'nt-13047', 'nt-963', 'nt-6927', 'nt-7807', 'nt-9127', 'nt-10428', 'nt-13891', 'nt-2239',
               'nt-3787', 'nt-5258', 'nt-6742', 'nt-7360', 'nt-7524', 'nt-7868', 'nt-8092', 'nt-9118', 'nt-9283',
               'nt-10150', 'nt-486', 'nt-628', 'nt-1310', 'nt-6041', 'nt-7496', 'nt-7929', 'nt-8168', 'nt-10167',
               'nt-2154', 'nt-5555', 'nt-6036', 'nt-8689', 'nt-9324', 'nt-12484', 'nt-4945', 'nt-6078', 'nt-6782',
               'nt-10751', 'nt-11460', 'nt-12926', 'nt-1736', 'nt-5185', 'nt-5907', 'nt-9307', 'nt-10303', 'nt-10563',
               'nt-2930', 'nt-4631', 'nt-5220', 'nt-8000', 'nt-11962', 'nt-13289', 'nt-28', 'nt-1922', 'nt-3600',
               'nt-3649', 'nt-8279', 'nt-9161', 'nt-9368', 'nt-12640', 'nt-1493', 'nt-4751', 'nt-4776', 'nt-4829',
               'nt-10149', 'nt-12615', 'nt-13388', 'nt-4354', 'nt-5018', 'nt-7771', 'nt-12480', 'nt-13619', 'nt-1861',
               'nt-2266', 'nt-2780', 'nt-3695', 'nt-7177', 'nt-7874', 'nt-9033', 'nt-9990', 'nt-10771', 'nt-2839',
               'nt-4387', 'nt-5678', 'nt-7312', 'nt-1278', 'nt-4312', 'nt-5578', 'nt-9204', 'nt-10528', 'nt-11205',
               'nt-1172', 'nt-3810', 'nt-6836', 'nt-9957', 'nt-3445', 'nt-4246', 'nt-4498', 'nt-5688', 'nt-9225',
               'nt-11760', 'nt-12213', 'nt-12684', 'nt-12834', 'nt-1630', 'nt-2558', 'nt-3166', 'nt-5397', 'nt-6523',
               'nt-6890', 'nt-8430', 'nt-10048', 'nt-11252', 'nt-13743', 'nt-1194', 'nt-5984', 'nt-6708', 'nt-7323',
               'nt-8461', 'nt-10724', 'nt-1048', 'nt-7144', 'nt-8718', 'nt-8870', 'nt-9993', 'nt-10132', 'nt-10474',
               'nt-10892', 'nt-13648', 'nt-14050', 'nt-14085', 'nt-155', 'nt-317', 'nt-4663', 'nt-9031', 'nt-9440',
               'nt-12745', 'nt-13714', 'nt-1338', 'nt-2141', 'nt-2863', 'nt-7351', 'nt-8440', 'nt-10060', 'nt-10198',
               'nt-11725', 'nt-12026', 'nt-12817', 'nt-3055', 'nt-5228', 'nt-5702', 'nt-12775', 'nt-3618', 'nt-1074',
               'nt-2341', 'nt-3314', 'nt-6940', 'nt-7810', 'nt-11087', 'nt-12303', 'nt-12706', 'nt-13100', 'nt-13205',
               'nt-6818', 'nt-9050', 'nt-11326', 'nt-12082', 'nt-13389', 'nt-878', 'nt-4781', 'nt-10480', 'nt-980',
               'nt-2820', 'nt-2939', 'nt-5286', 'nt-5657', 'nt-5783', 'nt-6089', 'nt-6412', 'nt-6633', 'nt-9375',
               'nt-9592', 'nt-11263', 'nt-12968', 'nt-282', 'nt-714', 'nt-3753', 'nt-6186', 'nt-7859', 'nt-12715',
               'nt-1047', 'nt-1590', 'nt-5243', 'nt-5250', 'nt-10328', 'nt-11838', 'nt-12529', 'nt-216', 'nt-1046',
               'nt-2177', 'nt-5172', 'nt-8317', 'nt-8520', 'nt-9791', 'nt-10520', 'nt-11586', 'nt-12189', 'nt-12738',
               'nt-465', 'nt-4115', 'nt-4215', 'nt-4283', 'nt-4753', 'nt-7157', 'nt-8585', 'nt-10697', 'nt-11554',
               'nt-785', 'nt-2658', 'nt-3221', 'nt-7373', 'nt-8670', 'nt-9096', 'nt-10587', 'nt-12718', 'nt-13345',
               'nt-97', 'nt-1170', 'nt-5625', 'nt-6157', 'nt-13108', 'nt-14051', 'nt-735', 'nt-3744', 'nt-4433',
               'nt-5541', 'nt-10056', 'nt-1098', 'nt-1306', 'nt-2332', 'nt-2982', 'nt-4927', 'nt-5112', 'nt-6111',
               'nt-6553', 'nt-6762', 'nt-6919', 'nt-11757', 'nt-3341', 'nt-3467', 'nt-4361', 'nt-5025', 'nt-5436',
               'nt-6545', 'nt-8747', 'nt-10313', 'nt-10546', 'nt-11111', 'nt-11274', 'nt-12777', 'nt-675', 'nt-2213',
               'nt-3914', 'nt-4683', 'nt-5400', 'nt-7930', 'nt-10969', 'nt-5852', 'nt-437', 'nt-1008', 'nt-4605',
               'nt-4729', 'nt-7058', 'nt-8602', 'nt-8700', 'nt-10005', 'nt-10314', 'nt-11670', 'nt-3507', 'nt-9327',
               'nt-1667', 'nt-1864', 'nt-4373', 'nt-4795', 'nt-7260', 'nt-7281', 'nt-8337', 'nt-10838', 'nt-10849',
               'nt-1411', 'nt-4295', 'nt-4532', 'nt-6383', 'nt-6503', 'nt-9227', 'nt-10183', 'nt-13361', 'nt-13555',
               'nt-1777', 'nt-2398', 'nt-3385', 'nt-6433', 'nt-9098', 'nt-252', 'nt-3953', 'nt-4536', 'nt-4828',
               'nt-8050', 'nt-10316', 'nt-11519', 'nt-12472', 'nt-12795', 'nt-2367', 'nt-4625', 'nt-4871', 'nt-4923',
               'nt-5703', 'nt-9226', 'nt-11344', 'nt-853', 'nt-1401', 'nt-3147', 'nt-5463', 'nt-6825', 'nt-7068',
               'nt-9478', 'nt-12470', 'nt-14149', 'nt-2337', 'nt-2775', 'nt-4808', 'nt-5999', 'nt-8004', 'nt-9352',
               'nt-11003', 'nt-11358', 'nt-12192', 'nt-13752', 'nt-1972', 'nt-2697', 'nt-3541', 'nt-3928', 'nt-11738',
               'nt-11799', 'nt-12783', 'nt-1817', 'nt-1917', 'nt-10906', 'nt-11297', 'nt-11927', 'nt-12358', 'nt-12958',
               'nt-13098', 'nt-13817', 'nt-13886', 'nt-3266', 'nt-4157', 'nt-4802', 'nt-5100', 'nt-5724', 'nt-7202',
               'nt-10957', 'nt-11046', 'nt-11141', 'nt-971', 'nt-2560', 'nt-3132', 'nt-5004', 'nt-6595', 'nt-7284',
               'nt-8604', 'nt-9815', 'nt-14026', 'nt-727', 'nt-2815', 'nt-10932', 'nt-10989', 'nt-11654', 'nt-12570',
               'nt-928', 'nt-945', 'nt-2781', 'nt-3067', 'nt-5113', 'nt-7318', 'nt-8031', 'nt-8217', 'nt-11097',
               'nt-11493', 'nt-13962', 'nt-2465', 'nt-2712', 'nt-3334', 'nt-5280', 'nt-7184', 'nt-7488', 'nt-8041',
               'nt-11010', 'nt-12195', 'nt-2763', 'nt-2831', 'nt-4192', 'nt-7304', 'nt-7773', 'nt-9747', 'nt-10029',
               'nt-12469', 'nt-316', 'nt-420', 'nt-5952', 'nt-7432', 'nt-8273', 'nt-11482', 'nt-13598', 'nt-6504',
               'nt-10101', 'nt-230', 'nt-501', 'nt-3279', 'nt-5980', 'nt-7574', 'nt-7839', 'nt-11606', 'nt-12220',
               'nt-4939', 'nt-5760', 'nt-5878', 'nt-9039', 'nt-11085', 'nt-5879', 'nt-6853', 'nt-7025', 'nt-11147',
               'nt-11512', 'nt-12576', 'nt-13642', 'nt-13698', 'nt-2125', 'nt-2208', 'nt-3458', 'nt-4211', 'nt-4650',
               'nt-5398', 'nt-7431', 'nt-10225', 'nt-11283', 'nt-12419', 'nt-564', 'nt-2738', 'nt-4534', 'nt-5073',
               'nt-5958', 'nt-8110', 'nt-12591', 'nt-13705', 'nt-1785', 'nt-1945', 'nt-3227', 'nt-3923', 'nt-5295',
               'nt-6423', 'nt-6710', 'nt-7239', 'nt-10469', 'nt-14145', 'nt-113', 'nt-224', 'nt-1727', 'nt-1786',
               'nt-2234', 'nt-4955', 'nt-5391', 'nt-6252', 'nt-8134', 'nt-12457', 'nt-12867', 'nt-1578', 'nt-3643',
               'nt-8633', 'nt-8760', 'nt-11899', 'nt-12080', 'nt-5223', 'nt-8245', 'nt-10622', 'nt-1188', 'nt-1381',
               'nt-5369', 'nt-6099', 'nt-6593', 'nt-6958', 'nt-8143', 'nt-12122', 'nt-12528', 'nt-13599', 'nt-1109',
               'nt-1198', 'nt-1906', 'nt-4419', 'nt-9343', 'nt-9609', 'nt-10881', 'nt-12889', 'nt-12998', 'nt-3811',
               'nt-6002', 'nt-9527', 'nt-10246', 'nt-10764', 'nt-11567', 'nt-13903', 'nt-1991', 'nt-4543', 'nt-4642',
               'nt-5230', 'nt-6871', 'nt-6977', 'nt-10202', 'nt-11031', 'nt-1277', 'nt-3121', 'nt-5382', 'nt-10193',
               'nt-11463', 'nt-1027', 'nt-1892', 'nt-4924', 'nt-6999', 'nt-10877', 'nt-11341', 'nt-13310', 'nt-1495',
               'nt-10863', 'nt-10890', 'nt-400', 'nt-2309', 'nt-3316', 'nt-7700', 'nt-8681', 'nt-10560', 'nt-1250',
               'nt-3904', 'nt-5730', 'nt-7507', 'nt-11452', 'nt-13850', 'nt-13976', 'nt-0', 'nt-3460', 'nt-5408',
               'nt-5847', 'nt-6297', 'nt-7941', 'nt-8117', 'nt-8285', 'nt-8868', 'nt-9490', 'nt-13371', 'nt-409',
               'nt-6974', 'nt-8329', 'nt-12873', 'nt-13315', 'nt-13398', 'nt-100', 'nt-3411', 'nt-3536', 'nt-5800',
               'nt-8492', 'nt-9005', 'nt-480', 'nt-6124', 'nt-10721', 'nt-10680', 'nt-10907', 'nt-11576', 'nt-865',
               'nt-3818', 'nt-7136', 'nt-8185', 'nt-8911', 'nt-12179', 'nt-13143', 'nt-13876', 'nt-1223', 'nt-1714',
               'nt-5341', 'nt-9493', 'nt-11025', 'nt-11054', 'nt-11109', 'nt-5681', 'nt-8591', 'nt-11428', 'nt-1527',
               'nt-3225', 'nt-4422', 'nt-5342', 'nt-5733', 'nt-7879', 'nt-8439', 'nt-9173', 'nt-13328', 'nt-13820',
               'nt-557', 'nt-936', 'nt-2001', 'nt-6438', 'nt-11459', 'nt-11775', 'nt-3005', 'nt-3045', 'nt-4826',
               'nt-5314', 'nt-6576', 'nt-9806', 'nt-9896', 'nt-12919', 'nt-13046', 'nt-189', 'nt-609', 'nt-3383',
               'nt-3531', 'nt-6033', 'nt-7158', 'nt-7875', 'nt-371', 'nt-419', 'nt-1880', 'nt-3022', 'nt-4863',
               'nt-8012', 'nt-10929', 'nt-13199', 'nt-14049', 'nt-231', 'nt-1113', 'nt-1818', 'nt-3204', 'nt-6509',
               'nt-7339', 'nt-7410', 'nt-10901', 'nt-11047', 'nt-12140', 'nt-491', 'nt-5270', 'nt-5773', 'nt-6766',
               'nt-7001', 'nt-7943', 'nt-9272', 'nt-205', 'nt-3146', 'nt-6992', 'nt-9445', 'nt-10262', 'nt-13280',
               'nt-545', 'nt-589', 'nt-2721', 'nt-2937', 'nt-5060', 'nt-6723', 'nt-7445', 'nt-10027', 'nt-10658',
               'nt-11648', 'nt-11663', 'nt-5102', 'nt-5205', 'nt-9332', 'nt-10188', 'nt-10786', 'nt-11800', 'nt-13731',
               'nt-14090', 'nt-12748', 'nt-13894', 'nt-1774', 'nt-5104', 'nt-6012', 'nt-6701', 'nt-7326', 'nt-8311',
               'nt-9311', 'nt-9510', 'nt-11550', 'nt-1106', 'nt-5114', 'nt-6312', 'nt-10599', 'nt-3821', 'nt-4588',
               'nt-6551', 'nt-7300', 'nt-10690', 'nt-5615', 'nt-6505', 'nt-7334', 'nt-7948', 'nt-10675', 'nt-10755',
               'nt-12231', 'nt-1083', 'nt-8803', 'nt-10839', 'nt-11545', 'nt-11610', 'nt-11751', 'nt-14071', 'nt-1058',
               'nt-5709', 'nt-6458', 'nt-9760', 'nt-3721', 'nt-5947', 'nt-7213', 'nt-7591', 'nt-7967', 'nt-8772',
               'nt-12709', 'nt-8034', 'nt-13964', 'nt-1229', 'nt-2180', 'nt-3504', 'nt-3694', 'nt-8767', 'nt-13428',
               'nt-13584', 'nt-13724', 'nt-1627', 'nt-9435', 'nt-1656', 'nt-4123', 'nt-4880', 'nt-6693', 'nt-10255',
               'nt-11704', 'nt-17', 'nt-185', 'nt-11531', 'nt-13447', 'nt-343', 'nt-827', 'nt-2716', 'nt-3375',
               'nt-3806', 'nt-8023', 'nt-8083', 'nt-13079', 'nt-4815', 'nt-6936', 'nt-7561', 'nt-9072', 'nt-10735',
               'nt-379', 'nt-1307', 'nt-3750', 'nt-5406', 'nt-5517', 'nt-8779', 'nt-10304', 'nt-11792', 'nt-13110',
               'nt-563', 'nt-2913', 'nt-3468', 'nt-6063', 'nt-6697', 'nt-8356', 'nt-10405', 'nt-3021', 'nt-4655',
               'nt-6751', 'nt-7911', 'nt-8436', 'nt-12075', 'nt-3644', 'nt-4068', 'nt-8775', 'nt-9793', 'nt-10818',
               'nt-12402', 'nt-12474', 'nt-13060', 'nt-1352', 'nt-3985', 'nt-4424', 'nt-6859', 'nt-11932', 'nt-1784',
               'nt-3684', 'nt-8054', 'nt-8362', 'nt-10370', 'nt-10909', 'nt-11889', 'nt-12911', 'nt-14136', 'nt-146',
               'nt-1506', 'nt-1853', 'nt-1982', 'nt-4034', 'nt-4586', 'nt-7146', 'nt-9554', 'nt-12086', 'nt-1980',
               'nt-4508', 'nt-5915', 'nt-8139', 'nt-12620', 'nt-1453', 'nt-1778', 'nt-2352', 'nt-2673', 'nt-6077',
               'nt-7159', 'nt-7380', 'nt-8680', 'nt-8702', 'nt-8926', 'nt-11728', 'nt-450', 'nt-3465', 'nt-4128',
               'nt-7486', 'nt-10878', 'nt-11034', 'nt-11538', 'nt-251', 'nt-287', 'nt-1467', 'nt-2653', 'nt-3833',
               'nt-5928', 'nt-6339', 'nt-7013', 'nt-10108', 'nt-12641', 'nt-13441', 'nt-13775', 'nt-7799', 'nt-11254',
               'nt-1120', 'nt-3689', 'nt-4261', 'nt-4716', 'nt-6271', 'nt-8379', 'nt-10134', 'nt-10497', 'nt-12408',
               'nt-177', 'nt-8708', 'nt-1713', 'nt-3273', 'nt-3865', 'nt-4972', 'nt-6749', 'nt-7876', 'nt-8360',
               'nt-10742', 'nt-11529', 'nt-12553', 'nt-13910', 'nt-649', 'nt-1195', 'nt-1841', 'nt-4271', 'nt-5169',
               'nt-5787', 'nt-1478', 'nt-7369', 'nt-13693', 'nt-13735', 'nt-1178', 'nt-2149', 'nt-3457', 'nt-5121',
               'nt-10507', 'nt-11088', 'nt-2456', 'nt-3627', 'nt-5501', 'nt-5644', 'nt-9135', 'nt-11841', 'nt-13182',
               'nt-1317', 'nt-2096', 'nt-2389', 'nt-5766', 'nt-9550', 'nt-10620', 'nt-11771', 'nt-1941', 'nt-4327',
               'nt-6413', 'nt-7892', 'nt-9811', 'nt-11032', 'nt-11093', 'nt-13187', 'nt-13505', 'nt-14002', 'nt-5942',
               'nt-7755', 'nt-7414', 'nt-7430', 'nt-10075', 'nt-10660', 'nt-12505', 'nt-39', 'nt-3039', 'nt-5811',
               'nt-8051', 'nt-8408', 'nt-8959', 'nt-13580', 'nt-13640', 'nt-1614', 'nt-5183', 'nt-5693', 'nt-9378',
               'nt-11483', 'nt-1452', 'nt-3199', 'nt-3270', 'nt-3749', 'nt-5896', 'nt-6586', 'nt-6705', 'nt-6875',
               'nt-7022', 'nt-7494', 'nt-48', 'nt-99', 'nt-2185', 'nt-2997', 'nt-3224', 'nt-3564', 'nt-8058',
               'nt-10521', 'nt-915', 'nt-1018', 'nt-1828', 'nt-2273', 'nt-7558', 'nt-8505', 'nt-9784', 'nt-9801',
               'nt-11458', 'nt-12572', 'nt-13171', 'nt-208', 'nt-8535', 'nt-10492', 'nt-11744', 'nt-8013', 'nt-9404',
               'nt-11505', 'nt-12954', 'nt-2003', 'nt-2699', 'nt-3692', 'nt-5482', 'nt-6182', 'nt-8159', 'nt-11694',
               'nt-12992', 'nt-13635', 'nt-3646', 'nt-3697', 'nt-6641', 'nt-8135', 'nt-10271', 'nt-13183', 'nt-2894',
               'nt-3854', 'nt-3874', 'nt-7094', 'nt-13000', 'nt-5158', 'nt-7266', 'nt-11703', 'nt-4426', 'nt-8794',
               'nt-10471', 'nt-13785', 'nt-1091', 'nt-3408', 'nt-5976', 'nt-6814', 'nt-7082', 'nt-7363', 'nt-10639',
               'nt-10885', 'nt-11056', 'nt-11226', 'nt-12411', 'nt-13144', 'nt-281', 'nt-601', 'nt-4537', 'nt-5547',
               'nt-1988', 'nt-9790', 'nt-11873', 'nt-359', 'nt-3880', 'nt-6323', 'nt-7908', 'nt-8477', 'nt-8691',
               'nt-10861', 'nt-11161', 'nt-11385', 'nt-11756', 'nt-13273', 'nt-3087', 'nt-3347', 'nt-4167', 'nt-4328',
               'nt-6127', 'nt-7861', 'nt-8942', 'nt-9961', 'nt-10059', 'nt-10897', 'nt-11978', 'nt-14031', 'nt-14096',
               'nt-117', 'nt-2848', 'nt-3577', 'nt-5464', 'nt-197', 'nt-1498', 'nt-5211', 'nt-10876', 'nt-11560',
               'nt-12794', 'nt-13338', 'nt-13364', 'nt-4739', 'nt-6462', 'nt-9834', 'nt-11429', 'nt-11732', 'nt-12970',
               'nt-869', 'nt-2473', 'nt-2614', 'nt-3335', 'nt-4161', 'nt-8636', 'nt-9277', 'nt-11248', 'nt-13834',
               'nt-13426', 'nt-13663', 'nt-13892', 'nt-1068', 'nt-8510', 'nt-9471', 'nt-10944', 'nt-13071', 'nt-451',
               'nt-1302', 'nt-3051', 'nt-3285', 'nt-5055', 'nt-9198', 'nt-10948', 'nt-12321', 'nt-12421', 'nt-1857',
               'nt-4755', 'nt-7427', 'nt-8202', 'nt-9826', 'nt-10376', 'nt-11302', 'nt-608', 'nt-3466', 'nt-6650',
               'nt-7778', 'nt-8334', 'nt-9286', 'nt-10054', 'nt-10462', 'nt-11862', 'nt-468', 'nt-2116', 'nt-3258',
               'nt-3318', 'nt-6415', 'nt-6640', 'nt-7933', 'nt-9019', 'nt-9067', 'nt-11880', 'nt-12036', 'nt-12212',
               'nt-2132', 'nt-3054', 'nt-4593', 'nt-10121', 'nt-12020', 'nt-12429', 'nt-12785', 'nt-13367', 'nt-5938',
               'nt-7796', 'nt-8376', 'nt-9725', 'nt-10260', 'nt-6675', 'nt-12072', 'nt-265', 'nt-677', 'nt-2114',
               'nt-4402', 'nt-5087', 'nt-5137', 'nt-7822', 'nt-10231', 'nt-12953', 'nt-6932', 'nt-8423', 'nt-9628',
               'nt-11124', 'nt-12423', 'nt-14073', 'nt-949', 'nt-3165', 'nt-5598', 'nt-7365', 'nt-8828', 'nt-9795',
               'nt-12360', 'nt-13386', 'nt-2518', 'nt-2747', 'nt-2991', 'nt-4809', 'nt-9541', 'nt-12286', 'nt-1579',
               'nt-1758', 'nt-3354', 'nt-8426', 'nt-11963', 'nt-13020', 'nt-1396', 'nt-3126', 'nt-3532', 'nt-5707',
               'nt-6097', 'nt-6250', 'nt-7717', 'nt-10159', 'nt-11496', 'nt-11585', 'nt-12975', 'nt-14093', 'nt-218',
               'nt-782', 'nt-1712', 'nt-3048', 'nt-3082', 'nt-7539', 'nt-8456', 'nt-10722', 'nt-11999', 'nt-13409',
               'nt-1265', 'nt-1983', 'nt-8037', 'nt-8516', 'nt-10035', 'nt-6082', 'nt-6577', 'nt-8714', 'nt-11802',
               'nt-13440', 'nt-1177', 'nt-7641', 'nt-8464', 'nt-10562', 'nt-11130', 'nt-382', 'nt-1039', 'nt-1096',
               'nt-5871', 'nt-8694', 'nt-9683', 'nt-11181', 'nt-11249', 'nt-12403', 'nt-131', 'nt-1321', 'nt-5583',
               'nt-11144', 'nt-12305', 'nt-503', 'nt-2128', 'nt-2575', 'nt-4431', 'nt-4786', 'nt-5997', 'nt-6038',
               'nt-6372', 'nt-6410', 'nt-8319', 'nt-10979', 'nt-13245', 'nt-14025', 'nt-2338', 'nt-8361', 'nt-9200',
               'nt-9621', 'nt-11188', 'nt-11446', 'nt-12733', 'nt-13059', 'nt-13742', 'nt-14122', 'nt-3364', 'nt-3824',
               'nt-4985', 'nt-7669', 'nt-11700', 'nt-393', 'nt-1222', 'nt-3681', 'nt-7864', 'nt-8715', 'nt-12111',
               'nt-40', 'nt-433', 'nt-796', 'nt-2087', 'nt-2490', 'nt-7824', 'nt-9690', 'nt-10355', 'nt-12710',
               'nt-13087', 'nt-7370', 'nt-8026', 'nt-9905', 'nt-11107', 'nt-12489', 'nt-5837', 'nt-6286', 'nt-9743',
               'nt-11149', 'nt-11276', 'nt-5010', 'nt-8946', 'nt-8976', 'nt-11122', 'nt-11220', 'nt-13215', 'nt-14055',
               'nt-959', 'nt-1559', 'nt-8882', 'nt-8897', 'nt-13421', 'nt-13849', 'nt-13941', 'nt-1566', 'nt-4282',
               'nt-4860', 'nt-1398', 'nt-2118', 'nt-2326', 'nt-4195', 'nt-5646', 'nt-10133', 'nt-12428', 'nt-13336',
               'nt-3450', 'nt-5943', 'nt-8011', 'nt-494', 'nt-1298', 'nt-2505', 'nt-4570', 'nt-5687', 'nt-8316',
               'nt-9293', 'nt-10204', 'nt-10750', 'nt-13325', 'nt-13657', 'nt-1674', 'nt-5808', 'nt-6436', 'nt-8188',
               'nt-8618', 'nt-10415', 'nt-12182', 'nt-245', 'nt-969', 'nt-3074', 'nt-4290', 'nt-6886', 'nt-9242',
               'nt-3946', 'nt-9004', 'nt-9071', 'nt-9178', 'nt-10893', 'nt-11761', 'nt-12524', 'nt-12662', 'nt-1168',
               'nt-1560', 'nt-2137', 'nt-5769', 'nt-10257', 'nt-11340', 'nt-13484', 'nt-80', 'nt-970', 'nt-1943',
               'nt-7047', 'nt-10967', 'nt-12509', 'nt-13464', 'nt-1175', 'nt-2121', 'nt-7551', 'nt-8394', 'nt-9259',
               'nt-9370', 'nt-11153', 'nt-11353', 'nt-1694', 'nt-3495', 'nt-3942', 'nt-7021', 'nt-7856', 'nt-8106',
               'nt-8234', 'nt-9866', 'nt-11842', 'nt-13158', 'nt-13465', 'nt-1907', 'nt-3540', 'nt-4176', 'nt-4339',
               'nt-5903', 'nt-6149', 'nt-7354', 'nt-8966', 'nt-10530', 'nt-10733', 'nt-11067', 'nt-167', 'nt-3298',
               'nt-3372', 'nt-10949', 'nt-11763', 'nt-1719', 'nt-1826', 'nt-1898', 'nt-6175', 'nt-6849', 'nt-10147',
               'nt-10452', 'nt-10711', 'nt-12150', 'nt-3686', 'nt-3849', 'nt-5106', 'nt-5675', 'nt-9953', 'nt-10195',
               'nt-10283', 'nt-11051', 'nt-12326', 'nt-12677', 'nt-733', 'nt-3109', 'nt-4539', 'nt-4855', 'nt-6399',
               'nt-9962', 'nt-11315', 'nt-4805', 'nt-13392', 'nt-6973', 'nt-8583', 'nt-8608', 'nt-11278', 'nt-12046',
               'nt-13378', 'nt-13566', 'nt-2442', 'nt-3503', 'nt-7935', 'nt-8315', 'nt-9210', 'nt-13721', 'nt-2799',
               'nt-3030', 'nt-4308', 'nt-7404', 'nt-12708', 'nt-13044', 'nt-792', 'nt-1130', 'nt-4060', 'nt-6747',
               'nt-9748', 'nt-10826', 'nt-11582', 'nt-11817', 'nt-12955', 'nt-14067', 'nt-54', 'nt-2424', 'nt-3355',
               'nt-10395', 'nt-10398', 'nt-13097', 'nt-3759', 'nt-3857', 'nt-4372', 'nt-4798', 'nt-5333', 'nt-9646',
               'nt-12287', 'nt-756', 'nt-1454', 'nt-1521', 'nt-2455', 'nt-5714', 'nt-9768', 'nt-10176', 'nt-13488',
               'nt-13764', 'nt-13816', 'nt-2584', 'nt-12098', 'nt-416', 'nt-579', 'nt-3515', 'nt-4369', 'nt-7527',
               'nt-8078', 'nt-9779', 'nt-12104', 'nt-793', 'nt-1760', 'nt-1843', 'nt-9607', 'nt-9822', 'nt-10160',
               'nt-11665', 'nt-13506', 'nt-13877', 'nt-1221', 'nt-2002', 'nt-3185', 'nt-3748', 'nt-3881', 'nt-4109',
               'nt-4850', 'nt-9145', 'nt-9418', 'nt-9847', 'nt-9927', 'nt-11449', 'nt-12237', 'nt-12987', 'nt-1936',
               'nt-2554', 'nt-5293', 'nt-6344', 'nt-6647', 'nt-9965', 'nt-10067', 'nt-10403', 'nt-13093', 'nt-240',
               'nt-1408', 'nt-4525', 'nt-4830', 'nt-7723', 'nt-9543', 'nt-10748', 'nt-13623', 'nt-587', 'nt-2626',
               'nt-8325', 'nt-9551', 'nt-10606', 'nt-11281', 'nt-11705', 'nt-11813', 'nt-4050', 'nt-5565', 'nt-2097',
               'nt-5210', 'nt-5521', 'nt-6004', 'nt-7687', 'nt-8021', 'nt-12936', 'nt-13529', 'nt-404', 'nt-3435',
               'nt-6145', 'nt-6452', 'nt-6754', 'nt-7901', 'nt-7963', 'nt-8068', 'nt-8952', 'nt-9483', 'nt-11095',
               'nt-12405', 'nt-12853', 'nt-1585', 'nt-5186', 'nt-1913', 'nt-6385', 'nt-8251', 'nt-9797', 'nt-10122',
               'nt-11808', 'nt-12935', 'nt-373', 'nt-3352', 'nt-5023', 'nt-7517', 'nt-3494', 'nt-3522', 'nt-3786',
               'nt-4337', 'nt-5115', 'nt-6139', 'nt-6604', 'nt-7252', 'nt-9361', 'nt-9920', 'nt-11857', 'nt-12496',
               'nt-2642', 'nt-3602', 'nt-3609', 'nt-4553', 'nt-7703', 'nt-7850', 'nt-1616', 'nt-1673', 'nt-3729',
               'nt-6730', 'nt-8355', 'nt-10140', 'nt-11561', 'nt-11960', 'nt-13265', 'nt-13366', 'nt-2358', 'nt-2601',
               'nt-3480', 'nt-5542', 'nt-5982', 'nt-6476', 'nt-7647', 'nt-9580', 'nt-11136', 'nt-11720', 'nt-493',
               'nt-4091', 'nt-4303', 'nt-6340', 'nt-7233', 'nt-9299', 'nt-11014', 'nt-12385', 'nt-13637', 'nt-4918',
               'nt-10654', 'nt-11724', 'nt-12171', 'nt-12397', 'nt-12907', 'nt-2887', 'nt-2948', 'nt-6432', 'nt-6986',
               'nt-7257', 'nt-8483', 'nt-8640', 'nt-2732', 'nt-3151', 'nt-3769', 'nt-4415', 'nt-8597', 'nt-11774',
               'nt-14091', 'nt-332', 'nt-1883', 'nt-1939', 'nt-7830', 'nt-8961', 'nt-9313', 'nt-10860', 'nt-11053',
               'nt-12149', 'nt-219', 'nt-470', 'nt-1233', 'nt-1343', 'nt-1574', 'nt-3390', 'nt-8822', 'nt-9506',
               'nt-10158', 'nt-11092', 'nt-11718', 'nt-12153', 'nt-637', 'nt-2133', 'nt-5373', 'nt-7073', 'nt-8270',
               'nt-10404', 'nt-12668', 'nt-13094', 'nt-2310', 'nt-3265', 'nt-6839', 'nt-7322', 'nt-11354', 'nt-11897',
               'nt-11974', 'nt-12577', 'nt-12705', 'nt-2265', 'nt-9290', 'nt-12652', 'nt-1429', 'nt-3569', 'nt-10540',
               'nt-12209', 'nt-13353', 'nt-5065', 'nt-6280', 'nt-6418', 'nt-8412', 'nt-10419', 'nt-11824', 'nt-12830',
               'nt-12943', 'nt-1274', 'nt-2322', 'nt-5200', 'nt-6257', 'nt-6954', 'nt-6968', 'nt-12789', 'nt-3763',
               'nt-9263', 'nt-9326', 'nt-11607', 'nt-11922', 'nt-12562', 'nt-12973', 'nt-13029', 'nt-2640', 'nt-8651',
               'nt-10168', 'nt-217', 'nt-855', 'nt-1173', 'nt-1962', 'nt-7428', 'nt-7896', 'nt-2331', 'nt-2931',
               'nt-7164', 'nt-7491', 'nt-12338', 'nt-12999', 'nt-3673', 'nt-6574', 'nt-7735', 'nt-9514', 'nt-10501',
               'nt-12306', 'nt-3893', 'nt-5247', 'nt-5925', 'nt-8674', 'nt-8723', 'nt-10963', 'nt-13028', 'nt-13884',
               'nt-24', 'nt-1075', 'nt-1484', 'nt-1636', 'nt-6248', 'nt-8266', 'nt-9061', 'nt-9498', 'nt-9556',
               'nt-160', 'nt-1583', 'nt-2345', 'nt-2733', 'nt-5790', 'nt-6520', 'nt-6941', 'nt-7770', 'nt-11310',
               'nt-14011', 'nt-5146', 'nt-7708', 'nt-8816', 'nt-11993', 'nt-12024', 'nt-12228', 'nt-13012', 'nt-14058',
               'nt-2810', 'nt-2922', 'nt-4485', 'nt-7666', 'nt-10151', 'nt-12302', 'nt-12703', 'nt-13526', 'nt-13561',
               'nt-1121', 'nt-1463', 'nt-3397', 'nt-5478', 'nt-7498', 'nt-9987', 'nt-10584', 'nt-11251', 'nt-12613',
               'nt-3679', 'nt-9482', 'nt-9783', 'nt-1294', 'nt-2980', 'nt-4071', 'nt-4180', 'nt-5525', 'nt-8407',
               'nt-8973', 'nt-9870', 'nt-10412', 'nt-13942', 'nt-6204', 'nt-10550', 'nt-12190', 'nt-836', 'nt-2730',
               'nt-4516', 'nt-5244', 'nt-5449', 'nt-6865', 'nt-7046', 'nt-8611', 'nt-9128', 'nt-9196', 'nt-12390',
               'nt-13822', 'nt-13990', 'nt-764', 'nt-2199', 'nt-3137', 'nt-8545', 'nt-9627', 'nt-10808', 'nt-11024',
               'nt-12473', 'nt-1878', 'nt-13463', 'nt-1219', 'nt-1569', 'nt-3705', 'nt-4512', 'nt-6565', 'nt-7466',
               'nt-12333', 'nt-13040', 'nt-13591', 'nt-598', 'nt-1141', 'nt-5273', 'nt-5451', 'nt-5462', 'nt-6084',
               'nt-6102', 'nt-10589', 'nt-11739', 'nt-137', 'nt-1593', 'nt-2734', 'nt-6636', 'nt-6952', 'nt-7286',
               'nt-8641', 'nt-8856', 'nt-9142', 'nt-9817', 'nt-10608', 'nt-11600', 'nt-11945', 'nt-8599', 'nt-10068',
               'nt-11425', 'nt-12683', 'nt-13241', 'nt-103', 'nt-1289', 'nt-2807', 'nt-4173', 'nt-4638', 'nt-5272',
               'nt-5888', 'nt-9883', 'nt-363', 'nt-2493', 'nt-2633', 'nt-4733', 'nt-6732', 'nt-10311', 'nt-11033',
               'nt-196', 'nt-698', 'nt-3768', 'nt-3936', 'nt-4040', 'nt-4767', 'nt-8621', 'nt-9042', 'nt-10496',
               'nt-10848', 'nt-12653', 'nt-1309', 'nt-5413', 'nt-6106', 'nt-8588', 'nt-8921', 'nt-9668', 'nt-11083',
               'nt-11189', 'nt-12909', 'nt-223', 'nt-3436', 'nt-3139', 'nt-5786', 'nt-6258', 'nt-7294', 'nt-8124',
               'nt-9027', 'nt-11419', 'nt-11706', 'nt-3024', 'nt-4691', 'nt-6773', 'nt-6899', 'nt-9500', 'nt-10184',
               'nt-12836', 'nt-12912', 'nt-12977', 'nt-652', 'nt-1053', 'nt-1884', 'nt-5987', 'nt-13449', 'nt-2214',
               'nt-2582', 'nt-2767', 'nt-3530', 'nt-5658', 'nt-2379', 'nt-4482', 'nt-4599', 'nt-5991', 'nt-7958',
               'nt-9398', 'nt-9638', 'nt-9914', 'nt-10741', 'nt-1535', 'nt-2007', 'nt-6094', 'nt-6234', 'nt-6268',
               'nt-7085', 'nt-10552', 'nt-10854', 'nt-10913', 'nt-13081', 'nt-13348', 'nt-4511', 'nt-6023', 'nt-11886',
               'nt-307', 'nt-5626', 'nt-13228', 'nt-1077', 'nt-1668', 'nt-2838', 'nt-3254', 'nt-3743', 'nt-3975',
               'nt-6862', 'nt-7662', 'nt-7840', 'nt-8710', 'nt-10461', 'nt-10972', 'nt-1536', 'nt-6864', 'nt-9586',
               'nt-10138', 'nt-10380', 'nt-11471', 'nt-1022', 'nt-2755', 'nt-3808', 'nt-7578', 'nt-9588', 'nt-10116',
               'nt-10171', 'nt-11401', 'nt-568', 'nt-1126', 'nt-3620', 'nt-6317', 'nt-6377', 'nt-7216', 'nt-7957',
               'nt-1007', 'nt-5889', 'nt-9767', 'nt-13295', 'nt-435', 'nt-1692', 'nt-1999', 'nt-2652', 'nt-3606',
               'nt-4904', 'nt-6028', 'nt-7160', 'nt-8039', 'nt-8841', 'nt-9430', 'nt-10327', 'nt-10642', 'nt-13489',
               'nt-14070', 'nt-2413', 'nt-2597', 'nt-4744', 'nt-6785', 'nt-7566', 'nt-8899', 'nt-10241', 'nt-10407',
               'nt-12276', 'nt-12536', 'nt-381', 'nt-3746', 'nt-8565', 'nt-10433', 'nt-10778', 'nt-11241', 'nt-13362',
               'nt-13667', 'nt-2419', 'nt-4200', 'nt-8289', 'nt-9888', 'nt-1327', 'nt-5239', 'nt-8198', 'nt-12426',
               'nt-9573', 'nt-10851', 'nt-11355', 'nt-4905', 'nt-5231', 'nt-5662', 'nt-7379', 'nt-10998', 'nt-11518',
               'nt-13160', 'nt-3282', 'nt-6148', 'nt-6863', 'nt-13375', 'nt-2027', 'nt-2079', 'nt-2250', 'nt-4678',
               'nt-8048', 'nt-10069', 'nt-10856', 'nt-11919', 'nt-12315', 'nt-14113', 'nt-2228', 'nt-2231', 'nt-5294',
               'nt-5813', 'nt-6201', 'nt-7229', 'nt-7636', 'nt-8300', 'nt-10961', 'nt-12788', 'nt-1318', 'nt-2138',
               'nt-2812', 'nt-4423', 'nt-7117', 'nt-8711', 'nt-9085', 'nt-11338', 'nt-11892', 'nt-13419', 'nt-60',
               'nt-6072', 'nt-6152', 'nt-6674', 'nt-8432', 'nt-10502', 'nt-1262', 'nt-2762', 'nt-3286', 'nt-7374',
               'nt-7675', 'nt-7780', 'nt-8152', 'nt-8194', 'nt-8295', 'nt-13908', 'nt-640', 'nt-2475', 'nt-3780',
               'nt-8249', 'nt-9732', 'nt-12069', 'nt-13641', 'nt-1790', 'nt-4660', 'nt-8778', 'nt-12299', 'nt-3751',
               'nt-5331', 'nt-7999', 'nt-11216', 'nt-1734', 'nt-2617', 'nt-3098', 'nt-6117', 'nt-7090', 'nt-9421',
               'nt-10248', 'nt-10894', 'nt-2842', 'nt-4277', 'nt-10263', 'nt-11753', 'nt-13396', 'nt-13689', 'nt-1928',
               'nt-3803', 'nt-4792', 'nt-5529', 'nt-8238', 'nt-1970', 'nt-10725', 'nt-12636', 'nt-13870', 'nt-13888',
               'nt-4915', 'nt-5150', 'nt-5550', 'nt-6921', 'nt-7175', 'nt-8240', 'nt-13084', 'nt-13683', 'nt-2790',
               'nt-6606', 'nt-7397', 'nt-7451', 'nt-10570', 'nt-11434', 'nt-1160', 'nt-2043', 'nt-2981', 'nt-3407',
               'nt-5402', 'nt-5474', 'nt-6913', 'nt-8833', 'nt-10318', 'nt-10330', 'nt-96', 'nt-1447', 'nt-4999',
               'nt-6287', 'nt-6456', 'nt-7543', 'nt-7849', 'nt-8594', 'nt-9149', 'nt-9569', 'nt-13331', 'nt-1780',
               'nt-3256', 'nt-4230', 'nt-8579', 'nt-768', 'nt-1750', 'nt-2102', 'nt-9365', 'nt-13495', 'nt-14099',
               'nt-111', 'nt-170', 'nt-473', 'nt-542', 'nt-2822', 'nt-2882', 'nt-7422', 'nt-11042', 'nt-11677',
               'nt-2013', 'nt-3986', 'nt-6051', 'nt-6190', 'nt-7097', 'nt-7800', 'nt-101', 'nt-3669', 'nt-9425',
               'nt-9670', 'nt-10444', 'nt-13067', 'nt-838', 'nt-1333', 'nt-1346', 'nt-6171', 'nt-9357', 'nt-9393',
               'nt-11114', 'nt-12670', 'nt-13293', 'nt-730', 'nt-1015', 'nt-3994', 'nt-8088', 'nt-9608', 'nt-472',
               'nt-1341', 'nt-6284', 'nt-7616', 'nt-75', 'nt-2639', 'nt-11221', 'nt-12740', 'nt-13165', 'nt-2051',
               'nt-5571', 'nt-148', 'nt-1158', 'nt-1503', 'nt-2387', 'nt-6615', 'nt-7031', 'nt-8484', 'nt-13569',
               'nt-200', 'nt-1851', 'nt-5655', 'nt-6251', 'nt-9713', 'nt-11417', 'nt-13575', 'nt-13856', 'nt-3163',
               'nt-3900', 'nt-5567', 'nt-10529', 'nt-244', 'nt-1879', 'nt-3872', 'nt-3980', 'nt-4931', 'nt-5919',
               'nt-6767', 'nt-6869', 'nt-7267', 'nt-10558', 'nt-10780', 'nt-12165', 'nt-187', 'nt-328', 'nt-646',
               'nt-1218', 'nt-1914', 'nt-6220', 'nt-7787', 'nt-7872', 'nt-7993', 'nt-9095', 'nt-11925', 'nt-3200',
               'nt-4403', 'nt-5152', 'nt-5765', 'nt-6121', 'nt-7689', 'nt-8090', 'nt-11375', 'nt-12865', 'nt-1413',
               'nt-7985', 'nt-7995', 'nt-9405', 'nt-10701', 'nt-12747', 'nt-13916', 'nt-270', 'nt-3459', 'nt-3548',
               'nt-4459', 'nt-5179', 'nt-7429', 'nt-8149', 'nt-1028', 'nt-2074', 'nt-2742', 'nt-5924', 'nt-4108',
               'nt-7534', 'nt-8226', 'nt-10387', 'nt-10982', 'nt-2182', 'nt-3395', 'nt-4619', 'nt-4779', 'nt-8336',
               'nt-9600', 'nt-10853', 'nt-2095', 'nt-4582', 'nt-4689', 'nt-5519', 'nt-5590', 'nt-7733', 'nt-12291',
               'nt-12373', 'nt-3136', 'nt-4182', 'nt-4825', 'nt-6787', 'nt-8631', 'nt-11119', 'nt-11791', 'nt-12004',
               'nt-8204', 'nt-918', 'nt-2126', 'nt-4174', 'nt-4649', 'nt-5034', 'nt-5459', 'nt-7947', 'nt-13327',
               'nt-243', 'nt-553', 'nt-671', 'nt-5767', 'nt-9637', 'nt-2417', 'nt-5581', 'nt-7107', 'nt-13200',
               'nt-810', 'nt-3235', 'nt-4684', 'nt-7327', 'nt-7982', 'nt-11368', 'nt-12597', 'nt-850', 'nt-3173',
               'nt-3565', 'nt-5850', 'nt-6996', 'nt-8365', 'nt-8769', 'nt-9681', 'nt-11168', 'nt-13839', 'nt-13841',
               'nt-305', 'nt-2983', 'nt-8288', 'nt-13819', 'nt-1297', 'nt-2063', 'nt-2386', 'nt-5494', 'nt-6364',
               'nt-2238', 'nt-6440', 'nt-7972', 'nt-9143', 'nt-10145', 'nt-10867', 'nt-268', 'nt-3520', 'nt-4224',
               'nt-4252', 'nt-6347', 'nt-6793', 'nt-7372', 'nt-7400', 'nt-7563', 'nt-12598', 'nt-2306', 'nt-2353',
               'nt-2545', 'nt-2862', 'nt-3118', 'nt-9885', 'nt-11532', 'nt-11686', 'nt-12862', 'nt-2496', 'nt-3926',
               'nt-5848', 'nt-7567', 'nt-7926', 'nt-9830', 'nt-11659', 'nt-13122', 'nt-13729', 'nt-2804', 'nt-10372',
               'nt-6826', 'nt-7664', 'nt-10774', 'nt-12274', 'nt-12758', 'nt-13467', 'nt-261', 'nt-800', 'nt-1425',
               'nt-4388', 'nt-5127', 'nt-5182', 'nt-5348', 'nt-5389', 'nt-5922', 'nt-8330', 'nt-12518', 'nt-3178',
               'nt-12952', 'nt-13376', 'nt-13480', 'nt-1557', 'nt-11346', 'nt-14075', 'nt-1541', 'nt-2183', 'nt-4561',
               'nt-4667', 'nt-6278', 'nt-8661', 'nt-9021', 'nt-1273', 'nt-2444', 'nt-2467', 'nt-4893', 'nt-12313',
               'nt-12498', 'nt-13152', 'nt-13783', 'nt-14144', 'nt-326', 'nt-594', 'nt-3647', 'nt-6559', 'nt-11384',
               'nt-12581', 'nt-13372', 'nt-2192', 'nt-7219', 'nt-13535', 'nt-2092', 'nt-4385', 'nt-4966', 'nt-5609',
               'nt-10506', 'nt-10954', 'nt-11844', 'nt-877', 'nt-938', 'nt-3075', 'nt-5387', 'nt-6763', 'nt-9185',
               'nt-10987', 'nt-2858', 'nt-6499', 'nt-9262', 'nt-10352', 'nt-11192', 'nt-12290', 'nt-13608', 'nt-14079',
               'nt-3135', 'nt-3712', 'nt-5192', 'nt-5916', 'nt-7016', 'nt-8005', 'nt-11309', 'nt-12617', 'nt-12905',
               'nt-994', 'nt-4468', 'nt-5502', 'nt-8912', 'nt-9504', 'nt-10664', 'nt-12088', 'nt-884', 'nt-2124',
               'nt-7452', 'nt-9029', 'nt-10575', 'nt-11931', 'nt-12368', 'nt-14015', 'nt-62', 'nt-1837', 'nt-2629',
               'nt-4457', 'nt-8392', 'nt-10882', 'nt-11783', 'nt-13172', 'nt-1909', 'nt-3612', 'nt-6113', 'nt-7598',
               'nt-10295', 'nt-10816', 'nt-11304', 'nt-12254', 'nt-1920', 'nt-2595', 'nt-2786', 'nt-3690', 'nt-12387',
               'nt-13352', 'nt-13949', 'nt-3112', 'nt-6729', 'nt-2974', 'nt-5355', 'nt-9018', 'nt-9078', 'nt-12844',
               'nt-13966', 'nt-14045', 'nt-1887', 'nt-3023', 'nt-3251', 'nt-5735', 'nt-6428', 'nt-10918', 'nt-12886',
               'nt-13244', 'nt-5614', 'nt-10978', 'nt-11257', 'nt-11938', 'nt-256', 'nt-3358', 'nt-3545', 'nt-4136',
               'nt-4440', 'nt-9626', 'nt-634', 'nt-12200', 'nt-1143', 'nt-1717', 'nt-2351', 'nt-3734', 'nt-3856',
               'nt-5038', 'nt-5717', 'nt-6652', 'nt-9066', 'nt-13587', 'nt-3475', 'nt-6143', 'nt-9023', 'nt-13370',
               'nt-6707', 'nt-13786', 'nt-276', 'nt-731', 'nt-3429', 'nt-3632', 'nt-4454', 'nt-6362', 'nt-7237',
               'nt-8302', 'nt-12337', 'nt-2380', 'nt-10661', 'nt-12676', 'nt-12812', 'nt-26', 'nt-1200', 'nt-2089',
               'nt-2524', 'nt-3637', 'nt-4413', 'nt-5086', 'nt-10761', 'nt-13435', 'nt-13959', 'nt-1385', 'nt-4097',
               'nt-6267', 'nt-6780', 'nt-7832', 'nt-9371', 'nt-1424', 'nt-4082', 'nt-5213', 'nt-5718', 'nt-5756',
               'nt-6613', 'nt-7265', 'nt-8574', 'nt-9278', 'nt-11352', 'nt-13662', 'nt-13818', 'nt-14048', 'nt-1854',
               'nt-4577', 'nt-5723', 'nt-5864', 'nt-10154', 'nt-11224', 'nt-11651', 'nt-12599', 'nt-13875', 'nt-958',
               'nt-2008', 'nt-5806', 'nt-9992', 'nt-11235', 'nt-1224', 'nt-1793', 'nt-2068', 'nt-2388', 'nt-5579',
               'nt-6275', 'nt-11767', 'nt-1142', 'nt-2150', 'nt-2651', 'nt-4437', 'nt-5414', 'nt-8121', 'nt-9939',
               'nt-10447', 'nt-11362', 'nt-13397', 'nt-341', 'nt-1628', 'nt-2714', 'nt-3091', 'nt-3274', 'nt-4799',
               'nt-9696', 'nt-11780', 'nt-12224', 'nt-12860', 'nt-14061', 'nt-4190', 'nt-5374', 'nt-7648', 'nt-8703',
               'nt-12196', 'nt-12339', 'nt-12672', 'nt-12903', 'nt-1376', 'nt-1842', 'nt-3855', 'nt-5892', 'nt-8218',
               'nt-12678', 'nt-479', 'nt-864', 'nt-1079', 'nt-1461', 'nt-1571', 'nt-7737', 'nt-7980', 'nt-9120',
               'nt-9174', 'nt-9548', 'nt-9704', 'nt-12535', 'nt-12982', 'nt-14060', 'nt-2032', 'nt-3403', 'nt-5561',
               'nt-5708', 'nt-8920', 'nt-9123', 'nt-9632', 'nt-9923', 'nt-10416', 'nt-2979', 'nt-3323', 'nt-1208',
               'nt-1230', 'nt-1268', 'nt-2148', 'nt-2677', 'nt-7605', 'nt-10209', 'nt-12483', 'nt-13236', 'nt-4376',
               'nt-5418', 'nt-6009', 'nt-9526', 'nt-10534', 'nt-14012', 'nt-771', 'nt-3233', 'nt-5043', 'nt-5552',
               'nt-9933', 'nt-10512', 'nt-11628', 'nt-12914', 'nt-13281', 'nt-8916', 'nt-10346', 'nt-10872', 'nt-11260',
               'nt-13238', 'nt-1322', 'nt-2869', 'nt-4276', 'nt-4310', 'nt-4731', 'nt-7686', 'nt-8348', 'nt-9546',
               'nt-9582', 'nt-10417', 'nt-11565', 'nt-9195', 'nt-274', 'nt-1393', 'nt-2397', 'nt-2876', 'nt-3591',
               'nt-8610', 'nt-9560', 'nt-12070', 'nt-4320', 'nt-4630', 'nt-7916', 'nt-13276', 'nt-474', 'nt-2175',
               'nt-4406', 'nt-6100', 'nt-6662', 'nt-7037', 'nt-573', 'nt-1011', 'nt-1150', 'nt-2539', 'nt-3193',
               'nt-4464', 'nt-8490', 'nt-8607', 'nt-9192', 'nt-11285', 'nt-12828', 'nt-4949', 'nt-11809', 'nt-12746',
               'nt-13595', 'nt-1038', 'nt-1716', 'nt-3924', 'nt-6987', 'nt-8216', 'nt-13686', 'nt-1270', 'nt-1657',
               'nt-1770', 'nt-3010', 'nt-5107', 'nt-6904', 'nt-7092', 'nt-8059', 'nt-11298', 'nt-12840', 'nt-618',
               'nt-4959', 'nt-7396', 'nt-9331', 'nt-55', 'nt-1801', 'nt-5139', 'nt-6935', 'nt-7499', 'nt-9561',
               'nt-9950', 'nt-12047', 'nt-13494', 'nt-2112', 'nt-2510', 'nt-4870', 'nt-5742', 'nt-10765', 'nt-12763',
               'nt-1101', 'nt-1641', 'nt-2768', 'nt-3571', 'nt-5920', 'nt-8320', 'nt-9675', 'nt-10706', 'nt-12065',
               'nt-13570', 'nt-6847', 'nt-7720', 'nt-7438', 'nt-8986', 'nt-9659', 'nt-9677', 'nt-10333', 'nt-12167',
               'nt-12574', 'nt-12780', 'nt-4145', 'nt-5118', 'nt-5135', 'nt-6617', 'nt-7852', 'nt-2460', 'nt-5266',
               'nt-5820', 'nt-7821', 'nt-8860', 'nt-10505', 'nt-10862', 'nt-12273', 'nt-13080', 'nt-2836', 'nt-3488',
               'nt-4298', 'nt-5613', 'nt-6798', 'nt-7106', 'nt-8077', 'nt-8576', 'nt-8704', 'nt-11712', 'nt-13137',
               'nt-13520', 'nt-13854', 'nt-82', 'nt-648', 'nt-2167', 'nt-3593', 'nt-5851', 'nt-6585', 'nt-8421',
               'nt-10561', 'nt-12944', 'nt-13725', 'nt-1533', 'nt-5026', 'nt-5701', 'nt-9014', 'nt-12846', 'nt-13898',
               'nt-1611', 'nt-3622', 'nt-4411', 'nt-5224', 'nt-5263', 'nt-5417', 'nt-6873', 'nt-6950', 'nt-10875',
               'nt-13112', 'nt-13317', 'nt-797', 'nt-2873', 'nt-4752', 'nt-6245', 'nt-9054', 'nt-12160', 'nt-13568',
               'nt-704', 'nt-2961', 'nt-2989', 'nt-4887', 'nt-5606', 'nt-8678', 'nt-10083', 'nt-11859', 'nt-12688',
               'nt-13943', 'nt-5695', 'nt-744', 'nt-1290', 'nt-3961', 'nt-4489', 'nt-4668', 'nt-4964', 'nt-7055',
               'nt-7214', 'nt-12012', 'nt-12379', 'nt-13479', 'nt-509', 'nt-3238', 'nt-3511', 'nt-9237', 'nt-11049',
               'nt-12097', 'nt-13675', 'nt-1003', 'nt-3449', 'nt-8190', 'nt-9379', 'nt-9383', 'nt-9422', 'nt-11968',
               'nt-591', 'nt-611', 'nt-2535', 'nt-3713', 'nt-5485', 'nt-6895', 'nt-8333', 'nt-842', 'nt-2000',
               'nt-2405', 'nt-2663', 'nt-4917', 'nt-9416', 'nt-10481', 'nt-10927', 'nt-11182', 'nt-12727', 'nt-13185',
               'nt-833', 'nt-1647', 'nt-3078', 'nt-4669', 'nt-11127', 'nt-12680', 'nt-12700', 'nt-13030', 'nt-1296',
               'nt-3244', 'nt-4728', 'nt-4879', 'nt-8006', 'nt-13616', 'nt-109', 'nt-6291', 'nt-6405', 'nt-10191',
               'nt-11361', 'nt-11611', 'nt-11736', 'nt-12092', 'nt-12974', 'nt-973', 'nt-2630', 'nt-4756', 'nt-5674',
               'nt-7163', 'nt-7806', 'nt-12704', 'nt-207', 'nt-1696', 'nt-3223', 'nt-3474', 'nt-4151', 'nt-5062',
               'nt-5974', 'nt-10571', 'nt-12174', 'nt-12', 'nt-202', 'nt-967', 'nt-5624', 'nt-5684', 'nt-6095',
               'nt-7038', 'nt-8045', 'nt-9868', 'nt-2606', 'nt-3550', 'nt-6603', 'nt-9515', 'nt-9663', 'nt-11080',
               'nt-11330', 'nt-2905', 'nt-6888', 'nt-8140', 'nt-9759', 'nt-14128', 'nt-19', 'nt-156', 'nt-1187',
               'nt-1723', 'nt-3361', 'nt-3869', 'nt-7544', 'nt-8647', 'nt-9139', 'nt-12193', 'nt-322', 'nt-981',
               'nt-1358', 'nt-4609', 'nt-4749', 'nt-8547', 'nt-8679', 'nt-13057', 'nt-1272', 'nt-1369', 'nt-3293',
               'nt-4500', 'nt-4932', 'nt-5199', 'nt-7758', 'nt-8248', 'nt-9426', 'nt-11207', 'nt-12650', 'nt-13801',
               'nt-2390', 'nt-5140', 'nt-7378', 'nt-7658', 'nt-8962', 'nt-9882', 'nt-12147', 'nt-234', 'nt-751',
               'nt-4002', 'nt-7726', 'nt-12013', 'nt-12764', 'nt-9292', 'nt-12008', 'nt-997', 'nt-4501', 'nt-8071',
               'nt-9396', 'nt-10008', 'nt-10250', 'nt-13516', 'nt-14118', 'nt-6336', 'nt-8435', 'nt-12816', 'nt-1918',
               'nt-12644', 'nt-12946', 'nt-2153', 'nt-4735', 'nt-5283', 'nt-9030', 'nt-9056', 'nt-10296', 'nt-10737',
               'nt-11219', 'nt-283', 'nt-2279', 'nt-2846', 'nt-3011', 'nt-4171', 'nt-6624', 'nt-8673', 'nt-10953',
               'nt-10320', 'nt-51', 'nt-1355', 'nt-2735', 'nt-3800', 'nt-6463', 'nt-798', 'nt-835', 'nt-3027',
               'nt-11804', 'nt-1665', 'nt-4266', 'nt-5092', 'nt-8044', 'nt-13393', 'nt-13766', 'nt-839', 'nt-2173',
               'nt-2761', 'nt-3312', 'nt-5454', 'nt-13615', 'nt-4245', 'nt-4757', 'nt-5072', 'nt-5439', 'nt-6330',
               'nt-8716', 'nt-9006', 'nt-9602', 'nt-11502', 'nt-175', 'nt-1844', 'nt-3562', 'nt-8737', 'nt-10179',
               'nt-10324', 'nt-11069', 'nt-11991', 'nt-13045', 'nt-13374', 'nt-13646', 'nt-21', 'nt-734', 'nt-3654',
               'nt-4021', 'nt-7236', 'nt-9651', 'nt-11564', 'nt-11788', 'nt-868', 'nt-1409', 'nt-3610', 'nt-5126',
               'nt-6422', 'nt-6538', 'nt-6725', 'nt-6953', 'nt-8975', 'nt-10340', 'nt-11571', 'nt-13121', 'nt-2289',
               'nt-3387', 'nt-3391', 'nt-4228', 'nt-7870', 'nt-8533', 'nt-13548', 'nt-1145', 'nt-3388', 'nt-4882',
               'nt-5825', 'nt-13500', 'nt-1690', 'nt-2134', 'nt-3485', 'nt-6543', 'nt-8657', 'nt-10904', 'nt-13090',
               'nt-2103', 'nt-2495', 'nt-3483', 'nt-5737', 'nt-6067', 'nt-9954', 'nt-11204', 'nt-11300', 'nt-12077',
               'nt-12686', 'nt-13437', 'nt-13989', 'nt-638', 'nt-1575', 'nt-4095', 'nt-5232', 'nt-6737', 'nt-8725',
               'nt-8746', 'nt-9489', 'nt-10610', 'nt-11173', 'nt-12061', 'nt-12879', 'nt-12847', 'nt-643', 'nt-1902',
               'nt-2466', 'nt-2814', 'nt-4727', 'nt-8782', 'nt-2621', 'nt-2788', 'nt-5042', 'nt-5338', 'nt-5429',
               'nt-6811', 'nt-7387', 'nt-10135', 'nt-462', 'nt-523', 'nt-1108', 'nt-6351', 'nt-9480', 'nt-11491',
               'nt-11882', 'nt-3076', 'nt-12099', 'nt-13900', 'nt-12843', 'nt-886', 'nt-2880', 'nt-5891', 'nt-6816',
               'nt-7064', 'nt-8222', 'nt-1052', 'nt-1190', 'nt-1480', 'nt-3424', 'nt-4535', 'nt-4789', 'nt-7209',
               'nt-9100', 'nt-9828', 'nt-11409', 'nt-1638', 'nt-1438', 'nt-2765', 'nt-3785', 'nt-3867', 'nt-6468',
               'nt-6908', 'nt-7953', 'nt-9111', 'nt-376', 'nt-759', 'nt-3731', 'nt-9243', 'nt-10631', 'nt-12000',
               'nt-12508', 'nt-12723', 'nt-948', 'nt-2301', 'nt-2685', 'nt-4184', 'nt-7962', 'nt-10010', 'nt-10070',
               'nt-11515', 'nt-12374', 'nt-13287', 'nt-1062', 'nt-2604', 'nt-4645', 'nt-7034', 'nt-7357', 'nt-8247',
               'nt-9354', 'nt-12214', 'nt-12729', 'nt-13658', 'nt-13664', 'nt-14137', 'nt-2184', 'nt-2632', 'nt-3097',
               'nt-3693', 'nt-4280', 'nt-4330', 'nt-5248', 'nt-6488', 'nt-8807', 'nt-10704', 'nt-11631', 'nt-8066',
               'nt-8873', 'nt-6088', 'nt-8052', 'nt-10674', 'nt-12482', 'nt-13050', 'nt-13291', 'nt-13767', 'nt-403',
               'nt-4083', 'nt-4116', 'nt-6639', 'nt-8757', 'nt-9820', 'nt-10574', 'nt-12543', 'nt-13319', 'nt-13644',
               'nt-983', 'nt-1603', 'nt-2285', 'nt-2454', 'nt-11810', 'nt-12357', 'nt-3295', 'nt-6266', 'nt-9216',
               'nt-10014', 'nt-10194', 'nt-10782', 'nt-12300', 'nt-12993', 'nt-13120', 'nt-13737', 'nt-822', 'nt-5713',
               'nt-10459', 'nt-2612', 'nt-5393', 'nt-7010', 'nt-7475', 'nt-8344', 'nt-11299', 'nt-11605', 'nt-14068',
               'nt-4606', 'nt-8100', 'nt-9136', 'nt-9619', 'nt-10061', 'nt-11450', 'nt-906', 'nt-3807', 'nt-8996',
               'nt-9947', 'nt-10730', 'nt-1949', 'nt-6983', 'nt-8861', 'nt-10968', 'nt-12478', 'nt-13296', 'nt-1303',
               'nt-2477', 'nt-3440', 'nt-3870', 'nt-4444', 'nt-6618', 'nt-12868', 'nt-3912', 'nt-5287', 'nt-7573',
               'nt-7970', 'nt-8382', 'nt-6172', 'nt-7169', 'nt-8351', 'nt-9929', 'nt-11909', 'nt-12633', 'nt-348',
               'nt-5549', 'nt-5592', 'nt-8127', 'nt-9310', 'nt-9712', 'nt-10827', 'nt-12765', 'nt-12856', 'nt-13609',
               'nt-13973', 'nt-141', 'nt-182', 'nt-226', 'nt-8339', 'nt-11856', 'nt-1967', 'nt-3012', 'nt-816',
               'nt-2916', 'nt-3574', 'nt-6607', 'nt-7775', 'nt-10249', 'nt-1305', 'nt-10109', 'nt-2936', 'nt-4196',
               'nt-4844', 'nt-7838', 'nt-8838', 'nt-9631', 'nt-3814', 'nt-5988', 'nt-8985', 'nt-10543', 'nt-11287',
               'nt-11473', 'nt-12388', 'nt-247', 'nt-4899', 'nt-7282', 'nt-8701', 'nt-9334', 'nt-11120', 'nt-13124',
               'nt-13154', 'nt-13499', 'nt-2945', 'nt-9138', 'nt-9593', 'nt-11282', 'nt-13390', 'nt-533', 'nt-2302',
               'nt-2470', 'nt-4219', 'nt-6249', 'nt-8698', 'nt-9234', 'nt-13385', 'nt-13666', 'nt-13803', 'nt-4106',
               'nt-5074', 'nt-5349', 'nt-5599', 'nt-5995', 'nt-8265', 'nt-8290', 'nt-9124', 'nt-9184', 'nt-10590',
               'nt-372', 'nt-401', 'nt-3901', 'nt-6843', 'nt-6989', 'nt-9698', 'nt-12275', 'nt-12375', 'nt-711',
               'nt-4309', 'nt-5642', 'nt-8687', 'nt-8887', 'nt-9476', 'nt-9903', 'nt-10812', 'nt-11845', 'nt-13848',
               'nt-1311', 'nt-1613', 'nt-3601', 'nt-4004', 'nt-10321', 'nt-11001', 'nt-11286', 'nt-13469', 'nt-337',
               'nt-4706', 'nt-4742', 'nt-8419', 'nt-9685', 'nt-10406', 'nt-10793', 'nt-11157', 'nt-13408', 'nt-428',
               'nt-2333', 'nt-6272', 'nt-6541', 'nt-13155', 'nt-14059', 'nt-2058', 'nt-2892', 'nt-3305', 'nt-5448',
               'nt-7739', 'nt-10745', 'nt-11623', 'nt-7794', 'nt-3037', 'nt-3394', 'nt-3653', 'nt-6389', 'nt-6651',
               'nt-6881', 'nt-7208', 'nt-12642', 'nt-298', 'nt-1134', 'nt-1940', 'nt-2262', 'nt-2383', 'nt-2547',
               'nt-3324', 'nt-5722', 'nt-6677', 'nt-7804', 'nt-10965', 'nt-406', 'nt-3563', 'nt-5175', 'nt-6743',
               'nt-12564', 'nt-2570', 'nt-3381', 'nt-4158', 'nt-4552', 'nt-4943', 'nt-7148', 'nt-13572', 'nt-732',
               'nt-1581', 'nt-3119', 'nt-3802', 'nt-3930', 'nt-5445', 'nt-8409', 'nt-9511', 'nt-10454', 'nt-12262',
               'nt-1332', 'nt-2615', 'nt-2949', 'nt-10213', 'nt-10437', 'nt-10450', 'nt-14077', 'nt-1235', 'nt-1971',
               'nt-3890', 'nt-5821', 'nt-8592', 'nt-9450', 'nt-11000', 'nt-11697', 'nt-330', 'nt-2143', 'nt-3342',
               'nt-4708', 'nt-5677', 'nt-7437', 'nt-12565', 'nt-4105', 'nt-6687', 'nt-11305', 'nt-3245', 'nt-7699',
               'nt-143', 'nt-225', 'nt-2219', 'nt-2990', 'nt-3561', 'nt-6363', 'nt-7743', 'nt-10682', 'nt-12175',
               'nt-794', 'nt-1769', 'nt-2107', 'nt-2771', 'nt-3764', 'nt-7127', 'nt-7201', 'nt-9909', 'nt-10077',
               'nt-4478', 'nt-5745', 'nt-7344', 'nt-7489', 'nt-7877', 'nt-9729', 'nt-10484', 'nt-11190', 'nt-12808',
               'nt-13181', 'nt-13994', 'nt-2168', 'nt-5587', 'nt-6498', 'nt-10689', 'nt-10693', 'nt-12486', 'nt-1866',
               'nt-2471', 'nt-10597', 'nt-384', 'nt-2928', 'nt-3215', 'nt-10169', 'nt-2682', 'nt-7793', 'nt-9291',
               'nt-10941', 'nt-5246', 'nt-6680', 'nt-13175', 'nt-2172', 'nt-3065', 'nt-3446', 'nt-5672', 'nt-7851',
               'nt-8923', 'nt-9376', 'nt-9538', 'nt-10143', 'nt-10275', 'nt-13747', 'nt-739', 'nt-7468', 'nt-10308',
               'nt-11271', 'nt-13466', 'nt-13592', 'nt-4304', 'nt-4994', 'nt-8915', 'nt-10504', 'nt-11457', 'nt-11543',
               'nt-11645', 'nt-11965', 'nt-13434', 'nt-6053', 'nt-8189', 'nt-13011', 'nt-2769', 'nt-4771', 'nt-5977',
               'nt-7246', 'nt-8531', 'nt-10465', 'nt-13527', 'nt-13869', 'nt-370', 'nt-996', 'nt-4294', 'nt-4517',
               'nt-5351', 'nt-6756', 'nt-8062', 'nt-10898', 'nt-12932', 'nt-13501', 'nt-128', 'nt-526', 'nt-952',
               'nt-1417', 'nt-2538', 'nt-3983', 'nt-4048', 'nt-6091', 'nt-7577', 'nt-8036', 'nt-8529', 'nt-9436',
               'nt-9539', 'nt-10288', 'nt-12702', 'nt-122', 'nt-4326', 'nt-6466', 'nt-6684', 'nt-9009', 'nt-10239',
               'nt-14054', 'nt-12532', 'nt-1643', 'nt-1895', 'nt-1989', 'nt-6799', 'nt-7405', 'nt-8457', 'nt-10212',
               'nt-11101', 'nt-454', 'nt-1197', 'nt-2311', 'nt-6910', 'nt-6961', 'nt-7296', 'nt-7585', 'nt-13715',
               'nt-6943', 'nt-12216', 'nt-572', 'nt-5890', 'nt-6978', 'nt-8706', 'nt-9656', 'nt-10557', 'nt-13586',
               'nt-8112', 'nt-1099', 'nt-5946', 'nt-6479', 'nt-7193', 'nt-9055', 'nt-9165', 'nt-9461', 'nt-12038',
               'nt-13168', 'nt-3142', 'nt-4335', 'nt-9645', 'nt-10811', 'nt-13858', 'nt-1979', 'nt-2947', 'nt-3164',
               'nt-3756', 'nt-5667', 'nt-6622', 'nt-9230', 'nt-841', 'nt-1136', 'nt-2541', 'nt-4465', 'nt-4754',
               'nt-5505', 'nt-7273', 'nt-9164', 'nt-12023', 'nt-13231', 'nt-460', 'nt-3167', 'nt-3382', 'nt-4017',
               'nt-4897', 'nt-6085', 'nt-10106', 'nt-10236', 'nt-12446', 'nt-13132', 'nt-443', 'nt-3677', 'nt-4810',
               'nt-5726', 'nt-7912', 'nt-3506', 'nt-3873', 'nt-6649', 'nt-6900', 'nt-6916', 'nt-11592', 'nt-13379',
               'nt-7668', 'nt-7954', 'nt-9622', 'nt-10205', 'nt-11145', 'nt-13036', 'nt-13314', 'nt-16', 'nt-263',
               'nt-2255', 'nt-6663', 'nt-6823', 'nt-10833', 'nt-12605', 'nt-13604', 'nt-300', 'nt-1029', 'nt-3442',
               'nt-11105', 'nt-11944', 'nt-12218', 'nt-13227', 'nt-4947', 'nt-6461', 'nt-7540', 'nt-10602', 'nt-11318',
               'nt-483', 'nt-2127', 'nt-3099', 'nt-4922', 'nt-6678', 'nt-8370', 'nt-9316', 'nt-9669', 'nt-3645',
               'nt-4881', 'nt-5826', 'nt-7651', 'nt-10470', 'nt-11264', 'nt-11975', 'nt-12059', 'nt-4679', 'nt-5566',
               'nt-5632', 'nt-5880', 'nt-6333', 'nt-7140', 'nt-8056', 'nt-13302', 'nt-787', 'nt-4092', 'nt-4556',
               'nt-5636', 'nt-6430', 'nt-7347', 'nt-7705', 'nt-8998', 'nt-11934', 'nt-12819', 'nt-14133', 'nt-352',
               'nt-966', 'nt-4188', 'nt-6980', 'nt-9223', 'nt-9382', 'nt-9803', 'nt-9889', 'nt-12519', 'nt-13443',
               'nt-344', 'nt-1908', 'nt-5430', 'nt-8130', 'nt-8663', 'nt-8949', 'nt-10956', 'nt-11622', 'nt-11839',
               'nt-2522', 'nt-5426', 'nt-5857', 'nt-7546', 'nt-11158', 'nt-11546', 'nt-1695', 'nt-2676', 'nt-6065',
               'nt-8374', 'nt-9427', 'nt-12732', 'nt-30', 'nt-70', 'nt-1263', 'nt-2573', 'nt-3232', 'nt-3431',
               'nt-4344', 'nt-5447', 'nt-5762', 'nt-6163', 'nt-7798', 'nt-10934', 'nt-11874', 'nt-13881', 'nt-1867',
               'nt-5433', 'nt-5792', 'nt-954', 'nt-1959', 'nt-3529', 'nt-3706', 'nt-4910', 'nt-6584', 'nt-8831',
               'nt-9317', 'nt-13113', 'nt-6794', 'nt-12102', 'nt-13821', 'nt-791', 'nt-1147', 'nt-2257', 'nt-5596',
               'nt-6643', 'nt-9402', 'nt-9614', 'nt-11871', 'nt-12861', 'nt-14146', 'nt-1014', 'nt-3202', 'nt-5694',
               'nt-7829', 'nt-10137', 'nt-10736', 'nt-11734', 'nt-12117', 'nt-12948', 'nt-13292', 'nt-6629', 'nt-1534',
               'nt-1709', 'nt-2824', 'nt-2978', 'nt-3077', 'nt-3083', 'nt-7657', 'nt-9013', 'nt-10566', 'nt-11708',
               'nt-9505', 'nt-12888', 'nt-13676', 'nt-2304', 'nt-9598', 'nt-289', 'nt-2081', 'nt-2678', 'nt-4113',
               'nt-6030', 'nt-11914', 'nt-12151', 'nt-599', 'nt-1631', 'nt-3064', 'nt-4839', 'nt-7066', 'nt-8147',
               'nt-9034', 'nt-12942', 'nt-13341', 'nt-9980', 'nt-813', 'nt-3451', 'nt-3635', 'nt-3860', 'nt-4873',
               'nt-5568', 'nt-9141', 'nt-10455', 'nt-10503', 'nt-11259', 'nt-13136', 'nt-7014', 'nt-9724', 'nt-9787',
               'nt-12504', 'nt-12972', 'nt-2046', 'nt-4598', 'nt-5772', 'nt-7600', 'nt-7965', 'nt-8385', 'nt-9132',
               'nt-10828', 'nt-11898', 'nt-12901', 'nt-1169', 'nt-1513', 'nt-2660', 'nt-5883', 'nt-6346', 'nt-8832',
               'nt-9494', 'nt-11233', 'nt-11982', 'nt-12687', 'nt-14043', 'nt-979', 'nt-2665', 'nt-3343', 'nt-3368',
               'nt-4483', 'nt-5354', 'nt-13107', 'nt-13626', 'nt-14140', 'nt-84', 'nt-3414', 'nt-8228', 'nt-9146',
               'nt-11266', 'nt-11584', 'nt-11727', 'nt-12437', 'nt-13212', 'nt-4453', 'nt-7035', 'nt-11827', 'nt-12937',
               'nt-13692', 'nt-63', 'nt-5081', 'nt-5527', 'nt-6494', 'nt-7469', 'nt-10752', 'nt-12546', 'nt-13223',
               'nt-14036', 'nt-3230', 'nt-4001', 'nt-4846', 'nt-5156', 'nt-6690', 'nt-10216', 'nt-10600', 'nt-13681',
               'nt-13969', 'nt-706', 'nt-7977', 'nt-8646', 'nt-12245', 'nt-12730', 'nt-3177', 'nt-4374', 'nt-5654',
               'nt-6017', 'nt-8449', 'nt-8987', 'nt-10299', 'nt-11071', 'nt-12447', 'nt-12638', 'nt-2362', 'nt-2843',
               'nt-4503', 'nt-9657', 'nt-12282', 'nt-12327', 'nt-12381', 'nt-378', 'nt-4255', 'nt-7118', 'nt-7487',
               'nt-7814', 'nt-9594', 'nt-917', 'nt-1834', 'nt-3089', 'nt-3409', 'nt-3757', 'nt-5802', 'nt-6253',
               'nt-7454', 'nt-14100', 'nt-4207', 'nt-9858', 'nt-10090', 'nt-10627', 'nt-12904', 'nt-12924', 'nt-781',
               'nt-2830', 'nt-7072', 'nt-7744', 'nt-9844', 'nt-12143', 'nt-12158', 'nt-12631', 'nt-13300', 'nt-1554',
               'nt-1708', 'nt-7176', 'nt-7376', 'nt-7564', 'nt-7601', 'nt-11050', 'nt-11979', 'nt-12603', 'nt-13218',
               'nt-3977', 'nt-7625', 'nt-10996', 'nt-694', 'nt-4793', 'nt-4977', 'nt-6212', 'nt-2286', 'nt-2461',
               'nt-3128', 'nt-3384', 'nt-5096', 'nt-5425', 'nt-8980', 'nt-12243', 'nt-818', 'nt-3913', 'nt-7764',
               'nt-8366', 'nt-8774', 'nt-8914', 'nt-13659', 'nt-765', 'nt-2288', 'nt-3426', 'nt-4778', 'nt-5971',
               'nt-6896', 'nt-8854', 'nt-9988', 'nt-12832', 'nt-12835', 'nt-12895', 'nt-13208', 'nt-4316', 'nt-6443',
               'nt-6570', 'nt-10899', 'nt-12735', 'nt-13439', 'nt-4249', 'nt-5797', 'nt-7192', 'nt-11423', 'nt-13254',
               'nt-13260', 'nt-13475', 'nt-3056', 'nt-3778', 'nt-5537', 'nt-7886', 'nt-8560', 'nt-10740', 'nt-12066',
               'nt-12093', 'nt-13305', 'nt-1969', 'nt-6966', 'nt-7660', 'nt-2264', 'nt-5758', 'nt-7760', 'nt-9155',
               'nt-9485', 'nt-9758', 'nt-12759', 'nt-13860', 'nt-13895', 'nt-762', 'nt-2334', 'nt-3211', 'nt-7497',
               'nt-8683', 'nt-10593', 'nt-12014', 'nt-12477', 'nt-12990', 'nt-2156', 'nt-2971', 'nt-3356', 'nt-4988',
               'nt-5593', 'nt-6827', 'nt-6903', 'nt-8032', 'nt-10065', 'nt-11477', 'nt-12766', 'nt-2777', 'nt-5316',
               'nt-13694', 'nt-3999', 'nt-5405', 'nt-6151', 'nt-12896', 'nt-13118', 'nt-13459', 'nt-13468', 'nt-840',
               'nt-1942', 'nt-10342', 'nt-11328', 'nt-13041', 'nt-13650', 'nt-13759', 'nt-2999', 'nt-4856', 'nt-4912',
               'nt-6591', 'nt-7030', 'nt-8866', 'nt-10388', 'nt-11186', 'nt-5379', 'nt-5660', 'nt-5939', 'nt-6772',
               'nt-11785', 'nt-1081', 'nt-1383', 'nt-2827', 'nt-4042', 'nt-6623', 'nt-9372', 'nt-10663', 'nt-11730',
               'nt-12499', 'nt-14110', 'nt-1852', 'nt-3655', 'nt-4362', 'nt-6666', 'nt-7081', 'nt-7125', 'nt-8040',
               'nt-9340', 'nt-10192', 'nt-12412', 'nt-1156', 'nt-2759', 'nt-5394', 'nt-5941', 'nt-11646', 'nt-13131',
               'nt-712', 'nt-1652', 'nt-5253', 'nt-6600', 'nt-8008', 'nt-8511', 'nt-8885', 'nt-10940', 'nt-12334',
               'nt-14007', 'nt-3804', 'nt-4130', 'nt-7314', 'nt-8989', 'nt-9037', 'nt-9736', 'nt-9949', 'nt-11319',
               'nt-11906', 'nt-12651', 'nt-13704', 'nt-159', 'nt-534', 'nt-1394', 'nt-1477', 'nt-3567', 'nt-12772',
               'nt-848', 'nt-1286', 'nt-2588', 'nt-3737', 'nt-4647', 'nt-5487', 'nt-9641', 'nt-9', 'nt-1283', 'nt-4750',
               'nt-7134', 'nt-7730', 'nt-8659', 'nt-12017', 'nt-12277', 'nt-12925', 'nt-115', 'nt-313', 'nt-2587',
               'nt-4480', 'nt-4591', 'nt-5377', 'nt-5460', 'nt-7444', 'nt-7665', 'nt-9682', 'nt-10888', 'nt-12404',
               'nt-3268', 'nt-3313', 'nt-3735', 'nt-4350', 'nt-5508', 'nt-6265', 'nt-8879', 'nt-11805', 'nt-12039',
               'nt-13233', 'nt-626', 'nt-1282', 'nt-1990', 'nt-3172', 'nt-3852', 'nt-5428', 'nt-6047', 'nt-6246',
               'nt-6365', 'nt-10799', 'nt-11213', 'nt-11223', 'nt-12691', 'nt-4234', 'nt-7303', 'nt-1299', 'nt-3120',
               'nt-7206', 'nt-9635', 'nt-12010', 'nt-12096', 'nt-13791', 'nt-653', 'nt-1860', 'nt-5138', 'nt-5254',
               'nt-8175', 'nt-8253', 'nt-8664', 'nt-9108', 'nt-10717', 'nt-588', 'nt-3955', 'nt-9839', 'nt-10129',
               'nt-11327', 'nt-11465', 'nt-14035', 'nt-3349', 'nt-9679', 'nt-12256', 'nt-769', 'nt-3935', 'nt-10802',
               'nt-13610', 'nt-293', 'nt-879', 'nt-7420', 'nt-7441', 'nt-8125', 'nt-8417', 'nt-10028', 'nt-11947',
               'nt-13627', 'nt-481', 'nt-715', 'nt-3660', 'nt-5233', 'nt-6425', 'nt-6728', 'nt-7003', 'nt-7138',
               'nt-8830', 'nt-9869', 'nt-11548', 'nt-12722', 'nt-13607', 'nt-13958', 'nt-335', 'nt-1240', 'nt-1576',
               'nt-5218', 'nt-5841', 'nt-7476', 'nt-12288', 'nt-1596', 'nt-3598', 'nt-3995', 'nt-4104', 'nt-5131',
               'nt-5212', 'nt-6123', 'nt-7096', 'nt-9140', 'nt-10896', 'nt-11547', 'nt-707', 'nt-900', 'nt-4049',
               'nt-5155', 'nt-5740', 'nt-12050', 'nt-1186', 'nt-3129', 'nt-3974', 'nt-12194', 'nt-874', 'nt-1492',
               'nt-7168', 'nt-10633', 'nt-12557', 'nt-5267', 'nt-8342', 'nt-8881', 'nt-10235', 'nt-10617', 'nt-11950',
               'nt-12185', 'nt-13382', 'nt-2230', 'nt-5329', 'nt-7745', 'nt-9159', 'nt-10510', 'nt-2059', 'nt-5403',
               'nt-9984', 'nt-10032', 'nt-10787', 'nt-5904', 'nt-8672', 'nt-10013', 'nt-3937', 'nt-9518', 'nt-11819',
               'nt-13178', 'nt-1772', 'nt-3155', 'nt-4436', 'nt-7569', 'nt-9363', 'nt-10542', 'nt-11390', 'nt-11748',
               'nt-11933', 'nt-3823', 'nt-12372', 'nt-13226', 'nt-178', 'nt-1368', 'nt-5793', 'nt-10568', 'nt-11721',
               'nt-12467', 'nt-13639', 'nt-168', 'nt-2557', 'nt-3357', 'nt-6376', 'nt-7754', 'nt-9194', 'nt-9304',
               'nt-12021', 'nt-3886', 'nt-4338', 'nt-7619', 'nt-10073', 'nt-12269', 'nt-12502', 'nt-613', 'nt-1164',
               'nt-2368', 'nt-3911', 'nt-3939', 'nt-6714', 'nt-6892', 'nt-8420', 'nt-11061', 'nt-12464', 'nt-2157',
               'nt-13961', 'nt-14127', 'nt-235', 'nt-888', 'nt-2098', 'nt-4079', 'nt-6032', 'nt-12284', 'nt-12852',
               'nt-13001', 'nt-5105', 'nt-5242', 'nt-5520', 'nt-9328', 'nt-12362', 'nt-12389', 'nt-696', 'nt-4769',
               'nt-5045', 'nt-8498', 'nt-8577', 'nt-9456', 'nt-11410', 'nt-12671', 'nt-13007', 'nt-13999', 'nt-5450',
               'nt-9025', 'nt-12728', 'nt-866', 'nt-1228', 'nt-4321', 'nt-6840', 'nt-9936', 'nt-9986', 'nt-10524',
               'nt-11462', 'nt-11992', 'nt-12782', 'nt-13406', 'nt-253', 'nt-843', 'nt-4315', 'nt-8282', 'nt-9335',
               'nt-10292', 'nt-1810', 'nt-2457', 'nt-2764', 'nt-4548', 'nt-5865', 'nt-6147', 'nt-7129', 'nt-8455',
               'nt-10915', 'nt-11811', 'nt-2443', 'nt-5669', 'nt-6501', 'nt-12005', 'nt-12986', 'nt-14014', 'nt-667',
               'nt-1421', 'nt-5252', 'nt-9850', 'nt-920', 'nt-5130', 'nt-5236', 'nt-5692', 'nt-6075', 'nt-7151',
               'nt-11154', 'nt-12252', 'nt-12985', 'nt-1016', 'nt-2123', 'nt-4218', 'nt-4242', 'nt-4914', 'nt-7900',
               'nt-8744', 'nt-9636', 'nt-10351', 'nt-2549', 'nt-2977', 'nt-7212', 'nt-8620', 'nt-11983', 'nt-13631',
               'nt-1061', 'nt-1118', 'nt-2006', 'nt-4223', 'nt-4840', 'nt-6381', 'nt-7120', 'nt-9665', 'nt-13945',
               'nt-13957', 'nt-986', 'nt-1184', 'nt-5630', 'nt-8438', 'nt-11996', 'nt-13010', 'nt-2797', 'nt-2865',
               'nt-3014', 'nt-4225', 'nt-4418', 'nt-9069', 'nt-9684', 'nt-10173', 'nt-10653', 'nt-804', 'nt-4134',
               'nt-5876', 'nt-6928', 'nt-7835', 'nt-8089', 'nt-9253', 'nt-10012', 'nt-10423', 'nt-2715', 'nt-3',
               'nt-1330', 'nt-5506', 'nt-8845', 'nt-10652', 'nt-12454', 'nt-12527', 'nt-13606', 'nt-64', 'nt-2514',
               'nt-4740', 'nt-6221', 'nt-7331', 'nt-7509', 'nt-10464', 'nt-10339', 'nt-10884', 'nt-11717', 'nt-12530',
               'nt-14095', 'nt-1023', 'nt-2821', 'nt-3663', 'nt-3672', 'nt-7395', 'nt-9901', 'nt-11126', 'nt-2631',
               'nt-3311', 'nt-7204', 'nt-11291', 'nt-277', 'nt-2357', 'nt-4859', 'nt-5591', 'nt-9130', 'nt-10117',
               'nt-12916', 'nt-13257', 'nt-1847', 'nt-3131', 'nt-5035', 'nt-8558', 'nt-10855', 'nt-11367', 'nt-11830',
               'nt-11884', 'nt-13838', 'nt-520', 'nt-1704', 'nt-2166', 'nt-2792', 'nt-3906', 'nt-4549', 'nt-5303',
               'nt-9247', 'nt-11504', 'nt-717', 'nt-849', 'nt-4299', 'nt-10037', 'nt-10285', 'nt-10845', 'nt-10857',
               'nt-10962', 'nt-11770', 'nt-1464', 'nt-2879', 'nt-5255', 'nt-6233', 'nt-8502', 'nt-9804', 'nt-10399',
               'nt-13931', 'nt-672', 'nt-6682', 'nt-98', 'nt-1666', 'nt-3848', 'nt-6856', 'nt-8676', 'nt-9052',
               'nt-3322', 'nt-4256', 'nt-8983', 'nt-9381', 'nt-10038', 'nt-11424', 'nt-6277', 'nt-6891', 'nt-7853',
               'nt-8305', 'nt-10306', 'nt-13780', 'nt-688', 'nt-8073', 'nt-8840', 'nt-2139', 'nt-2875', 'nt-1140',
               'nt-7182', 'nt-7820', 'nt-8561', 'nt-8935', 'nt-9102', 'nt-10141', 'nt-11062', 'nt-4214', 'nt-5835',
               'nt-7788', 'nt-10544', 'nt-946', 'nt-2667', 'nt-3041', 'nt-9413', 'nt-9703', 'nt-10960', 'nt-11137',
               'nt-14016', 'nt-1392', 'nt-2645', 'nt-5805', 'nt-7983', 'nt-8811', 'nt-9268', 'nt-11064', 'nt-11199',
               'nt-297', 'nt-687', 'nt-3328', 'nt-4692', 'nt-6391', 'nt-8154', 'nt-10626', 'nt-3969', 'nt-4434',
               'nt-5545', 'nt-6979', 'nt-10573', 'nt-11250', 'nt-12234', 'nt-1261', 'nt-1978', 'nt-6829', 'nt-8069',
               'nt-1519', 'nt-2421', 'nt-4603', 'nt-4946', 'nt-6897', 'nt-6962', 'nt-7276', 'nt-10939', 'nt-13917',
               'nt-4724', 'nt-6409', 'nt-7440', 'nt-9059', 'nt-11065', 'nt-12431', 'nt-1490', 'nt-4120', 'nt-5240',
               'nt-5859', 'nt-7433', 'nt-7579', 'nt-9819', 'nt-1137', 'nt-10402', 'nt-13412', 'nt-2579', 'nt-3685',
               'nt-4618', 'nt-5838', 'nt-10211', 'nt-2292', 'nt-4194', 'nt-5524', 'nt-5755', 'nt-5934', 'nt-8544',
               'nt-9462', 'nt-12600', 'nt-12839', 'nt-13197', 'nt-1496', 'nt-2564', 'nt-6011', 'nt-6532', 'nt-12164',
               'nt-5584', 'nt-5975', 'nt-6838', 'nt-7584', 'nt-7845', 'nt-8814', 'nt-9417', 'nt-9469', 'nt-10397',
               'nt-11946', 'nt-12159', 'nt-12647', 'nt-10640', 'nt-2845', 'nt-7532', 'nt-11232', 'nt-13420', 'nt-882',
               'nt-1251', 'nt-2702', 'nt-3401', 'nt-5320', 'nt-6547', 'nt-7382', 'nt-10323', 'nt-10509', 'nt-5526',
               'nt-8314', 'nt-13333', 'nt-13679', 'nt-2917', 'nt-3028', 'nt-3344', 'nt-6144', 'nt-6356', 'nt-7460',
               'nt-7465', 'nt-8257', 'nt-9224', 'nt-9355', 'nt-11194', 'nt-12266', 'nt-1110', 'nt-1934', 'nt-2071',
               'nt-2794', 'nt-8297', 'nt-13322', 'nt-2370', 'nt-7860', 'nt-8260', 'nt-11619', 'nt-12858', 'nt-802',
               'nt-1364', 'nt-1516', 'nt-1622', 'nt-7135', 'nt-8016', 'nt-10064', 'nt-1947', 'nt-6507', 'nt-6852',
               'nt-8029', 'nt-9429', 'nt-1132', 'nt-3981', 'nt-4135', 'nt-11422', 'nt-12587', 'nt-12767', 'nt-3272',
               'nt-3464', 'nt-4902', 'nt-7603', 'nt-8441', 'nt-8786', 'nt-8972', 'nt-10396', 'nt-12128', 'nt-14006',
               'nt-985', 'nt-1664', 'nt-1732', 'nt-4041', 'nt-10227', 'nt-11303', 'nt-11351', 'nt-12239', 'nt-14107',
               'nt-127', 'nt-204', 'nt-1012', 'nt-1755', 'nt-3576', 'nt-7693', 'nt-10079', 'nt-11937', 'nt-12176',
               'nt-12367', 'nt-13219', 'nt-3997', 'nt-4495', 'nt-8310', 'nt-8740', 'nt-9003', 'nt-9256', 'nt-9611',
               'nt-10383', 'nt-13259', 'nt-3984', 'nt-4569', 'nt-5061', 'nt-7226', 'nt-7474', 'nt-9596', 'nt-10208',
               'nt-11142', 'nt-11650', 'nt-12015', 'nt-73', 'nt-5040', 'nt-7753', 'nt-9877', 'nt-927', 'nt-2901',
               'nt-3508', 'nt-4272', 'nt-7827', 'nt-11246', 'nt-12063', 'nt-861', 'nt-3707', 'nt-4720', 'nt-6770',
               'nt-6831', 'nt-7614', 'nt-7951', 'nt-10951', 'nt-11084', 'nt-1216', 'nt-5124', 'nt-10830', 'nt-13987',
               'nt-1862', 'nt-7084', 'nt-9825', 'nt-10669', 'nt-11400', 'nt-11574', 'nt-2585', 'nt-4393', 'nt-5300',
               'nt-7464', 'nt-9967', 'nt-10022', 'nt-11430', 'nt-12580', 'nt-2548', 'nt-2935', 'nt-4052', 'nt-5748',
               'nt-6153', 'nt-6500', 'nt-9241', 'nt-9252', 'nt-9392', 'nt-12058', 'nt-487', 'nt-1252', 'nt-2472',
               'nt-9400', 'nt-11444', 'nt-2829', 'nt-3032', 'nt-3642', 'nt-4709', 'nt-5052', 'nt-7126', 'nt-7586',
               'nt-9035', 'nt-9466', 'nt-9671', 'nt-11467', 'nt-11507', 'nt-11649', 'nt-11822', 'nt-12805', 'nt-13772',
               'nt-2515', 'nt-2581', 'nt-4070', 'nt-5395', 'nt-8322', 'nt-10301', 'nt-90', 'nt-1326', 'nt-6049',
               'nt-6602', 'nt-12037', 'nt-12837', 'nt-5432', 'nt-595', 'nt-1543', 'nt-3934', 'nt-6235', 'nt-7889',
               'nt-12116', 'nt-12497', 'nt-3149', 'nt-3262', 'nt-3299', 'nt-5279', 'nt-7449', 'nt-7470', 'nt-12711',
               'nt-12813', 'nt-581', 'nt-7306', 'nt-7816', 'nt-10051', 'nt-11052', 'nt-13874', 'nt-3512', 'nt-6211',
               'nt-6667', 'nt-7172', 'nt-7949', 'nt-8859', 'nt-13133', 'nt-13811', 'nt-758', 'nt-3496', 'nt-4581',
               'nt-6160', 'nt-7844', 'nt-8764', 'nt-14132', 'nt-2041', 'nt-2534', 'nt-3042', 'nt-4386', 'nt-7436',
               'nt-9395', 'nt-10928', 'nt-302', 'nt-1436', 'nt-1663', 'nt-2294', 'nt-3845', 'nt-4784', 'nt-4848',
               'nt-9211', 'nt-10185', 'nt-10280', 'nt-14124', 'nt-37', 'nt-108', 'nt-2909', 'nt-6925', 'nt-8108',
               'nt-8301', 'nt-10538', 'nt-5900', 'nt-5953', 'nt-6568', 'nt-7008', 'nt-8186', 'nt-9676', 'nt-11356',
               'nt-1849', 'nt-3444', 'nt-3897', 'nt-4342', 'nt-5996', 'nt-7293', 'nt-11895', 'nt-12479', 'nt-4069',
               'nt-7104', 'nt-10112', 'nt-10591', 'nt-10942', 'nt-12692', 'nt-1580', 'nt-1722', 'nt-2688', 'nt-5149',
               'nt-5600', 'nt-6902', 'nt-7124', 'nt-7223', 'nt-8167', 'nt-9807', 'nt-11711', 'nt-2484', 'nt-2859',
               'nt-3652', 'nt-4368', 'nt-4721', 'nt-5604', 'nt-9329', 'nt-10124', 'nt-12172', 'nt-2160', 'nt-2402',
               'nt-4682', 'nt-4780', 'nt-6907', 'nt-7920', 'nt-1256', 'nt-4127', 'nt-5193', 'nt-5897', 'nt-11618',
               'nt-2272', 'nt-4726', 'nt-5022', 'nt-7797', 'nt-11414', 'nt-13271', 'nt-14102', 'nt-4528', 'nt-5000',
               'nt-5116', 'nt-6214', 'nt-7572', 'nt-195', 'nt-957', 'nt-6429', 'nt-7100', 'nt-8057', 'nt-8349',
               'nt-519', 'nt-2235', 'nt-2819', 'nt-3174', 'nt-4509', 'nt-8815', 'nt-9565', 'nt-11513', 'nt-11981',
               'nt-11989', 'nt-13603', 'nt-7295', 'nt-8475', 'nt-8971', 'nt-9634', 'nt-11559', 'nt-11778', 'nt-723',
               'nt-2437', 'nt-9077', 'nt-10093', 'nt-11640', 'nt-1951', 'nt-4132', 'nt-4463', 'nt-6868', 'nt-6889',
               'nt-8783', 'nt-9776', 'nt-10382', 'nt-10679', 'nt-67', 'nt-1901', 'nt-4027', 'nt-5905', 'nt-8758',
               'nt-9366', 'nt-13574', 'nt-5543', 'nt-6396', 'nt-8460', 'nt-12967', 'nt-13770', 'nt-3469', 'nt-3556',
               'nt-3822', 'nt-5856', 'nt-6305', 'nt-6563', 'nt-6564', 'nt-7280', 'nt-12707', 'nt-13911', 'nt-1958',
               'nt-2020', 'nt-5476', 'nt-5827', 'nt-6357', 'nt-6688', 'nt-12350', 'nt-1996', 'nt-4701', 'nt-5562',
               'nt-6098', 'nt-6300', 'nt-6945', 'nt-7984', 'nt-12161', 'nt-13680', 'nt-407', 'nt-722', 'nt-820',
               'nt-851', 'nt-5024', 'nt-5134', 'nt-6880', 'nt-14030', 'nt-3267', 'nt-2430', 'nt-4665', 'nt-4969',
               'nt-5271', 'nt-6626', 'nt-1043', 'nt-4061', 'nt-7009', 'nt-11941', 'nt-11964', 'nt-12466', 'nt-2287',
               'nt-2729', 'nt-4747', 'nt-5570', 'nt-6727', 'nt-7099', 'nt-10219', 'nt-10438', 'nt-12110', 'nt-13433',
               'nt-2198', 'nt-3008', 'nt-5036', 'nt-5173', 'nt-7514', 'nt-8501', 'nt-11997', 'nt-580', 'nt-1655',
               'nt-1699', 'nt-7786', 'nt-10424', 'nt-10644', 'nt-66', 'nt-257', 'nt-750', 'nt-1455', 'nt-2335',
               'nt-3229', 'nt-5993', 'nt-6043', 'nt-11335', 'nt-495', 'nt-2976', 'nt-8442', 'nt-8941', 'nt-9832',
               'nt-12240', 'nt-12804', 'nt-13298', 'nt-13727', 'nt-340', 'nt-2359', 'nt-2381', 'nt-3716', 'nt-8157',
               'nt-8343', 'nt-9057', 'nt-10645', 'nt-10781', 'nt-12771', 'nt-757', 'nt-5652', 'nt-6769', 'nt-7388',
               'nt-8227', 'nt-10163', 'nt-10477', 'nt-11867', 'nt-13930', 'nt-14044', 'nt-46', 'nt-353', 'nt-423',
               'nt-9762', 'nt-10435', 'nt-10551', 'nt-11470', 'nt-13734', 'nt-821', 'nt-2674', 'nt-3776', 'nt-4346',
               'nt-6132', 'nt-7272', 'nt-9574', 'nt-10094', 'nt-12452', 'nt-1952', 'nt-3437', 'nt-4250', 'nt-8119',
               'nt-8993', 'nt-11359', 'nt-11616', 'nt-1335', 'nt-5616', 'nt-6711', 'nt-9521', 'nt-9578', 'nt-11234',
               'nt-12178', 'nt-12934', 'nt-342', 'nt-3088', 'nt-5453', 'nt-5685', 'nt-5725', 'nt-8991', 'nt-11935',
               'nt-12260', 'nt-12513', 'nt-7843', 'nt-8743', 'nt-9629', 'nt-10823', 'nt-801', 'nt-1430', 'nt-5358',
               'nt-8869', 'nt-9595', 'nt-10196', 'nt-10226', 'nt-13523', 'nt-13645', 'nt-505', 'nt-2698', 'nt-3377',
               'nt-4334', 'nt-4866', 'nt-5219', 'nt-8443', 'nt-9229', 'nt-10031', 'nt-11814', 'nt-10798', 'nt-13954',
               'nt-12226', 'nt-575', 'nt-991', 'nt-2100', 'nt-2506', 'nt-7112', 'nt-8182', 'nt-9735', 'nt-10274',
               'nt-8748', 'nt-1006', 'nt-2687', 'nt-4700', 'nt-1095', 'nt-2342', 'nt-2451', 'nt-3733', 'nt-4025',
               'nt-11279', 'nt-13308', 'nt-13824', 'nt-11681', 'nt-3747', 'nt-3905', 'nt-4155', 'nt-7299', 'nt-12095',
               'nt-12279', 'nt-12849', 'nt-4938', 'nt-7927', 'nt-10080', 'nt-10912', 'nt-13400', 'nt-32', 'nt-1234',
               'nt-2728', 'nt-2964', 'nt-6632', 'nt-8087', 'nt-8984', 'nt-9106', 'nt-10265', 'nt-12876', 'nt-13243',
               'nt-13968', 'nt-266', 'nt-2072', 'nt-2719', 'nt-4389', 'nt-6239', 'nt-8974', 'nt-11238', 'nt-12369',
               'nt-365', 'nt-829', 'nt-12206', 'nt-2048', 'nt-3406', 'nt-4051', 'nt-4332', 'nt-6042', 'nt-10490',
               'nt-11096', 'nt-11823', 'nt-12534', 'nt-13625', 'nt-914', 'nt-2303', 'nt-3868', 'nt-4672', 'nt-5123',
               'nt-6981', 'nt-7415', 'nt-11583', 'nt-13014', 'nt-13978', 'nt-354', 'nt-710', 'nt-1923', 'nt-3108',
               'nt-4906', 'nt-9686', 'nt-11347', 'nt-11427', 'nt-11905', 'nt-12251', 'nt-4806', 'nt-4981', 'nt-6074',
               'nt-9921', 'nt-10731', 'nt-11807', 'nt-52', 'nt-2036', 'nt-3579', 'nt-3864', 'nt-9082', 'nt-9089',
               'nt-9107', 'nt-11487', 'nt-12054', 'nt-14083', 'nt-267', 'nt-6819', 'nt-8431', 'nt-9270', 'nt-11500',
               'nt-1112', 'nt-2972', 'nt-4306', 'nt-6419', 'nt-7748', 'nt-8537', 'nt-9269', 'nt-12950', 'nt-13065',
               'nt-136', 'nt-809', 'nt-4497', 'nt-4783', 'nt-4845', 'nt-5548', 'nt-5739', 'nt-8197', 'nt-8586',
               'nt-9780', 'nt-12595', 'nt-35', 'nt-2028', 'nt-6193', 'nt-6944', 'nt-7457', 'nt-10834', 'nt-12774',
               'nt-13970', 'nt-3157', 'nt-3362', 'nt-6142', 'nt-7150', 'nt-10305', 'nt-10657', 'nt-13882', 'nt-5706',
               'nt-6343', 'nt-9133', 'nt-10486', 'nt-10659', 'nt-12476', 'nt-1903', 'nt-7111', 'nt-9074', 'nt-10332',
               'nt-10794', 'nt-12910', 'nt-13696', 'nt-847', 'nt-4065', 'nt-4963', 'nt-6260', 'nt-7023', 'nt-9206',
               'nt-13511', 'nt-241', 'nt-846', 'nt-1055', 'nt-1816', 'nt-2993', 'nt-6885', 'nt-9540', 'nt-12205',
               'nt-6103', 'nt-2093', 'nt-3130', 'nt-10770', 'nt-11781', 'nt-1119', 'nt-5352', 'nt-319', 'nt-345',
               'nt-5058', 'nt-7731', 'nt-8742', 'nt-9007', 'nt-9119', 'nt-9403', 'nt-10384', 'nt-13702', 'nt-45',
               'nt-2009', 'nt-5563', 'nt-6131', 'nt-6276', 'nt-6293', 'nt-9468', 'nt-9728', 'nt-10847', 'nt-12112',
               'nt-13163', 'nt-5479', 'nt-6064', 'nt-7825', 'nt-8929', 'nt-9113', 'nt-10758', 'nt-760', 'nt-1886',
               'nt-4153', 'nt-4563', 'nt-10522', 'nt-11183', 'nt-11624', 'nt-13416', 'nt-1435', 'nt-1689', 'nt-1742',
               'nt-1964', 'nt-2501', 'nt-3320', 'nt-3816', 'nt-4886', 'nt-6524', 'nt-6761', 'nt-11722', 'nt-11973',
               'nt-13216', 'nt-1677', 'nt-1766', 'nt-2532', 'nt-2602', 'nt-2722', 'nt-2833', 'nt-3058', 'nt-4632',
               'nt-7011', 'nt-10432', 'nt-13710', 'nt-1080', 'nt-3683', 'nt-7180', 'nt-7398', 'nt-7694', 'nt-8111',
               'nt-9585', 'nt-11079', 'nt-13279', 'nt-1546', 'nt-1743', 'nt-4366', 'nt-4640', 'nt-4888', 'nt-5844',
               'nt-7471', 'nt-9981', 'nt-10613', 'nt-10970', 'nt-222', 'nt-1921', 'nt-2468', 'nt-2904', 'nt-4965',
               'nt-7655', 'nt-9479', 'nt-12281', 'nt-12378', 'nt-338', 'nt-2251', 'nt-3634', 'nt-4414', 'nt-5580',
               'nt-10049', 'nt-10684', 'nt-10813', 'nt-703', 'nt-4738', 'nt-8141', 'nt-8262', 'nt-13669', 'nt-1865',
               'nt-7443', 'nt-8810', 'nt-12078', 'nt-13890', 'nt-14138', 'nt-740', 'nt-767', 'nt-1926', 'nt-2985',
               'nt-4247', 'nt-6309', 'nt-7644', 'nt-10886', 'nt-11436', 'nt-571', 'nt-1683', 'nt-3003', 'nt-3573',
               'nt-6345', 'nt-7463', 'nt-9650', 'nt-11110', 'nt-12443', 'nt-377', 'nt-3784', 'nt-3931', 'nt-7991',
               'nt-8720', 'nt-8836', 'nt-10062', 'nt-11784', 'nt-13525', 'nt-2129', 'nt-2550', 'nt-5133', 'nt-6229',
               'nt-6378', 'nt-9674', 'nt-11597', 'nt-11661', 'nt-11669', 'nt-3708', 'nt-5927', 'nt-6279', 'nt-2191',
               'nt-2643', 'nt-4285', 'nt-1827', 'nt-2295', 'nt-13177', 'nt-6724', 'nt-737', 'nt-2291', 'nt-7241',
               'nt-9385', 'nt-9849', 'nt-12500', 'nt-2035', 'nt-4206', 'nt-4585', 'nt-5166', 'nt-6731', 'nt-9739',
               'nt-11184', 'nt-13486', 'nt-3902', 'nt-9339', 'nt-13885', 'nt-774', 'nt-2708', 'nt-3513', 'nt-4961',
               'nt-5324', 'nt-5582', 'nt-10993', 'nt-12383', 'nt-6159', 'nt-7406', 'nt-7707', 'nt-8556', 'nt-8799',
               'nt-11658', 'nt-12991', 'nt-5534', 'nt-5558', 'nt-6621', 'nt-7036', 'nt-7222', 'nt-8478', 'nt-9032',
               'nt-9833', 'nt-4013', 'nt-6071', 'nt-7818', 'nt-11499', 'nt-13427', 'nt-13797', 'nt-13835', 'nt-666',
               'nt-1701', 'nt-5070', 'nt-7308', 'nt-11229', 'nt-6322', 'nt-9639', 'nt-9707', 'nt-10253', 'nt-11170',
               'nt-1377', 'nt-1474', 'nt-1691', 'nt-3248', 'nt-3376', 'nt-6069', 'nt-6445', 'nt-12294', 'nt-10578',
               'nt-11893', 'nt-13576', 'nt-8444', 'nt-1502', 'nt-3127', 'nt-4392', 'nt-6350', 'nt-6394', 'nt-8968',
               'nt-9769', 'nt-10757', 'nt-1153', 'nt-2988', 'nt-7078', 'nt-8695', 'nt-9122', 'nt-11613', 'nt-13926',
               'nt-2891', 'nt-5676', 'nt-6449', 'nt-6554', 'nt-7887', 'nt-8205', 'nt-9952', 'nt-12342', 'nt-14101',
               'nt-1992', 'nt-3050', 'nt-3241', 'nt-3331', 'nt-4390', 'nt-5619', 'nt-7054', 'nt-7364', 'nt-7681',
               'nt-8243', 'nt-11747', 'nt-12265', 'nt-12923', 'nt-6178', 'nt-10322', 'nt-12033', 'nt-12448', 'nt-4',
               'nt-461', 'nt-5304', 'nt-5795', 'nt-7297', 'nt-9872', 'nt-12996', 'nt-13049', 'nt-13204', 'nt-543',
               'nt-1181', 'nt-2724', 'nt-3336', 'nt-4240', 'nt-6480', 'nt-8835', 'nt-10278', 'nt-10410', 'nt-11262',
               'nt-2193', 'nt-6070', 'nt-7203', 'nt-9073', 'nt-13190', 'nt-504', 'nt-4057', 'nt-4352', 'nt-6448',
               'nt-6508', 'nt-7772', 'nt-7893', 'nt-8115', 'nt-8328', 'nt-12359', 'nt-13853', 'nt-176', 'nt-2611',
               'nt-2681', 'nt-4616', 'nt-6217', 'nt-7791', 'nt-8462', 'nt-10623', 'nt-12663', 'nt-1567', 'nt-3085',
               'nt-3971', 'nt-8128', 'nt-8132', 'nt-8548', 'nt-10699', 'nt-12407', 'nt-12918', 'nt-13955', 'nt-368',
               'nt-1295', 'nt-1729', 'nt-2011', 'nt-5044', 'nt-6205', 'nt-7688', 'nt-8736', 'nt-8776', 'nt-9900',
               'nt-10698', 'nt-12539', 'nt-1213', 'nt-6465', 'nt-7782', 'nt-555', 'nt-1131', 'nt-1442', 'nt-2052',
               'nt-4039', 'nt-6946', 'nt-8787', 'nt-10995', 'nt-11211', 'nt-11719', 'nt-329', 'nt-3766', 'nt-4986',
               'nt-6424', 'nt-7338', 'nt-10641', 'nt-11657', 'nt-11970', 'nt-9771', 'nt-1875', 'nt-2403', 'nt-3340',
               'nt-4760', 'nt-8900', 'nt-9321', 'nt-10436', 'nt-11171', 'nt-13552', 'nt-5009', 'nt-5277', 'nt-8514',
               'nt-8875', 'nt-9542', 'nt-11468', 'nt-12655', 'nt-1869', 'nt-2064', 'nt-2296', 'nt-3797', 'nt-4794',
               'nt-6567', 'nt-9367', 'nt-9802', 'nt-10475', 'nt-11396', 'nt-12398', 'nt-6531', 'nt-10456', 'nt-12083',
               'nt-12417', 'nt-12697', 'nt-13503', 'nt-506', 'nt-2603', 'nt-6386', 'nt-7259', 'nt-11057', 'nt-13530',
               'nt-2422', 'nt-2854', 'nt-4149', 'nt-5970', 'nt-6654', 'nt-9716', 'nt-9836', 'nt-11836', 'nt-386',
               'nt-964', 'nt-2504', 'nt-3839', 'nt-8713', 'nt-9570', 'nt-10294', 'nt-12825', 'nt-13448', 'nt-13889',
               'nt-269', 'nt-1863', 'nt-2399', 'nt-2646', 'nt-5869', 'nt-8934', 'nt-9906', 'nt-9916', 'nt-10648',
               'nt-1232', 'nt-2671', 'nt-4036', 'nt-4078', 'nt-4898', 'nt-8224', 'nt-8858', 'nt-8924', 'nt-9249',
               'nt-1670', 'nt-3667', 'nt-3991', 'nt-6993', 'nt-7421', 'nt-8552', 'nt-11081', 'nt-11201', 'nt-12798',
               'nt-569', 'nt-1193', 'nt-1403', 'nt-3678', 'nt-7191', 'nt-8096', 'nt-9121', 'nt-9752', 'nt-10468',
               'nt-11464', 'nt-5721', 'nt-6044', 'nt-8616', 'nt-9221', 'nt-10458', 'nt-12089', 'nt-12776', 'nt-13913',
               'nt-982', 'nt-3500', 'nt-3559', 'nt-3967', 'nt-7642', 'nt-7702', 'nt-8043', 'nt-8133', 'nt-8499',
               'nt-9934', 'nt-74', 'nt-1791', 'nt-1976', 'nt-3781', 'nt-7274', 'nt-9049', 'nt-12634', 'nt-3727',
               'nt-4764', 'nt-5319', 'nt-5868', 'nt-8877', 'nt-9261', 'nt-11030', 'nt-126', 'nt-133', 'nt-1731',
               'nt-3998', 'nt-5910', 'nt-6964', 'nt-7482', 'nt-9076', 'nt-9325', 'nt-9708', 'nt-10088', 'nt-11588',
               'nt-4384', 'nt-6969', 'nt-7147', 'nt-8853', 'nt-8888', 'nt-9878', 'nt-9995', 'nt-11887', 'nt-13944',
               'nt-1911', 'nt-3346', 'nt-3631', 'nt-7341', 'nt-7696', 'nt-8378', 'nt-9531', 'nt-10986', 'nt-7508',
               'nt-1076', 'nt-2574', 'nt-2607', 'nt-3597', 'nt-7343', 'nt-8582', 'nt-8650', 'nt-9455', 'nt-10615',
               'nt-13135', 'nt-13417', 'nt-13678', 'nt-14057', 'nt-149', 'nt-6255', 'nt-6821', 'nt-9201', 'nt-9281',
               'nt-10580', 'nt-11112', 'nt-11406', 'nt-121', 'nt-1201', 'nt-1437', 'nt-5093', 'nt-5499', 'nt-5824',
               'nt-6956', 'nt-8067', 'nt-10715', 'nt-1809', 'nt-3203', 'nt-4269', 'nt-6664', 'nt-6877', 'nt-7622',
               'nt-11166', 'nt-11525', 'nt-12533', 'nt-13774', 'nt-308', 'nt-336', 'nt-721', 'nt-4267', 'nt-4367',
               'nt-5649', 'nt-9271', 'nt-10719', 'nt-12141', 'nt-12578', 'nt-324', 'nt-1334', 'nt-2261', 'nt-2798',
               'nt-3903', 'nt-4698', 'nt-4723', 'nt-5560', 'nt-5951', 'nt-7137', 'nt-7292', 'nt-13201', 'nt-1494',
               'nt-3843', 'nt-5461', 'nt-6703', 'nt-7685', 'nt-7979', 'nt-12458', 'nt-13070', 'nt-13410', 'nt-1397',
               'nt-4265', 'nt-6709', 'nt-7045', 'nt-9346', 'nt-13414', 'nt-2069', 'nt-13347', 'nt-2543', 'nt-4466',
               'nt-4717', 'nt-7630', 'nt-8508', 'nt-9183', 'nt-9616', 'nt-12084', 'nt-13733', 'nt-14001', 'nt-161',
               'nt-1465', 'nt-5077', 'nt-5839', 'nt-6669', 'nt-7041', 'nt-7610', 'nt-7987', 'nt-1744', 'nt-6597',
               'nt-8359', 'nt-9114', 'nt-11372', 'nt-13851', 'nt-1031', 'nt-2726', 'nt-3555', 'nt-5256', 'nt-7277',
               'nt-7638', 'nt-8377', 'nt-10930', 'nt-11875', 'nt-13019', 'nt-13579', 'nt-13865', 'nt-872', 'nt-1497',
               'nt-1806', 'nt-1933', 'nt-3990', 'nt-6016', 'nt-6224', 'nt-7719', 'nt-7934', 'nt-10923', 'nt-3007',
               'nt-3715', 'nt-9217', 'nt-742', 'nt-3831', 'nt-5754', 'nt-7235', 'nt-7865', 'nt-9075', 'nt-10442',
               'nt-13462', 'nt-2313', 'nt-3175', 'nt-5819', 'nt-7051', 'nt-7656', 'nt-8239', 'nt-13430', 'nt-5918',
               'nt-12514', 'nt-12666', 'nt-1291', 'nt-2354', 'nt-3850', 'nt-7960', 'nt-9549', 'nt-10975', 'nt-11667',
               'nt-13746', 'nt-1117', 'nt-2609', 'nt-2654', 'nt-4085', 'nt-5265', 'nt-5516', 'nt-9171', 'nt-11556',
               'nt-11910', 'nt-13220', 'nt-817', 'nt-1129', 'nt-2559', 'nt-5168', 'nt-9584', 'nt-2973', 'nt-4790',
               'nt-6720', 'nt-6997', 'nt-8072', 'nt-10583', 'nt-12071', 'nt-12341', 'nt-12790', 'nt-6173', 'nt-6721',
               'nt-6796', 'nt-7053', 'nt-9737', 'nt-692', 'nt-1183', 'nt-1651', 'nt-6068', 'nt-6274', 'nt-7981',
               'nt-8658', 'nt-9488', 'nt-12590', 'nt-1637', 'nt-8655', 'nt-9351', 'nt-10457', 'nt-12123', 'nt-12773',
               'nt-536', 'nt-4568', 'nt-4615', 'nt-7994', 'nt-11140', 'nt-12601', 'nt-614', 'nt-2952', 'nt-4693',
               'nt-5751', 'nt-8615', 'nt-8892', 'nt-11671', 'nt-174', 'nt-405', 'nt-1846', 'nt-3009', 'nt-6056',
               'nt-7461', 'nt-9973', 'nt-13005', 'nt-13814', 'nt-303', 'nt-2691', 'nt-2851', 'nt-8429', 'nt-11553',
               'nt-13021', 'nt-13476', 'nt-112', 'nt-153', 'nt-3086', 'nt-3711', 'nt-5893', 'nt-9841', 'nt-10277',
               'nt-12130', 'nt-1733', 'nt-3392', 'nt-5594', 'nt-5753', 'nt-7329', 'nt-7692', 'nt-8235', 'nt-8523',
               'nt-8902', 'nt-10936', 'nt-11256', 'nt-12438', 'nt-1005', 'nt-1157', 'nt-3456', 'nt-7402', 'nt-8893',
               'nt-9250', 'nt-13139', 'nt-13720', 'nt-475', 'nt-3680', 'nt-4264', 'nt-6226', 'nt-9720', 'nt-11116',
               'nt-12365', 'nt-12874', 'nt-13719', 'nt-3922', 'nt-7248', 'nt-13161', 'nt-1703', 'nt-2503', 'nt-2666',
               'nt-5260', 'nt-5576', 'nt-6967', 'nt-6991', 'nt-9524', 'nt-11389', 'nt-12750', 'nt-559', 'nt-968',
               'nt-1073', 'nt-1650', 'nt-3057', 'nt-6311', 'nt-6692', 'nt-8309', 'nt-10637', 'nt-11683', 'nt-12884',
               'nt-444', 'nt-6048', 'nt-9289', 'nt-9520', 'nt-10612', 'nt-11660', 'nt-12018', 'nt-12622', 'nt-13297',
               'nt-1446', 'nt-1475', 'nt-1767', 'nt-7340', 'nt-7878', 'nt-8099', 'nt-8519', 'nt-10087', 'nt-10707',
               'nt-11306', 'nt-11679', 'nt-12850', 'nt-13477', 'nt-13518', 'nt-789', 'nt-2073', 'nt-3798', 'nt-8947',
               'nt-10594', 'nt-10871', 'nt-11165', 'nt-1518', 'nt-1562', 'nt-3979', 'nt-5206', 'nt-5833', 'nt-5998',
               'nt-7425', 'nt-9231', 'nt-9484', 'nt-12053', 'nt-12593', 'nt-1259', 'nt-2016', 'nt-3525', 'nt-3842',
               'nt-5396', 'nt-5963', 'nt-6210', 'nt-8753', 'nt-9152', 'nt-9347', 'nt-10099', 'nt-11045', 'nt-13194',
               'nt-3304', 'nt-4984', 'nt-7142', 'nt-8919', 'nt-9022', 'nt-10105', 'nt-11086', 'nt-11902', 'nt-661',
               'nt-1501', 'nt-2462', 'nt-2968', 'nt-3373', 'nt-4121', 'nt-5741', 'nt-8209', 'nt-11695', 'nt-12103',
               'nt-12436', 'nt-13164', 'nt-228', 'nt-3100', 'nt-3516', 'nt-3629', 'nt-3861', 'nt-4697', 'nt-6421',
               'nt-6497', 'nt-10118', 'nt-2409', 'nt-4596', 'nt-4941', 'nt-5438', 'nt-6334', 'nt-6416', 'nt-6744',
               'nt-7922', 'nt-9989', 'nt-11713', 'nt-14056', 'nt-3639', 'nt-3891', 'nt-4976', 'nt-5101', 'nt-5141',
               'nt-5407', 'nt-11398', 'nt-11834', 'nt-12526', 'nt-3073', 'nt-3470', 'nt-3510', 'nt-3740', 'nt-4028',
               'nt-6552', 'nt-6566', 'nt-7652', 'nt-10394', 'nt-11218', 'nt-13232', 'nt-14032', 'nt-799', 'nt-808',
               'nt-2171', 'nt-2395', 'nt-3628', 'nt-7345', 'nt-11558', 'nt-12440', 'nt-13946', 'nt-2509', 'nt-6046',
               'nt-7254', 'nt-10517', 'nt-10746', 'nt-11371', 'nt-12181', 'nt-12779', 'nt-13269', 'nt-7552', 'nt-11986',
               'nt-12618', 'nt-12716', 'nt-1891', 'nt-2005', 'nt-3226', 'nt-4138', 'nt-5188', 'nt-13101', 'nt-896',
               'nt-1345', 'nt-3123', 'nt-7017', 'nt-7060', 'nt-8549', 'nt-9275', 'nt-12660', 'nt-2592', 'nt-3767',
               'nt-4281', 'nt-5310', 'nt-9431', 'nt-1552', 'nt-3484', 'nt-3970', 'nt-5002', 'nt-5886', 'nt-6154',
               'nt-13590', 'nt-13907', 'nt-5715', 'nt-6215', 'nt-6810', 'nt-9162', 'nt-10667', 'nt-7156', 'nt-8922',
               'nt-10181', 'nt-13597', 'nt-940', 'nt-2884', 'nt-3792', 'nt-9345', 'nt-10489', 'nt-13350', 'nt-14147',
               'nt-439', 'nt-1577', 'nt-2021', 'nt-2704', 'nt-4022', 'nt-4307', 'nt-4821', 'nt-11418', 'nt-11435',
               'nt-14080', 'nt-2220', 'nt-5781', 'nt-7618', 'nt-8229', 'nt-9537', 'nt-10103', 'nt-12604', 'nt-12754',
               'nt-6027', 'nt-7353', 'nt-3351', 'nt-3788', 'nt-4550', 'nt-7049', 'nt-8476', 'nt-11455', 'nt-464',
               'nt-1315', 'nt-2178', 'nt-2410', 'nt-4072', 'nt-6408', 'nt-6455', 'nt-7350', 'nt-12414', 'nt-12450',
               'nt-12588', 'nt-13806', 'nt-2040', 'nt-3432', 'nt-676', 'nt-4118', 'nt-4707', 'nt-5359', 'nt-8263',
               'nt-8272', 'nt-8550', 'nt-13207', 'nt-13845', 'nt-1206', 'nt-10420', 'nt-11861', 'nt-13272', 'nt-14151',
               'nt-720', 'nt-3862', 'nt-4319', 'nt-5083', 'nt-5575', 'nt-6120', 'nt-6328', 'nt-6451', 'nt-8707',
               'nt-10611', 'nt-12208', 'nt-522', 'nt-528', 'nt-780', 'nt-1316', 'nt-2683', 'nt-3745', 'nt-4165',
               'nt-5084', 'nt-12101', 'nt-13399', 'nt-512', 'nt-1036', 'nt-3771', 'nt-11039', 'nt-7631', 'nt-12959',
               'nt-13472', 'nt-6101', 'nt-7740', 'nt-7750', 'nt-9884', 'nt-12255', 'nt-12632', 'nt-14082', 'nt-331',
               'nt-5164', 'nt-5729', 'nt-6477', 'nt-8437', 'nt-9300', 'nt-11243', 'nt-11580', 'nt-3587', 'nt-213',
               'nt-1685', 'nt-188', 'nt-2271', 'nt-5860', 'nt-8874', 'nt-10604', 'nt-11702', 'nt-12762', 'nt-13563',
               'nt-13995', 'nt-1441', 'nt-5097', 'nt-8507', 'nt-8969', 'nt-9800', 'nt-9966', 'nt-11172', 'nt-11637',
               'nt-11668', 'nt-5647', 'nt-7952', 'nt-8660', 'nt-394', 'nt-1612', 'nt-2094', 'nt-5177', 'nt-6517',
               'nt-9719', 'nt-9856', 'nt-11317', 'nt-11581', 'nt-13016', 'nt-13828', 'nt-107', 'nt-447', 'nt-3040',
               'nt-4114', 'nt-4835', 'nt-6020', 'nt-12271', 'nt-12349', 'nt-13660', 'nt-13923', 'nt-248', 'nt-647',
               'nt-3651', 'nt-9236', 'nt-10170', 'nt-11544', 'nt-12107', 'nt-12965', 'nt-988', 'nt-3116', 'nt-3840',
               'nt-4958', 'nt-6181', 'nt-6441', 'nt-6965', 'nt-7667', 'nt-8497', 'nt-2077', 'nt-2305', 'nt-8109',
               'nt-9303', 'nt-10749', 'nt-11815', 'nt-8368', 'nt-9408', 'nt-9545', 'nt-10347', 'nt-11967', 'nt-12415',
               'nt-972', 'nt-4520', 'nt-8165', 'nt-10576', 'nt-2537', 'nt-7040', 'nt-8428', 'nt-8827', 'nt-11134',
               'nt-11685', 'nt-12204', 'nt-12902', 'nt-13230', 'nt-13773', 'nt-3650', 'nt-5830', 'nt-14047', 'nt-220',
               'nt-10153', 'nt-905', 'nt-1822', 'nt-2773', 'nt-3297', 'nt-3326', 'nt-4460', 'nt-6242', 'nt-6560',
               'nt-6758', 'nt-7455', 'nt-8705', 'nt-10187', 'nt-11402', 'nt-13827', 'nt-13863', 'nt-6108', 'nt-7110',
               'nt-8848', 'nt-8850', 'nt-9036', 'nt-10215', 'nt-10266', 'nt-13015', 'nt-13762', 'nt-6467', 'nt-7942',
               'nt-8332', 'nt-9792', 'nt-10806', 'nt-11332', 'nt-684', 'nt-2316', 'nt-2994', 'nt-12628', 'nt-13991',
               'nt-2941', 'nt-3396', 'nt-4635', 'nt-6791', 'nt-7697', 'nt-9661', 'nt-10696', 'nt-11099', 'nt-12323',
               'nt-766', 'nt-1935', 'nt-2898', 'nt-7627', 'nt-9831', 'nt-10244', 'nt-1510', 'nt-5334', 'nt-5420',
               'nt-10107', 'nt-11928', 'nt-434', 'nt-463', 'nt-1697', 'nt-6557', 'nt-6976', 'nt-10291', 'nt-566',
               'nt-1359', 'nt-2517', 'nt-2878', 'nt-3615', 'nt-6008', 'nt-7803', 'nt-8170', 'nt-8910', 'nt-10377',
               'nt-12621', 'nt-13567', 'nt-955', 'nt-2314', 'nt-7520', 'nt-11017', 'nt-13795', 'nt-4671', 'nt-5241',
               'nt-6459', 'nt-7547', 'nt-7966', 'nt-8669', 'nt-12770', 'nt-12947', 'nt-1756', 'nt-2249', 'nt-5481',
               'nt-11448', 'nt-12623', 'nt-795', 'nt-2186', 'nt-3154', 'nt-6841', 'nt-9809', 'nt-11348', 'nt-13086',
               'nt-633', 'nt-978', 'nt-1367', 'nt-6316', 'nt-6580', 'nt-6681', 'nt-7734', 'nt-8773', 'nt-9525',
               'nt-9575', 'nt-9915', 'nt-11615', 'nt-13089', 'nt-2415', 'nt-4033', 'nt-4872', 'nt-7456', 'nt-8172',
               'nt-13491', 'nt-311', 'nt-3546', 'nt-7738', 'nt-9160', 'nt-9994', 'nt-13967', 'nt-140', 'nt-663',
               'nt-2211', 'nt-3283', 'nt-10082', 'nt-3688', 'nt-6657', 'nt-9444', 'nt-11138', 'nt-11269', 'nt-14005',
               'nt-2431', 'nt-4479', 'nt-13707', 'nt-10870', 'nt-10908', 'nt-11779', 'nt-551', 'nt-2356', 'nt-10650',
               'nt-11290', 'nt-11888', 'nt-13013', 'nt-741', 'nt-4323', 'nt-7502', 'nt-9001', 'nt-9092', 'nt-11009',
               'nt-11749', 'nt-13611', 'nt-14116', 'nt-2275', 'nt-2840', 'nt-2853', 'nt-3523', 'nt-7635', 'nt-8904',
               'nt-10605', 'nt-12820', 'nt-13487', 'nt-2340', 'nt-5079', 'nt-5559', 'nt-5875', 'nt-6495', 'nt-6658',
               'nt-8953', 'nt-9464', 'nt-9740', 'nt-10207', 'nt-14087', 'nt-3604', 'nt-3841', 'nt-3972', 'nt-6370',
               'nt-11210', 'nt-11379', 'nt-1035', 'nt-2030', 'nt-3918', 'nt-7063', 'nt-7131', 'nt-8568', 'nt-9897',
               'nt-11563', 'nt-13497', 'nt-1204', 'nt-1591', 'nt-2826', 'nt-4676', 'nt-7234', 'nt-9008', 'nt-11106',
               'nt-11382', 'nt-453', 'nt-2672', 'nt-2857', 'nt-9044', 'nt-9239', 'nt-9356', 'nt-11731', 'nt-13354',
               'nt-1572', 'nt-2290', 'nt-3317', 'nt-3418', 'nt-6434', 'nt-9197', 'nt-903', 'nt-1422', 'nt-6820',
               'nt-11292', 'nt-11828', 'nt-12566', 'nt-13661', 'nt-5339', 'nt-8148', 'nt-11666', 'nt-560', 'nt-837',
               'nt-1360', 'nt-3289', 'nt-3682', 'nt-8009', 'nt-8207', 'nt-8590', 'nt-10162', 'nt-10411', 'nt-12301',
               'nt-12461', 'nt-631', 'nt-845', 'nt-4493', 'nt-8761', 'nt-9190', 'nt-1848', 'nt-5564', 'nt-7728',
               'nt-8271', 'nt-2044', 'nt-3145', 'nt-4162', 'nt-4813', 'nt-5858', 'nt-7062', 'nt-7714', 'nt-8171',
               'nt-10375', 'nt-12719', 'nt-12995', 'nt-13115', 'nt-11699', 'nt-14019', 'nt-11', 'nt-144', 'nt-1069',
               'nt-2169', 'nt-2521', 'nt-3190', 'nt-3641', 'nt-4891', 'nt-7020', 'nt-11578', 'nt-12219', 'nt-12422',
               'nt-885', 'nt-4430', 'nt-5375', 'nt-8781', 'nt-10994', 'nt-13102', 'nt-2744', 'nt-3049', 'nt-4621',
               'nt-4837', 'nt-5167', 'nt-6801', 'nt-11395', 'nt-11714', 'nt-12307', 'nt-13700', 'nt-14023', 'nt-14104',
               'nt-4699', 'nt-4913', 'nt-5965', 'nt-7076', 'nt-8166', 'nt-8201', 'nt-8246', 'nt-10233', 'nt-10463',
               'nt-10902', 'nt-249', 'nt-1410', 'nt-3138', 'nt-3176', 'nt-3917', 'nt-4743', 'nt-6158', 'nt-8015',
               'nt-11812', 'nt-383', 'nt-4031', 'nt-4340', 'nt-6648', 'nt-8335', 'nt-12261', 'nt-13031', 'nt-5872',
               'nt-7634', 'nt-9699', 'nt-12752', 'nt-689', 'nt-1387', 'nt-3726', 'nt-5546', 'nt-7004', 'nt-8388',
               'nt-8992', 'nt-9854', 'nt-11858', 'nt-12585', 'nt-1', 'nt-166', 'nt-1287', 'nt-4094', 'nt-6745',
               'nt-7333', 'nt-11160', 'nt-11432', 'nt-11497', 'nt-11540', 'nt-13064', 'nt-3871', 'nt-8876', 'nt-13203',
               'nt-351', 'nt-4654', 'nt-6874', 'nt-7673', 'nt-9172', 'nt-10120', 'nt-10655', 'nt-11077', 'nt-12542',
               'nt-12550', 'nt-13589', 'nt-826', 'nt-3337', 'nt-4803', 'nt-5670', 'nt-13540', 'nt-2657', 'nt-3908',
               'nt-6150', 'nt-6392', 'nt-8250', 'nt-11439', 'nt-12739', 'nt-12848', 'nt-2147', 'nt-2237', 'nt-4973',
               'nt-9387', 'nt-10905', 'nt-11741', 'nt-824', 'nt-2047', 'nt-2253', 'nt-3544', 'nt-4451', 'nt-5936',
               'nt-5940', 'nt-9528', 'nt-9935', 'nt-10000', 'nt-10832', 'nt-11270', 'nt-13052', 'nt-2394', 'nt-5356',
               'nt-5720', 'nt-7944', 'nt-10601', 'nt-12586', 'nt-3249', 'nt-4690', 'nt-6054', 'nt-7721', 'nt-8638',
               'nt-8927', 'nt-13880', 'nt-13896', 'nt-85', 'nt-3805', 'nt-3879', 'nt-7423', 'nt-10242', 'nt-10728',
               'nt-11912', 'nt-6496', 'nt-7795', 'nt-8614', 'nt-10565', 'nt-1762', 'nt-2736', 'nt-3043', 'nt-4311',
               'nt-9267', 'nt-12131', 'nt-13602', 'nt-13', 'nt-1855', 'nt-3300', 'nt-9083', 'nt-10281', 'nt-471',
               'nt-2014', 'nt-2508', 'nt-3882', 'nt-8199', 'nt-9848', 'nt-11198', 'nt-12006', 'nt-1094', 'nt-2034',
               'nt-3921', 'nt-5225', 'nt-7320', 'nt-8978', 'nt-9723', 'nt-10572', 'nt-10635', 'nt-12169', 'nt-318',
               'nt-1698', 'nt-2162', 'nt-4156', 'nt-4851', 'nt-7690', 'nt-8688', 'nt-8771', 'nt-8982', 'nt-9507',
               'nt-960', 'nt-1738', 'nt-1815', 'nt-4818', 'nt-7833', 'nt-8466', 'nt-9359', 'nt-12073', 'nt-12682',
               'nt-13092', 'nt-2540', 'nt-3521', 'nt-9406', 'nt-12679', 'nt-895', 'nt-1684', 'nt-2464', 'nt-2485',
               'nt-8623', 'nt-9764', 'nt-11524', 'nt-12821', 'nt-14034', 'nt-1633', 'nt-8804', 'nt-1752', 'nt-3443',
               'nt-6288', 'nt-8648', 'nt-9757', 'nt-11242', 'nt-13453', 'nt-442', 'nt-1382', 'nt-2195', 'nt-4063',
               'nt-4580', 'nt-6035', 'nt-7975', 'nt-10071', 'nt-2805', 'nt-4494', 'nt-5732', 'nt-6527', 'nt-8532',
               'nt-3837', 'nt-889', 'nt-4008', 'nt-4122', 'nt-10125', 'nt-10197', 'nt-13425', 'nt-1037', 'nt-1707',
               'nt-1850', 'nt-9691', 'nt-10959', 'nt-2760', 'nt-6040', 'nt-7545', 'nt-10309', 'nt-13224', 'nt-13471',
               'nt-13788', 'nt-4903', 'nt-13174', 'nt-291', 'nt-1468', 'nt-1775', 'nt-5078', 'nt-5507', 'nt-5994',
               'nt-6824', 'nt-6866', 'nt-7442', 'nt-8571', 'nt-8891', 'nt-10910', 'nt-574', 'nt-748', 'nt-1858',
               'nt-2232', 'nt-3762', 'nt-4620', 'nt-4842', 'nt-4900', 'nt-5873', 'nt-9871', 'nt-12207', 'nt-12391',
               'nt-227', 'nt-3966', 'nt-13316', 'nt-6555', 'nt-9348', 'nt-9755', 'nt-11495', 'nt-12148', 'nt-12311',
               'nt-12328', 'nt-12511', 'nt-13634', 'nt-2900', 'nt-3578', 'nt-4251', 'nt-6174', 'nt-6367', 'nt-6373',
               'nt-9754', 'nt-11557', 'nt-13859', 'nt-2469', 'nt-3325', 'nt-4397', 'nt-5076', 'nt-5197', 'nt-7881',
               'nt-10058', 'nt-10825', 'nt-12187', 'nt-1894', 'nt-2091', 'nt-3263', 'nt-4137', 'nt-4997', 'nt-7165',
               'nt-7741', 'nt-8671', 'nt-9053', 'nt-9080', 'nt-11345', 'nt-893', 'nt-2277', 'nt-2605', 'nt-5047',
               'nt-6269', 'nt-6777', 'nt-10556', 'nt-13003', 'nt-13629', 'nt-1726', 'nt-2297', 'nt-9902', 'nt-13508',
               'nt-397', 'nt-1648', 'nt-3948', 'nt-4782', 'nt-8414', 'nt-10247', 'nt-4968', 'nt-5586', 'nt-5774',
               'nt-8798', 'nt-9749', 'nt-11369', 'nt-12769', 'nt-1033', 'nt-7330', 'nt-13996', 'nt-6914', 'nt-6933',
               'nt-832', 'nt-1285', 'nt-3399', 'nt-3599', 'nt-6411', 'nt-6994', 'nt-10692', 'nt-11343', 'nt-12340',
               'nt-13027', 'nt-5902', 'nt-7087', 'nt-7858', 'nt-9700', 'nt-10513', 'nt-11959', 'nt-13649', 'nt-3732',
               'nt-5143', 'nt-5628', 'nt-6990', 'nt-8200', 'nt-9333', 'nt-10066', 'nt-10430', 'nt-12074', 'nt-12521',
               'nt-1374', 'nt-4193', 'nt-5471', 'nt-5523', 'nt-11795', 'nt-1395', 'nt-3310', 'nt-3662', 'nt-4996',
               'nt-5111', 'nt-13184', 'nt-639', 'nt-2500', 'nt-2837', 'nt-8630', 'nt-9931', 'nt-10298', 'nt-999',
               'nt-1482', 'nt-4353', 'nt-4450', 'nt-8751', 'nt-9218', 'nt-10917', 'nt-11896', 'nt-12608', 'nt-13655',
               'nt-13748', 'nt-4333', 'nt-7315', 'nt-3617', 'nt-3987', 'nt-2772', 'nt-6045', 'nt-8399', 'nt-8512',
               'nt-1897', 'nt-2811', 'nt-3482', 'nt-10676', 'nt-10777', 'nt-10924', 'nt-13442', 'nt-2911', 'nt-5456',
               'nt-10903', 'nt-11570', 'nt-11572', 'nt-11692', 'nt-12627', 'nt-2084', 'nt-3699', 'nt-6887', 'nt-8473',
               'nt-9576', 'nt-12726', 'nt-13904', 'nt-1440', 'nt-1896', 'nt-4011', 'nt-4930', 'nt-456', 'nt-2317',
               'nt-2530', 'nt-5673', 'nt-6406', 'nt-8908', 'nt-13538', 'nt-1735', 'nt-6087', 'nt-7505', 'nt-8094',
               'nt-8990', 'nt-14094', 'nt-477', 'nt-4644', 'nt-5932', 'nt-7659', 'nt-8296', 'nt-8665', 'nt-8930',
               'nt-9150', 'nt-11433', 'nt-13618', 'nt-14066', 'nt-2960', 'nt-9534', 'nt-10228', 'nt-11575', 'nt-2244',
               'nt-2407', 'nt-3031', 'nt-3059', 'nt-4035', 'nt-4191', 'nt-4714', 'nt-4869', 'nt-8093', 'nt-11940',
               'nt-233', 'nt-1276', 'nt-10312', 'nt-11461', 'nt-11475', 'nt-12335', 'nt-13126', 'nt-14042', 'nt-693',
               'nt-749', 'nt-2986', 'nt-4142', 'nt-6616', 'nt-7809', 'nt-7931', 'nt-11311', 'nt-12648', 'nt-13344',
               'nt-14039', 'nt-1888', 'nt-7059', 'nt-1139', 'nt-3486', 'nt-6522', 'nt-9091', 'nt-12003', 'nt-3159',
               'nt-3770', 'nt-4164', 'nt-7187', 'nt-7503', 'nt-8450', 'nt-9015', 'nt-10869', 'nt-12032', 'nt-12878',
               'nt-1065', 'nt-3000', 'nt-4486', 'nt-10958', 'nt-12309', 'nt-13213', 'nt-930', 'nt-1739', 'nt-3182',
               'nt-4391', 'nt-4978', 'nt-6238', 'nt-7056', 'nt-10817', 'nt-12201', 'nt-13088', 'nt-13210', 'nt-5221',
               'nt-7270', 'nt-8470', 'nt-10017', 'nt-11443', 'nt-11481', 'nt-11852', 'nt-11891', 'nt-13771', 'nt-3801',
               'nt-13196', 'nt-1071', 'nt-8546', 'nt-9358', 'nt-11604', 'nt-12818', 'nt-172', 'nt-610', 'nt-907',
               'nt-4062', 'nt-7736', 'nt-8136', 'nt-9415', 'nt-10178', 'nt-10238', 'nt-11441', 'nt-12121', 'nt-13162',
               'nt-13514', 'nt-1336', 'nt-2364', 'nt-2965', 'nt-6105', 'nt-6608', 'nt-10146', 'nt-10422', 'nt-10545',
               'nt-10865', 'nt-13534', 'nt-13728', 'nt-13810', 'nt-2022', 'nt-5930', 'nt-9892', 'nt-12749', 'nt-590',
               'nt-1300', 'nt-3113', 'nt-3472', 'nt-3779', 'nt-7725', 'nt-9689', 'nt-641', 'nt-2414', 'nt-10267',
               'nt-1545', 'nt-5209', 'nt-7632', 'nt-9599', 'nt-11492', 'nt-13039', 'nt-825', 'nt-1086', 'nt-2569',
               'nt-4653', 'nt-4664', 'nt-5129', 'nt-9932', 'nt-10026', 'nt-10259', 'nt-11163', 'nt-11509', 'nt-12052',
               'nt-13988', 'nt-685', 'nt-2492', 'nt-3932', 'nt-7145', 'nt-708', 'nt-116', 'nt-169', 'nt-1399',
               'nt-1550', 'nt-1553', 'nt-4559', 'nt-6358', 'nt-6712', 'nt-7411', 'nt-8275', 'nt-11404', 'nt-8276',
               'nt-9648', 'nt-4395', 'nt-6759', 'nt-8338', 'nt-9948', 'nt-10518', 'nt-11152', 'nt-11393', 'nt-1449',
               'nt-2104', 'nt-3260', 'nt-6301', 'nt-6439', 'nt-7971', 'nt-9156', 'nt-9812', 'nt-11021', 'nt-6',
               'nt-2817', 'nt-10553', 'nt-14028', 'nt-5585', 'nt-6262', 'nt-8639', 'nt-8645', 'nt-11881', 'nt-1275',
               'nt-3709', 'nt-6917', 'nt-7366', 'nt-1998', 'nt-3526', 'nt-6578', 'nt-10980', 'nt-2260', 'nt-2527',
               'nt-4852', 'nt-5180', 'nt-5208', 'nt-7915', 'nt-7936', 'nt-8954', 'nt-10705', 'nt-11179', 'nt-11797',
               'nt-11961', 'nt-13258', 'nt-1765', 'nt-3594', 'nt-5470', 'nt-6282', 'nt-6855', 'nt-8173', 'nt-11094',
               'nt-13365', 'nt-4446', 'nt-4916', 'nt-5082', 'nt-5229', 'nt-8321', 'nt-9026', 'nt-9434', 'nt-10097',
               'nt-11501', 'nt-11921', 'nt-1412', 'nt-3538', 'nt-5829', 'nt-8948', 'nt-9794', 'nt-11942', 'nt-1730',
               'nt-3607', 'nt-4472', 'nt-5698', 'nt-6750', 'nt-7355', 'nt-8129', 'nt-9040', 'nt-9438', 'nt-10222',
               'nt-12575', 'nt-12696', 'nt-6162', 'nt-2194', 'nt-3002', 'nt-3092', 'nt-4816', 'nt-6611', 'nt-7837',
               'nt-10425', 'nt-1249', 'nt-5607', 'nt-8603', 'nt-1460', 'nt-3700', 'nt-6878', 'nt-9180', 'nt-9448',
               'nt-10047', 'nt-13051', 'nt-987', 'nt-1056', 'nt-8805', 'nt-10549', 'nt-11397', 'nt-12203', 'nt-14103',
               'nt-1115', 'nt-2243', 'nt-4296', 'nt-5515', 'nt-5779', 'nt-7826', 'nt-9955', 'nt-10431', 'nt-13307',
               'nt-4874', 'nt-10036', 'nt-658', 'nt-726', 'nt-2382', 'nt-3656', 'nt-7891', 'nt-9233', 'nt-12829',
               'nt-9840', 'nt-10866', 'nt-53', 'nt-350', 'nt-395', 'nt-582', 'nt-1013', 'nt-3419', 'nt-5119', 'nt-7785',
               'nt-8126', 'nt-9763', 'nt-13311', 'nt-484', 'nt-2268', 'nt-8308', 'nt-8765', 'nt-11489', 'nt-1357',
               'nt-1600', 'nt-5777', 'nt-6332', 'nt-6400', 'nt-10592', 'nt-10840', 'nt-11746', 'nt-12127', 'nt-132',
               'nt-4408', 'nt-4712', 'nt-4920', 'nt-4936', 'nt-9068', 'nt-11090', 'nt-13745', 'nt-180', 'nt-1433',
               'nt-2119', 'nt-3013', 'nt-3462', 'nt-4124', 'nt-6882', 'nt-10369', 'nt-10796', 'nt-13422', 'nt-612',
               'nt-984', 'nt-1041', 'nt-1686', 'nt-2130', 'nt-2834', 'nt-6696', 'nt-8784', 'nt-10791', 'nt-636',
               'nt-1946', 'nt-5132', 'nt-5511', 'nt-12881', 'nt-2668', 'nt-2855', 'nt-7359', 'nt-10729', 'nt-11180',
               'nt-12661', 'nt-458', 'nt-1111', 'nt-11255', 'nt-13072', 'nt-9975', 'nt-7624', 'nt-13829', 'nt-2226',
               'nt-6368', 'nt-6656', 'nt-7855', 'nt-9960', 'nt-10668', 'nt-10858', 'nt-11957', 'nt-5818', 'nt-6207',
               'nt-6371', 'nt-7179', 'nt-9491', 'nt-11675', 'nt-13387', 'nt-13452', 'nt-163', 'nt-2963', 'nt-3363',
               'nt-8278', 'nt-10041', 'nt-11735', 'nt-12091', 'nt-13763', 'nt-13815', 'nt-962', 'nt-1167', 'nt-1889',
               'nt-4612', 'nt-7348', 'nt-10237', 'nt-11632', 'nt-13235', 'nt-552', 'nt-752', 'nt-5589', 'nt-8596',
               'nt-10251', 'nt-10619', 'nt-11952', 'nt-12180', 'nt-12768', 'nt-13099', 'nt-1965', 'nt-2953', 'nt-3072',
               'nt-3671', 'nt-4260', 'nt-5380', 'nt-5419', 'nt-5645', 'nt-6225', 'nt-6660', 'nt-12264', 'nt-12793',
               'nt-2076', 'nt-10997', 'nt-13789', 'nt-746', 'nt-3234', 'nt-3359', 'nt-5264', 'nt-6083', 'nt-7190',
               'nt-7594', 'nt-10221', 'nt-10511', 'nt-2662', 'nt-5956', 'nt-6549', 'nt-10685', 'nt-12491', 'nt-2992',
               'nt-3275', 'nt-3687', 'nt-3761', 'nt-9796', 'nt-10095', 'nt-11498', 'nt-7', 'nt-157', 'nt-3016',
               'nt-3963', 'nt-3976', 'nt-4183', 'nt-4481', 'nt-4937', 'nt-6118', 'nt-6982', 'nt-11403', 'nt-1093',
               'nt-2088', 'nt-3919', 'nt-4458', 'nt-10011', 'nt-10916', 'nt-13843', 'nt-1671', 'nt-2802', 'nt-4212',
               'nt-4766', 'nt-5734', 'nt-6926', 'nt-8312', 'nt-8458', 'nt-10931', 'nt-12721', 'nt-14008', 'nt-2248',
               'nt-8383', 'nt-11674', 'nt-11693', 'nt-12898', 'nt-292', 'nt-333', 'nt-815', 'nt-3035', 'nt-4448',
               'nt-4885', 'nt-10666', 'nt-12512', 'nt-13450', 'nt-13932', 'nt-13934', 'nt-4289', 'nt-5949', 'nt-11678',
               'nt-12495', 'nt-1244', 'nt-4292', 'nt-5159', 'nt-6830', 'nt-7589', 'nt-10800', 'nt-13751', 'nt-13879',
               'nt-4187', 'nt-4992', 'nt-5629', 'nt-7419', 'nt-10831', 'nt-13927', 'nt-2151', 'nt-5291', 'nt-5337',
               'nt-5458', 'nt-5969', 'nt-8777', 'nt-1813', 'nt-2555', 'nt-5700', 'nt-7057', 'nt-7684', 'nt-8606',
               'nt-12263', 'nt-680', 'nt-892', 'nt-1264', 'nt-2212', 'nt-4467', 'nt-6086', 'nt-6327', 'nt-8867',
               'nt-9465', 'nt-10708', 'nt-12802', 'nt-642', 'nt-3892', 'nt-6231', 'nt-8346', 'nt-8471', 'nt-12913',
               'nt-1070', 'nt-2025', 'nt-3006', 'nt-3252', 'nt-5385', 'nt-5457', 'nt-8589', 'nt-10276', 'nt-13157',
               'nt-2293', 'nt-4909', 'nt-5360', 'nt-6003', 'nt-7089', 'nt-13636', 'nt-4438', 'nt-1499', 'nt-13377',
               'nt-309', 'nt-860', 'nt-5274', 'nt-5309', 'nt-6052', 'nt-9020', 'nt-10015', 'nt-10466', 'nt-10712',
               'nt-11484', 'nt-12400', 'nt-2785', 'nt-665', 'nt-1448', 'nt-1882', 'nt-2995', 'nt-3070', 'nt-7256',
               'nt-11016', 'nt-11536', 'nt-12799', 'nt-858', 'nt-3828', 'nt-5176', 'nt-5627', 'nt-6851', 'nt-6985',
               'nt-7869', 'nt-8386', 'nt-9274', 'nt-10043', 'nt-13933', 'nt-3988', 'nt-4177', 'nt-5120', 'nt-9337',
               'nt-13048', 'nt-13652', 'nt-2638', 'nt-4533', 'nt-4710', 'nt-5014', 'nt-7288', 'nt-7783', 'nt-10582',
               'nt-10688', 'nt-1054', 'nt-1481', 'nt-1680', 'nt-2566', 'nt-3353', 'nt-5335', 'nt-8425', 'nt-10269',
               'nt-11593', 'nt-13802', 'nt-5788', 'nt-9453', 'nt-9917', 'nt-12906', 'nt-13058', 'nt-13974', 'nt-14115',
               'nt-1238', 'nt-2883', 'nt-6261', 'nt-7639', 'nt-8734', 'nt-9265', 'nt-9864', 'nt-13547', 'nt-13921',
               'nt-12007', 'nt-12425', 'nt-10824', 'nt-1653', 'nt-2245', 'nt-2596', 'nt-4053', 'nt-5207', 'nt-6112',
               'nt-6116', 'nt-9512', 'nt-9837', 'nt-13062', 'nt-13559', 'nt-716', 'nt-2070', 'nt-2450', 'nt-5651',
               'nt-8677', 'nt-8889', 'nt-9775', 'nt-10671', 'nt-10952', 'nt-12094', 'nt-883', 'nt-1361', 'nt-2796',
               'nt-4400', 'nt-6200', 'nt-6314', 'nt-6788', 'nt-8851', 'nt-12838', 'nt-13438', 'nt-364', 'nt-3478',
               'nt-10947', 'nt-4432', 'nt-5610', 'nt-8693', 'nt-8829', 'nt-12118', 'nt-12238', 'nt-2222', 'nt-6588',
               'nt-7535', 'nt-8345', 'nt-8906', 'nt-11265', 'nt-12133', 'nt-4067', 'nt-5392', 'nt-6209', 'nt-6722',
               'nt-8307', 'nt-10126', 'nt-10921', 'nt-1372', 'nt-5500', 'nt-6569', 'nt-8696', 'nt-10488', 'nt-862',
               'nt-3581', 'nt-6140', 'nt-6546', 'nt-7079', 'nt-7255', 'nt-12569', 'nt-13263', 'nt-13339', 'nt-3281',
               'nt-3448', 'nt-5198', 'nt-7519', 'nt-12689', 'nt-876', 'nt-1212', 'nt-2062', 'nt-10467', 'nt-10732',
               'nt-10744', 'nt-12554', 'nt-375', 'nt-4791', 'nt-4868', 'nt-5215', 'nt-5840', 'nt-7909', 'nt-10210',
               'nt-11511', 'nt-13887', 'nt-88', 'nt-238', 'nt-4020', 'nt-5336', 'nt-6060', 'nt-9064', 'nt-11469',
               'nt-11772', 'nt-12756', 'nt-8741', 'nt-1089', 'nt-2152', 'nt-2756', 'nt-4322', 'nt-5053', 'nt-6571',
               'nt-10554', 'nt-11930', 'nt-1423', 'nt-4814', 'nt-10096', 'nt-13687', 'nt-455', 'nt-1127', 'nt-1985',
               'nt-8231', 'nt-8826', 'nt-9364', 'nt-20', 'nt-8749', 'nt-2488', 'nt-2497', 'nt-2664', 'nt-5041',
               'nt-9301', 'nt-9958', 'nt-10142', 'nt-11530', 'nt-14134', 'nt-538', 'nt-2578', 'nt-5350', 'nt-5409',
               'nt-5794', 'nt-8113', 'nt-421', 'nt-1564', 'nt-3501', 'nt-4657', 'nt-6119', 'nt-7183', 'nt-9924',
               'nt-380', 'nt-1082', 'nt-6482', 'nt-9589', 'nt-12356', 'nt-12558', 'nt-13638', 'nt-7950', 'nt-3306',
               'nt-4529', 'nt-5743', 'nt-7285', 'nt-7316', 'nt-8821', 'nt-10911', 'nt-11707', 'nt-12537', 'nt-13418',
               'nt-13624', 'nt-4378', 'nt-7155', 'nt-7565', 'nt-8105', 'nt-11803', 'nt-61', 'nt-1034', 'nt-1771',
               'nt-3179', 'nt-3198', 'nt-3835', 'nt-5801', 'nt-6484', 'nt-8403', 'nt-9497', 'nt-10341', 'nt-12547',
               'nt-280', 'nt-490', 'nt-1092', 'nt-1649', 'nt-8898', 'nt-9458', 'nt-10547', 'nt-446', 'nt-2757',
               'nt-2871', 'nt-4237', 'nt-7119', 'nt-9741', 'nt-13902', 'nt-695', 'nt-943', 'nt-1805', 'nt-2613',
               'nt-5488', 'nt-8076', 'nt-9718', 'nt-10023', 'nt-11420', 'nt-11872', 'nt-13684', 'nt-2323', 'nt-2371',
               'nt-4785', 'nt-5128', 'nt-5181', 'nt-5913', 'nt-11167', 'nt-11451', 'nt-13867', 'nt-1556', 'nt-1625',
               'nt-6526', 'nt-7613', 'nt-9944', 'nt-11870', 'nt-13303', 'nt-1547', 'nt-4452', 'nt-4998', 'nt-7781',
               'nt-10214', 'nt-10920', 'nt-13167', 'nt-12673', 'nt-2131', 'nt-2436', 'nt-3549', 'nt-7542', 'nt-9442',
               'nt-12891', 'nt-13023', 'nt-4119', 'nt-5832', 'nt-7244', 'nt-7500', 'nt-11386', 'nt-12594', 'nt-13758',
               'nt-12062', 'nt-427', 'nt-2408', 'nt-8151', 'nt-9552', 'nt-11058', 'nt-12589', 'nt-12698', 'nt-36',
               'nt-306', 'nt-396', 'nt-1491', 'nt-3287', 'nt-4775', 'nt-10922']


def majority_vote(answers, vote_with_weight=True):
    """
    Determine the final nsql execution answer by majority vote.
    """

    def _compare_answer_vote_with_count(a, b):
        """
        compare count sum.
        """
        return 1 if a[1]['count'] > b[1]['count'] else -1

    def _compare_answer_vote_with_weight(a, b):
        """
        compare prob sum.
        """
        return 1 if a[1]['prob'] > b[1]['prob'] else -1

    candi_answer_dict = {}

    for answer_str, _, logprob in answers:
        answer = tuple(str(answer_str).split(", "))
        if answer not in candi_answer_dict.keys():
            candi_answer_dict[answer] = {}
            candi_answer_dict[answer]['count'] = 0
            candi_answer_dict[answer]['prob'] = 0
        candi_answer_dict[answer]['count'] += 1
        candi_answer_dict[answer]['prob'] += math.exp(logprob)

    if vote_with_weight:
        sorted_candi_answer_list = sorted(list(candi_answer_dict.items()),
                                          key=cmp_to_key(_compare_answer_vote_with_count),
                                          reverse=True)
    else:
        sorted_candi_answer_list = sorted(list(candi_answer_dict.items()),
                                          key=cmp_to_key(_compare_answer_vote_with_weight),
                                          reverse=True)

    pred_answer_info = sorted_candi_answer_list[0]
    pred_answer = list(pred_answer_info[0])
    return pred_answer


if __name__ == '__main__':
    with open("../results_wikitq_full/test_unfiltered_wikitq_qa_balanced.json", "r") as f:
        all_data = json.load(f)

    right = 0
    solvable_right = 0
    solvable_all = 0
    missing_right = 0
    missing_all = 0

    unsolve_not_missing_right = 0
    unsolve_not_missing_all = 0

    for eid, data_item in tqdm(all_data.items()):
        try:
            pred = majority_vote(data_item['generations']['default_template'])
        except:
            pred = []
        gold = data_item['ori_data_item']['answer_text']
        question = data_item['ori_data_item']['question']
        nt_id = data_item['ori_data_item']['id']

        if nt_id in solvable_val:
            solvable_all += 1
        elif nt_id in missing_ids:
            missing_all += 1
        else:
            unsolve_not_missing_all += 1

        if eval_ex_match(pred, gold, 4, question):
            right += 1
            if nt_id in solvable_val:
                solvable_right += 1
            elif nt_id in missing_ids:
                missing_right += 1
            else:
                unsolve_not_missing_right += 1

    print("Acc: {}/{} ".format(right, len(all_data)), right / len(all_data))
    print("Solvable Acc: {}/{} ".format(solvable_right, solvable_all), solvable_right / solvable_all)
    print("Unsolvable not in Missing Acc: {}/{} ".format(unsolve_not_missing_right, unsolve_not_missing_all), unsolve_not_missing_right / unsolve_not_missing_all)
    print("Missing Acc: {}/{} ".format(missing_right, missing_all), missing_right / missing_all)
